@@ -86,7 +86,6 @@ lt_dlhandle lt_dlopenext       (const char *filename);
 # endif
 #endif
 
-using namespace amf;
 using namespace gnash;
 using namespace std;
 using namespace cygnal;
@@ -125,11 +124,11 @@ extern "C" {
         return init;
     }
 
-    boost::shared_ptr<amf::Buffer> oflaDemo_read_func()
+    boost::shared_ptr<cygnal::Buffer> oflaDemo_read_func()
     {
 // 	GNASH_REPORT_FUNCTION;
 	
-	boost::shared_ptr<amf::Buffer> buf = oflaDemo.getResponse();
+	boost::shared_ptr<cygnal::Buffer> buf = oflaDemo.getResponse();
 // 	log_network("%s", hexify(data, safe, true));
 
         return buf;
@@ -141,23 +140,23 @@ extern "C" {
     {
 // 	GNASH_REPORT_FUNCTION;
 
-	boost::shared_ptr<amf::Buffer> buf = oflaDemo.getResponse();
+	boost::shared_ptr<cygnal::Buffer> buf = oflaDemo.getResponse();
 
-        vector<boost::shared_ptr<amf::Element> > request =
+        vector<boost::shared_ptr<cygnal::Element> > request =
 	    oflaDemo.parseOflaDemoRequest(data, size);
         
         if (request.size() == 0) {
             // Send the packet to notify the client that the
             // NetConnection::connect() was sucessful. After the client
             // receives this, the handhsake is completed.
-            boost::shared_ptr<amf::Buffer> error =
+            boost::shared_ptr<cygnal::Buffer> error =
                 oflaDemo.encodeResult(RTMPMsg::NC_CALL_FAILED);
             // This builds the full header,which is required as the first part
             // of the packet.
-            boost::shared_ptr<amf::Buffer> head = oflaDemo.encodeHeader(0x3,
+            boost::shared_ptr<cygnal::Buffer> head = oflaDemo.encodeHeader(0x3,
                                           RTMP::HEADER_12, error->allocated(),
                                           RTMP::INVOKE, RTMPMsg::FROM_SERVER);
-            boost::scoped_ptr<amf::Buffer> response(new amf::Buffer(
+            boost::scoped_ptr<cygnal::Buffer> response(new cygnal::Buffer(
                                    error->allocated() + head->allocated()));
             *response = head;
             *response += error;
@@ -251,7 +250,7 @@ main(int argc, char *argv[])
     // This is the main message processing loop for rtmp. All message received require
     // a response.
     do {
-        boost::shared_ptr<amf::Buffer> bufptr(new amf::Buffer);
+        boost::shared_ptr<cygnal::Buffer> bufptr(new cygnal::Buffer);
         if (infile.empty()) {
             net.readNet(netfd, *bufptr);
         } else {
@@ -263,10 +262,10 @@ main(int argc, char *argv[])
             }
         }
         
-        vector<boost::shared_ptr<amf::Element> > request = net.parseOflaDemoRequest(
+        vector<boost::shared_ptr<cygnal::Element> > request = net.parseOflaDemoRequest(
             bufptr->reference(), bufptr->allocated());
         if (request[3]) {
-            boost::shared_ptr<amf::Buffer> result = net.formatOflaDemoResponse(request[1]->to_number(), *request[3]);
+            boost::shared_ptr<cygnal::Buffer> result = net.formatOflaDemoResponse(request[1]->to_number(), *request[3]);
             if (net.writeNet(netfd, *result)) {
                 log_debug("Sent oflaDemo test response response to client.");
             }
@@ -386,16 +385,16 @@ OflaDemoTest::~OflaDemoTest()
 
 // Parse an OflaDemo Request message coming from the Red5 oflaDemo_test. This
 // method should only be used for testing purposes.
-vector<boost::shared_ptr<amf::Element > >
+vector<boost::shared_ptr<cygnal::Element > >
 OflaDemoTest::parseOflaDemoRequest(boost::uint8_t *ptr, size_t size)
 {
     GNASH_REPORT_FUNCTION;
 
     demoService demo;
-    AMF amf;
-    vector<boost::shared_ptr<amf::Element > > headers;
+    cygnal::AMF amf;
+    vector<boost::shared_ptr<cygnal::Element > > headers;
 
-    boost::shared_ptr<amf::Element> el1 = amf.extractAMF(ptr, ptr+size);
+    boost::shared_ptr<cygnal::Element> el1 = amf.extractAMF(ptr, ptr+size);
     if (!el1) {
         log_error("No AMF data in message!");
         return headers;
@@ -406,7 +405,7 @@ OflaDemoTest::parseOflaDemoRequest(boost::uint8_t *ptr, size_t size)
     headers.push_back(el1);
 
     // The second element is a number
-    boost::shared_ptr<amf::Element> el2 = amf.extractAMF(ptr, ptr+size);
+    boost::shared_ptr<cygnal::Element> el2 = amf.extractAMF(ptr, ptr+size);
     if (!el2) {
         log_error("No AMF data in message!");
         return headers;
@@ -417,9 +416,9 @@ OflaDemoTest::parseOflaDemoRequest(boost::uint8_t *ptr, size_t size)
     if (method == "demoService.getListOfAvailableFLVs") {
         // Get the path from the NetConnection object we recieved from the
         // client at the end of the handshake process.
-        boost::shared_ptr<amf::Element> version;
-        boost::shared_ptr<amf::Element> tcurl;
-        boost::shared_ptr<amf::Element> swfurl;
+        boost::shared_ptr<cygnal::Element> version;
+        boost::shared_ptr<cygnal::Element> tcurl;
+        boost::shared_ptr<cygnal::Element> swfurl;
         
         boost::shared_ptr<gnash::RTMPMsg> msg = getNetConnection();
         if (msg) {
@@ -455,24 +454,24 @@ OflaDemoTest::parseOflaDemoRequest(boost::uint8_t *ptr, size_t size)
             toparr.makeECMAArray();
             
             size_t total_size = 0;
-            vector<boost::shared_ptr<amf::Buffer> > buffers;
+            vector<boost::shared_ptr<cygnal::Buffer> > buffers;
             for (it=mediafiles.begin(); it<mediafiles.end(); ++it) {
-                vector<boost::shared_ptr<amf::Element> > data;
+                vector<boost::shared_ptr<cygnal::Element> > data;
                 
                 boost::shared_ptr<demoService::filestats_t> file = *it;
-                boost::shared_ptr<amf::Element> obj(new amf::Element);
+                boost::shared_ptr<cygnal::Element> obj(new cygnal::Element);
                 obj->makeECMAArray();
                 obj->setName(file->name);
                 
-                boost::shared_ptr<amf::Element> modified(new amf::Element);
+                boost::shared_ptr<cygnal::Element> modified(new cygnal::Element);
                 modified->makeString("lastModified", file->last);
                 obj->addProperty(modified);
 
-                boost::shared_ptr<amf::Element> name(new amf::Element);
+                boost::shared_ptr<cygnal::Element> name(new cygnal::Element);
                 name->makeString("name", file->name);
                 obj->addProperty(name);
 
-                boost::shared_ptr<amf::Element> size(new amf::Element);
+                boost::shared_ptr<cygnal::Element> size(new cygnal::Element);
                 size->makeString("size", file->size);
                 obj->addProperty(size);
 
@@ -480,31 +479,31 @@ OflaDemoTest::parseOflaDemoRequest(boost::uint8_t *ptr, size_t size)
                 toparr.addProperty(obj);
             }
             
-            boost::shared_ptr<amf::Buffer> topenc = toparr.encode();
+            boost::shared_ptr<cygnal::Buffer> topenc = toparr.encode();
             total_size += topenc->allocated();
             
             // Start with the method name for the INVOKE
-            amf::Element method;
+            cygnal::Element method;
             method.makeString("_result");
-            boost::shared_ptr<amf::Buffer> methodenc  = method.encode();
+            boost::shared_ptr<cygnal::Buffer> methodenc  = method.encode();
             total_size += methodenc->allocated();
             
             // Add the stream ID
-            amf::Element sid;
+            cygnal::Element sid;
             sid.makeNumber(2); // FIXME: needs a real value!
-            boost::shared_ptr<amf::Buffer> sidenc  = sid.encode();
+            boost::shared_ptr<cygnal::Buffer> sidenc  = sid.encode();
             total_size += sidenc->allocated();
 
             // There there is always a NULL object to start the data
             Element null;
             null.makeNull();
-            boost::shared_ptr<amf::Buffer> encnull  = null.encode();
+            boost::shared_ptr<cygnal::Buffer> encnull  = null.encode();
             total_size += encnull->allocated();
 
-            boost::shared_ptr<amf::Buffer> result(new amf::Buffer(total_size+amf::AMF_HEADER_SIZE+RTMP_MAX_HEADER_SIZE+10));            
-            _response.reset(new amf::Buffer(total_size+amf::AMF_HEADER_SIZE+RTMP_MAX_HEADER_SIZE+10));
+            boost::shared_ptr<cygnal::Buffer> result(new cygnal::Buffer(total_size+cygnal::AMF_HEADER_SIZE+RTMP_MAX_HEADER_SIZE+10));            
+            _response.reset(new cygnal::Buffer(total_size+cygnal::AMF_HEADER_SIZE+RTMP_MAX_HEADER_SIZE+10));
 #if 0
-            boost::shared_ptr<amf::Buffer> head = encodeHeader(0x3,
+            boost::shared_ptr<cygnal::Buffer> head = encodeHeader(0x3,
 			    RTMP::HEADER_8, total_size,
 			    RTMP::INVOKE, RTMPMsg::FROM_SERVER);
             *result = head;
@@ -516,11 +515,11 @@ OflaDemoTest::parseOflaDemoRequest(boost::uint8_t *ptr, size_t size)
 
 #if 0
             // Followed by all the encoded objects and properties
-            vector<boost::shared_ptr<amf::Buffer> >::iterator rit;
+            vector<boost::shared_ptr<cygnal::Buffer> >::iterator rit;
             for (rit=buffers.begin(); rit<buffers.end(); ++rit) {
-                boost::shared_ptr<amf::Buffer> buf = *rit;
+                boost::shared_ptr<cygnal::Buffer> buf = *rit;
                 *_response += buf;
-                std::vector<boost::shared_ptr<amf::Element> > data1;
+                std::vector<boost::shared_ptr<cygnal::Element> > data1;
             }
 #endif
         }
@@ -535,11 +534,11 @@ OflaDemoTest::parseOflaDemoRequest(boost::uint8_t *ptr, size_t size)
 // is only used for testing by developers. The format appears to be
 // a string '_result', followed by the number of the test, and then two
 // NULL objects.
-boost::shared_ptr<amf::Buffer>
-OflaDemoTest::formatOflaDemoResponse(double num, amf::Element &el)
+boost::shared_ptr<cygnal::Buffer>
+OflaDemoTest::formatOflaDemoResponse(double num, cygnal::Element &el)
 {
 //    GNASH_REPORT_FUNCTION;
-    boost::shared_ptr<amf::Buffer> data = amf::AMF::encodeElement(el);
+    boost::shared_ptr<cygnal::Buffer> data = cygnal::AMF::encodeElement(el);
     if (data) {
 	return formatOflaDemoResponse(num, data->reference(), data->allocated());
     } else {
@@ -550,14 +549,14 @@ OflaDemoTest::formatOflaDemoResponse(double num, amf::Element &el)
     return data;
 }
 
-boost::shared_ptr<amf::Buffer>
-OflaDemoTest::formatOflaDemoResponse(double num, amf::Buffer &data)
+boost::shared_ptr<cygnal::Buffer>
+OflaDemoTest::formatOflaDemoResponse(double num, cygnal::Buffer &data)
 {
 //    GNASH_REPORT_FUNCTION;
     return formatOflaDemoResponse(num, data.reference(), data.allocated());
 }
 
-boost::shared_ptr<amf::Buffer>
+boost::shared_ptr<cygnal::Buffer>
 OflaDemoTest::formatOflaDemoResponse(double num, boost::uint8_t *data, size_t size)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -572,11 +571,11 @@ OflaDemoTest::formatOflaDemoResponse(double num, boost::uint8_t *data, size_t si
     Element null;
     null.makeNull();
 
-    boost::shared_ptr<amf::Buffer> encoflaDemo = oflaDemo.encode();
-    boost::shared_ptr<amf::Buffer> encidx  = index.encode();   
-    boost::shared_ptr<amf::Buffer> encnull  = null.encode();   
+    boost::shared_ptr<cygnal::Buffer> encoflaDemo = oflaDemo.encode();
+    boost::shared_ptr<cygnal::Buffer> encidx  = index.encode();   
+    boost::shared_ptr<cygnal::Buffer> encnull  = null.encode();   
 
-    boost::shared_ptr<amf::Buffer> buf(new amf::Buffer(encoflaDemo->size()
+    boost::shared_ptr<cygnal::Buffer> buf(new cygnal::Buffer(encoflaDemo->size()
 						       + encidx->size()
 						       + encnull->size() + size));
 
