@@ -19,6 +19,7 @@
 #ifndef _RTMP_SERVER_H_
 #define _RTMP_SERVER_H_
 
+#include <vector>
 #include <boost/cstdint.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/array.hpp>
@@ -55,37 +56,37 @@ public:
     ///     INVOKE operation of ::connect(). serverFinish() is
     ///     actually used to extract the AMF data from the packet, and
     ///     handShakeResponse() is used to construct the response packet.
-    boost::shared_ptr<amf::Element> processClientHandShake(int fd);
+    boost::shared_ptr<cygnal::Element> processClientHandShake(int fd);
 
-    bool packetSend(amf::Buffer &buf);
-    bool packetRead(amf::Buffer &buf);
+    bool packetSend(cygnal::Buffer &buf);
+    bool packetRead(cygnal::Buffer &buf);
     
     // These are handlers for the various types
-    boost::shared_ptr<amf::Buffer> encodeResult(gnash::RTMPMsg::rtmp_status_e status);
-    boost::shared_ptr<amf::Buffer> encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string &filename);
-    boost::shared_ptr<amf::Buffer> encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string &filename, double &transid);
-    boost::shared_ptr<amf::Buffer> encodeResult(gnash::RTMPMsg::rtmp_status_e status, double &transid);
-    boost::shared_ptr<amf::Buffer> encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string &filename, double &transid, double &clientid);
+    boost::shared_ptr<cygnal::Buffer> encodeResult(gnash::RTMPMsg::rtmp_status_e status);
+    boost::shared_ptr<cygnal::Buffer> encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string &filename);
+    boost::shared_ptr<cygnal::Buffer> encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string &filename, double &transid);
+    boost::shared_ptr<cygnal::Buffer> encodeResult(gnash::RTMPMsg::rtmp_status_e status, double &transid);
+    boost::shared_ptr<cygnal::Buffer> encodeResult(gnash::RTMPMsg::rtmp_status_e status, const std::string &filename, double &transid, double &clientid);
 
     // Encode a Ping for the client
-    boost::shared_ptr<amf::Buffer> encodePing(rtmp_ping_e type, boost::uint32_t milliseconds);
-    boost::shared_ptr<amf::Buffer> encodePing(rtmp_ping_e type);
-    // boost::shared_ptr<amf::Buffer> encodeUser(user_control_e type, boost::uint32_t milliseconds);
-    boost::shared_ptr<amf::Buffer> encodeAudio(boost::uint8_t *data, size_t size);
-    boost::shared_ptr<amf::Buffer> encodeVideo(boost::uint8_t *data, size_t size);
+    boost::shared_ptr<cygnal::Buffer> encodePing(rtmp_ping_e type, boost::uint32_t milliseconds);
+    boost::shared_ptr<cygnal::Buffer> encodePing(rtmp_ping_e type);
+    // boost::shared_ptr<cygnal::Buffer> encodeUser(user_control_e type, boost::uint32_t milliseconds);
+    boost::shared_ptr<cygnal::Buffer> encodeAudio(boost::uint8_t *data, size_t size);
+    boost::shared_ptr<cygnal::Buffer> encodeVideo(boost::uint8_t *data, size_t size);
 
     // Encode a onBWDone message for the client
-    boost::shared_ptr<amf::Buffer> encodeBWDone(double id);
+    boost::shared_ptr<cygnal::Buffer> encodeBWDone(double id);
 
     // Parse an Echo Request message coming from the Red5 echo_test.
-    std::vector<boost::shared_ptr<amf::Element > > parseEchoRequest(amf::Buffer &buf) { return parseEchoRequest(buf.reference(), buf.size()); };
-    std::vector<boost::shared_ptr<amf::Element > > parseEchoRequest(boost::uint8_t *buf, size_t size);
+    std::vector<boost::shared_ptr<cygnal::Element > > parseEchoRequest(cygnal::Buffer &buf) { return parseEchoRequest(buf.reference(), buf.size()); };
+    std::vector<boost::shared_ptr<cygnal::Element > > parseEchoRequest(boost::uint8_t *buf, size_t size);
     // format a response to the 'echo' test used for testing Gnash.
-    boost::shared_ptr<amf::Buffer> formatEchoResponse(double num, amf::Element &el);
-    boost::shared_ptr<amf::Buffer> formatEchoResponse(double num, amf::Buffer &data);
-    boost::shared_ptr<amf::Buffer> formatEchoResponse(double num, boost::uint8_t *data, size_t size);    
-    void addReference(boost::uint16_t index, amf::Element &el) { _references[index] = el; };
-    amf::Element &getReference(boost::uint16_t index) { return _references[index]; };
+    boost::shared_ptr<cygnal::Buffer> formatEchoResponse(double num, cygnal::Element &el);
+    boost::shared_ptr<cygnal::Buffer> formatEchoResponse(double num, cygnal::Buffer &data);
+    boost::shared_ptr<cygnal::Buffer> formatEchoResponse(double num, boost::uint8_t *data, size_t size);    
+    void addReference(boost::uint16_t index, cygnal::Element &el) { _references[index] = el; };
+    cygnal::Element &getReference(boost::uint16_t index) { return _references[index]; };
 
     bool sendFile(int fd, const std::string &filespec);
 
@@ -104,7 +105,7 @@ public:
 
     size_t sendToClient(std::vector<int> &fds, boost::uint8_t *data,
 			size_t size);
-    size_t sendToClient(std::vector<int> &fds,amf::Buffer &data);
+    size_t sendToClient(std::vector<int> &fds,cygnal::Buffer &data);
 
     void setNetConnection(gnash::RTMPMsg *msg) { _netconnect.reset(msg); };
     void setNetConnection(boost::shared_ptr<gnash::RTMPMsg> msg) { _netconnect = msg; };
@@ -116,12 +117,12 @@ private:
     ///     This is only called by processClientHandshake() to compare
     ///     the handshakes to make sure they match, and to extract the
     ///     initial AMF data from packet.
-    boost::shared_ptr<amf::Buffer> serverFinish(int fd,
-			amf::Buffer &handshake1, amf::Buffer &handshake2);
+    boost::shared_ptr<cygnal::Buffer> serverFinish(int fd,
+			cygnal::Buffer &handshake1, cygnal::Buffer &handshake2);
     /// \method handShakeResponse
     ///     This is only called by processClientHandshake() to
     ///     construct the handshake response to the client.
-    bool handShakeResponse(int fd, amf::Buffer &buf);
+    bool handShakeResponse(int fd, cygnal::Buffer &buf);
     
     /// This is used by the boost tokenizer functions, and is defined
     /// here purely for convienience.
@@ -133,7 +134,7 @@ private:
     std::string		_docroot;
     std::string		_filespec;
     boost::uint32_t     _filesize;
-    std::map<boost::uint16_t, amf::Element> _references;
+    std::map<boost::uint16_t, cygnal::Element> _references;
 #ifdef CLIENT_ID_NUMERIC
     std::array<double>	_clientids;
 #else
