@@ -26,10 +26,9 @@
 
 namespace gnash {
 
-class ObjectURI;
+struct ObjectURI;
 class as_object;
 
-
 extern "C" {
 
 void fileio_class_init(as_object& global, const ObjectURI& uri);  
@@ -43,5 +42,5 @@ void fileio_class_init(as_object& global, const ObjectURI& uri);
 
 // Local Variables:
 // mode: C++
-// indent-tabs-mode: t
+// indent-tabs-mode: nil
 // End:
