@@ -47,12 +47,7 @@
 #include "noseek_fd_adapter.h"
 #include "VM.h"
 #include "SystemClock.h"
-
-#ifdef USE_FFMPEG
-# include "MediaHandlerFfmpeg.h"
-#elif defined(USE_GST)
-# include "MediaHandlerGst.h"
-#endif
+#include "ExternalInterface.h"
 
 #include "GnashSystemIOHeaders.h" // for write() 
 #include "log.h"
@@ -64,12 +59,9 @@
 using namespace gnash;
 
 namespace {
-gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
+    gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 }
 
-/// @todo Shouldn't Player be in 'gnash' namespace ?
-
-/*static private*/
 void
 Player::setFlashVars(const std::string& varstr)
 {
@@ -81,6 +73,12 @@ Player::setFlashVars(const std::string& varstr)
     _gui->addFlashVars(vars);
 }
 
+void
+Player::setScriptableVar(const std::string &name, const std::string &value)
+{
+    _gui->addScriptableVar(name, value);
+}
+
 Player::Player()
     :
 #if defined(RENDERER_CAIRO)
@@ -105,6 +103,7 @@ Player::Player()
     _fpsDebugTime(0.0),
 #endif
     _hostfd(-1),
+    _controlfd(-1),
     _startFullscreen(false),
     _hideMenu(false)
 {
@@ -180,22 +179,20 @@ Player::init_sound()
 {
 
     if (_doSound) {
+        try {
 #ifdef SOUND_SDL
-        try {
-            _soundHandler.reset(sound::create_sound_handler_sdl(_audioDump));
-        } catch (SoundException& ex) {
-            log_error(_("Could not create sound handler: %s."
-                " Will continue w/out sound."), ex.what());
-        }
-        if (! _audioDump.empty()) {
-            // add a silent stream to the audio pool so that our output file
-            // is homogenous;  we actually want silent wave data when no sounds
-            // are playing on the stage
-            _soundHandler->attach_aux_streamer(silentStream, (void*) this);
-        }
+            _soundHandler.reset(sound::create_sound_handler_sdl(
+                        _mediaHandler.get(), _audioDump));
 #elif defined(SOUND_AHI)
-        try {
-            _soundHandler.reset(sound::create_sound_handler_aos4(_audioDump));
+            _soundHandler.reset(sound::create_sound_handler_aos4(
+                        _mediaHandler.get(), _audioDump));
+#elif defined(SOUND_MKIT)
+            _soundHandler.reset(sound::create_sound_handler_mkit(
+                        _mediaHandler.get(), _audioDump));
+#else
+            log_error(_("Sound requested but no sound support compiled in"));
+            return;
+#endif
         } catch (SoundException& ex) {
             log_error(_("Could not create sound handler: %s."
                 " Will continue w/out sound."), ex.what());
@@ -206,36 +203,17 @@ Player::init_sound()
             // are playing on the stage
             _soundHandler->attach_aux_streamer(silentStream, (void*) this);
         }
-#elif defined(SOUND_GST)
-        _soundHandler.reset(media::create_sound_handler_gst());
-#else
-        log_error(_("Sound requested but no sound support compiled in"));
-        return;
-#endif
     }
 }
 
-void
-Player::init_media()
-{
-#ifdef USE_FFMPEG
-        _mediaHandler.reset( new gnash::media::ffmpeg::MediaHandlerFfmpeg() );
-#elif defined(USE_GST)
-        _mediaHandler.reset( new gnash::media::gst::MediaHandlerGst() );
-#else
-        log_error(_("No media support compiled in"));
-        return;
-#endif
-        
-        gnash::media::MediaHandler::set(_mediaHandler);
-}
-
-
 void
 Player::init_gui()
 {
-    if (_doRender) _gui = getGui(); 
-    else _gui.reset(new NullGui(_doLoop, *_runResources));
+    if (_doRender) {
+        _gui = getGui();
+    } else {
+        _gui.reset(new NullGui(_doLoop, *_runResources));
+    }
 
     _gui->setMaxAdvances(_maxAdvances);
 
@@ -269,17 +247,13 @@ Player::load_movie()
     }
 
     try {
-        if ( _infile == "-" )
-        {
+        if ( _infile == "-" ) {
             std::auto_ptr<IOChannel> in (
-                    noseek_fd_adapter::make_stream(fileno(stdin)));
+                noseek_fd_adapter::make_stream(fileno(stdin)));
             md = MovieFactory::makeMovie(in, _url, *_runResources, false);
-        }
-        else
-        {
+        } else {
             URL url(_infile);
-            if ( url.protocol() == "file" )
-            {
+            if ( url.protocol() == "file" ) {
                 std::string path = url.path();
                 // We'll need to allow load of the file, no matter virtual url
                 // specified...
@@ -301,8 +275,7 @@ Player::load_movie()
         md = NULL;
     }
 
-    if ( ! md )
-    {
+    if ( ! md ) {
         fprintf(stderr, "Could not load movie '%s'\n", _infile.c_str());
         return NULL;
     }
@@ -310,18 +283,15 @@ Player::load_movie()
     return md;
 }
 
-/* \brief Run, used to open a new flash file. Using previous initialization */
-int
+/// \brief Run, used to open a new flash file. Using previous initialization
+void
 Player::run(int argc, char* argv[], const std::string& infile,
         const std::string& url)
 {
-    
     // Call this at run() time, so the caller has
     // a cache of setting some parameter before calling us...
     // (example: setDoSound(), setWindowId() etc.. ) 
     init_logfile();
-    init_media();
-    init_sound();
    
     // gnash.cpp should check that a filename is supplied.
     assert (!infile.empty());
@@ -336,18 +306,19 @@ Player::run(int argc, char* argv[], const std::string& infile,
     }
 
     // Set _root._url (either explicit of from infile)
-    if (!url.empty()) _url = url;
-    else _url = infile;
+    if (!url.empty()) {
+        _url = url;
+    } else {
+        _url = infile;
+    }
 
     // Parse player parameters. These are not passed to the SWF, but rather
     // control stage properties etc.
-    Params::const_iterator it = params.find("base");
-    const URL baseURL = (it == params.end()) ? _baseurl :
+    Params::const_iterator it = _params.find("base");
+    const URL baseURL = (it == _params.end()) ? _baseurl :
                                                URL(it->second, _baseurl);
-
     /// The RunResources should be populated before parsing.
     _runResources.reset(new RunResources(baseURL.str()));
-    _runResources->setSoundHandler(_soundHandler);
 
     boost::shared_ptr<SWF::TagLoadersTable> loaders(new SWF::TagLoadersTable());
     addDefaultLoaders(*loaders);
@@ -357,6 +328,25 @@ Player::run(int argc, char* argv[], const std::string& infile,
     boost::shared_ptr<StreamProvider> sp(new StreamProvider(np));
 
     _runResources->setStreamProvider(sp);
+
+    // Set the Hardware video decoding resources. none, vaapi, xv, omap
+    _runResources->setHWAccelBackend(_hwaccel);
+    // Set the Renderer resource, opengl, agg, or cairo
+    _runResources->setRenderBackend(_renderer);
+
+    _mediaHandler.reset(media::MediaFactory::instance().get(_media));
+
+    if (!_mediaHandler.get()) {
+        boost::format fmt =
+            boost::format(_("Non-existent media handler %1% specified"))
+            % _media;
+        throw GnashException(fmt.str());
+    }
+
+    _runResources->setMediaHandler(_mediaHandler);
+    
+    init_sound();
+    _runResources->setSoundHandler(_soundHandler);
     
     init_gui();
 
@@ -364,22 +354,36 @@ Player::run(int argc, char* argv[], const std::string& infile,
     // note that this will also initialize the renderer
     // which is *required* during movie loading
     if (!_gui->init(argc, &argv)) {
-        std::cerr << "Could not initialize gui." << std::endl;
-        return EXIT_FAILURE;
+        throw GnashException("Could not initialize GUI");
     }
 
     // Parse querystring (before FlashVars, see
     // testsuite/misc-ming.all/FlashVarsTest*)
     setFlashVars(URL(_url).querystring());
-    
-    // Add FlashVars.
-    Params::const_iterator fv = params.find("flashvars");
-    if (fv != params.end()) setFlashVars(fv->second);
 
+    // Add FlashVars.
+    Params::const_iterator fv = _params.find("flashvars");
+    if (fv != _params.end()) {
+        setFlashVars(fv->second);
+    }
+
+#if 0
+    // Add Scriptable Variables. These values become the default, but
+    // they can be reset from JavaScript via ExternalInterface. These
+    // are passed to Gnash using the '-P' option, and have nothing to
+    // to do with 'flashVars'.
+    fv = _params.begin();
+    for (fv=_params.begin(); fv != _params.end(); fv++) {
+        if (fv->first != "flashvars") {
+            setScriptableVar(fv->first, fv->second);
+        }
+    }
+#endif
+    
     // Load the actual movie.
     _movieDef = load_movie();
     if (!_movieDef) {
-        return EXIT_FAILURE;
+        throw GnashException("Could not load movie!");
     }
 
     // Get info about the width & height of the movie.
@@ -418,8 +422,18 @@ Player::run(int argc, char* argv[], const std::string& infile,
     // Register Player to receive FsCommand events from the core.
     root.registerFSCommandCallback(_callbacksHandler.get());
 
+    log_debug("Player Host FD #%d, Player Control FD #%d", 
+                     _hostfd, _controlfd);
+    
     // Set host requests fd (if any)
-    if ( _hostfd != -1 ) root.setHostFD(_hostfd);
+    if ( _hostfd != -1 ) {
+        root.setHostFD(_hostfd);
+    }
+    
+    if (_controlfd != -1) {
+        root.setControlFD(_controlfd);        
+//        _gui->setFDCallback(_controlfd, boost::bind(&Gui::quit, boost::ref(_gui)));
+    }
 
     _gui->setStage(&root);
     
@@ -431,7 +445,9 @@ Player::run(int argc, char* argv[], const std::string& infile,
     // because the gui window should be properly set up by this point.
     RcInitFile& rcfile = RcInitFile::getDefaultInstance();
 
-    if (rcfile.startStopped()) _gui->stop();
+    if (rcfile.startStopped()) {
+        _gui->stop();
+    }
 
     // Start loader thread
     // NOTE: the loader thread might (in IMPORT tag parsing)
@@ -464,35 +480,59 @@ Player::run(int argc, char* argv[], const std::string& infile,
     // Now handle stage alignment and scale mode. This should be done after
     // the GUI is created, after its stage member is set, and after the
     // interface callbacks are registered.
-    it = params.find("salign");
-    if (it != params.end()) {
+    it = _params.find("salign");
+    if (it != _params.end()) {
         log_debug("Setting align");
         const short align = stringToStageAlign(it->second);
         root.setStageAlignment(align);
     }
 
-    it = params.find("scale");
-    if (it != params.end()) {
-		
+    it = _params.find("allowscriptaccess");
+    if (it != _params.end()) {
+        std::string access = it->second;
         StringNoCaseEqual noCaseCompare;
         const std::string& str = it->second;
-		
-        movie_root::ScaleMode mode = movie_root::showAll;
+                
+        movie_root::AllowScriptAccessMode mode = 
+            movie_root::SCRIPT_ACCESS_SAME_DOMAIN;
+        
+        if (noCaseCompare(str, "never")) {
+            mode = movie_root::SCRIPT_ACCESS_NEVER;
+        } 
+        else if (noCaseCompare(str, "sameDomain")) {
+            mode = movie_root::SCRIPT_ACCESS_SAME_DOMAIN;
+        } 
+        else if (noCaseCompare(str, "always")) {
+            mode = movie_root::SCRIPT_ACCESS_ALWAYS;
+        }
+        log_debug("Setting allowscriptaccess to %s", mode);
+        root.setAllowScriptAccess(mode);
+    }
 
-		if (noCaseCompare(str, "noScale")) mode = movie_root::noScale;
-		else if (noCaseCompare(str, "exactFit")) mode = movie_root::exactFit;
-		else if (noCaseCompare(str, "noBorder")) mode = movie_root::noBorder;
+    it = _params.find("scale");
+    if (it != _params.end()) {                
+        StringNoCaseEqual noCaseCompare;
+        const std::string& str = it->second;
+        movie_root::ScaleMode mode = movie_root::SCALEMODE_SHOWALL;
+        
+        if (noCaseCompare(str, "noScale")) {
+            mode = movie_root::SCALEMODE_NOSCALE;
+        } 
+        else if (noCaseCompare(str, "exactFit")) {
+            mode = movie_root::SCALEMODE_EXACTFIT;
+        }
+        else if (noCaseCompare(str, "noBorder")) {
+            mode = movie_root::SCALEMODE_NOBORDER;
+        }
 
         log_debug("Setting scale mode");
-	    root.setStageScaleMode(mode);
+            root.setStageScaleMode(mode);
     }
 
     // Set up screenshots. 
     if (!_screenshots.empty()) {
-
         std::istringstream is(_screenshots);
         std::string arg;
-
         bool last = false;
         ScreenShotter::FrameList v;
 
@@ -502,7 +542,7 @@ Player::run(int argc, char* argv[], const std::string& infile,
                 const size_t frame = boost::lexical_cast<size_t>(arg);
                 v.push_back(frame);
             }
-            catch (boost::bad_lexical_cast&) {}
+            catch (const boost::bad_lexical_cast&) {}
         }
 
         // Use default if filename is empty.
@@ -516,7 +556,7 @@ Player::run(int argc, char* argv[], const std::string& infile,
 
         _gui->requestScreenShots(v, last, _screenshotFile);
     }
-    
+
     _gui->run();
 
     log_debug("Main loop ended, cleaning up");
@@ -524,9 +564,13 @@ Player::run(int argc, char* argv[], const std::string& infile,
     // Clean up as much as possible, so valgrind will help find actual leaks.
     gnash::clear();
 
-    return EXIT_SUCCESS;
 }
 
+void
+Player::CallbacksHandler::exit()
+{
+    _gui.quit();
+}
 
 void
 Player::CallbacksHandler::error(const std::string& msg)
@@ -544,38 +588,33 @@ std::string
 Player::CallbacksHandler::call(const std::string& event, const std::string& arg)
 {
     StringNoCaseEqual noCaseCompare;
-	
+        
     if (event == "Mouse.hide") {
         return _gui.showMouse(false) ? "true" : "false";
     }
 
-    if (event == "Mouse.show")
-    {
+    if (event == "Mouse.show") {
         return _gui.showMouse(true) ? "true" : "false";
     }
     
-    if (event == "Stage.displayState")
-    {
+    if (event == "Stage.displayState") {
         if (arg == "fullScreen") _gui.setFullscreen();
         else if (arg == "normal") _gui.unsetFullscreen();
         return "";
     }
 
-    if (event == "Stage.scaleMode" || event == "Stage.align" )
-    {
+    if (event == "Stage.scaleMode" || event == "Stage.align" ) {
         _gui.updateStageMatrix();
         return "";
     }
 
-    if (event == "Stage.showMenu")
-    {
+    if (event == "Stage.showMenu") {
         if (noCaseCompare(arg, "true")) _gui.showMenu(true);
         else if (noCaseCompare(arg, "false")) _gui.showMenu(false);
         return "";
     }
 
-    if (event == "Stage.resize")
-    {
+    if (event == "Stage.resize") {
         if ( _gui.isPlugin() ) {
             log_debug("Player doing nothing on Stage.resize as we're a plugin");
             return "";
@@ -590,23 +629,57 @@ Player::CallbacksHandler::call(const std::string& event, const std::string& arg)
         return "";
     }
 
+    if (event == "ExternalInterface.Play") {
+        _gui.play();
+        return "";
+    }
 
-    if (event == "System.capabilities.screenResolutionX")
-    {
+    if (event == "ExternalInterface.StopPlay") {
+        _gui.pause();
+        return "";
+    }
+
+    if (event == "ExternalInterface.Rewind") {
+        _gui.restart();
+        return "";
+    }
+
+    if (event == "ExternalInterface.Pan") {
+	// FIXME: the 3 args are encoded as 1:2:0
+	log_unimpl("ExternalInterface.Pan");
+        return "";
+    }
+
+    if (event == "ExternalInterface.IsPlaying") {
+	return (_gui.isStopped()) ? "false" : "true";
+    }
+
+    if (event == "ExternalInterface.SetZoomRect") {
+	// FIXME: the 4 arguments are encoded as 1:2:0:1
+	log_unimpl("ExternalInterface.SetZoomRect");
+        return "";
+    }
+
+    if (event == "ExternalInterface.Zoom") {
+	// The 1 argument is a percentage to zoom
+	int percent = strtol(arg.c_str(), NULL, 0);
+	log_unimpl("ExternalInterface.Zoom(%d)", percent);
+        return "";
+    }
+
+    if (event == "System.capabilities.screenResolutionX") {
         std::ostringstream ss;
         ss << _gui.getScreenResX();
         return ss.str();
     }
 
-    if (event == "System.capabilities.screenResolutionY")
-    {
+    if (event == "System.capabilities.screenResolutionY") {
         std::ostringstream ss;
         ss << _gui.getScreenResY();
         return ss.str();
     }
 
-    if (event == "System.capabilities.pixelAspectRatio")
-    {
+    if (event == "System.capabilities.pixelAspectRatio") {
         std::ostringstream ss;
         // Whether the pp actively limits the precision or simply
         // gets a slightly different result isn't clear.
@@ -614,20 +687,17 @@ Player::CallbacksHandler::call(const std::string& event, const std::string& arg)
         return ss.str();
     }
 
-    if (event == "System.capabilities.screenDPI")
-    {
+    if (event == "System.capabilities.screenDPI") {
         std::ostringstream ss;
         ss << _gui.getScreenDPI();
         return ss.str();
     }
 
-    if (event == "System.capabilities.screenColor")
-    {
+    if (event == "System.capabilities.screenColor") {
         return _gui.getScreenColor();
     }
 
-    if (event == "System.capabilities.playerType")
-    {
+    if (event == "System.capabilities.playerType") {
         return _gui.isPlugin() ? "PlugIn" : "StandAlone";
     }
 
@@ -645,11 +715,13 @@ Player::CallbacksHandler::notify(const std::string& command,
 
     // it's _hostfd, but we're a static method...
     const int hostfd = _player.getHostFD();
-    if (hostfd != -1)
-    {
+    if (hostfd != -1) {
         //log_debug("user-provided host requests fd is %d", hostfd);
         std::stringstream request;
-        request << "INVOKE " << command << ":" << args << std::endl;
+	std::vector<as_value> fnargs;
+	fnargs.push_back(as_value(command));
+	fnargs.push_back(as_value(args));
+	request << ExternalInterface::makeInvoke("fsCommand", fnargs);
 
         std::string requestString = request.str();
         const char* cmd = requestString.c_str();
@@ -658,13 +730,11 @@ Player::CallbacksHandler::notify(const std::string& command,
         // NOTE: we assuming the hostfd is set in blocking mode here..
         //log_debug("Attempt to write INVOKE requests fd %d", hostfd);
         int ret = write(hostfd, cmd, len);
-        if ( ret == -1 )
-        {
+        if ( ret == -1 ) {
             log_error("Could not write to user-provided host "
                       "requests fd %d: %s", hostfd, strerror(errno));
         }
-        if ( static_cast<size_t>(ret) < len )
-        {
+        if ( static_cast<size_t>(ret) < len ) {
             log_error("Could only write %d bytes over %d required to "
                       "user-provided host requests fd %d",
                       ret, len, hostfd);
@@ -678,8 +748,7 @@ Player::CallbacksHandler::notify(const std::string& command,
 
     /// Fscommands can be ignored using an rcfile setting. As a 
     /// plugin they are always ignored.
-    if (_gui.isPlugin())
-    {
+    if (_gui.isPlugin()) {
         // We log the request to the fd above
         log_debug(_("Running as plugin: skipping internal "
                     "handling of FsCommand %s%s."));
@@ -698,23 +767,20 @@ Player::CallbacksHandler::notify(const std::string& command,
     // quit, fullscreen, showmenu, exec, allowscale, and trapallkeys.
     
     // FSCommand quit
-    if (noCaseCompare(command, "quit"))
-    {
+    if (noCaseCompare(command, "quit")) {
         _gui.quit();
         return;
     }
 
     // FSCommand fullscreen
-    if (noCaseCompare(command, "fullscreen"))
-    {
+    if (noCaseCompare(command, "fullscreen")) {
         if (noCaseCompare(args, "true")) _gui.setFullscreen();
         else if (noCaseCompare(args, "false")) _gui.unsetFullscreen();
         return;
     }
        
     // FSCommand showmenu
-    if (noCaseCompare(command, "showmenu"))
-    {
+    if (noCaseCompare(command, "showmenu")) {
         if (noCaseCompare(args, "true")) _gui.showMenu(true);
         else if (noCaseCompare(args, "false")) _gui.showMenu(false);
         return;
@@ -724,28 +790,24 @@ Player::CallbacksHandler::notify(const std::string& command,
     // Note: the pp insists that the file to execute should be in 
     // a subdirectory 'fscommand' of the 'projector' executable's
     // location. In SWF5 there were no restrictions.
-    if (noCaseCompare(command, "exec"))
-    {
+    if (noCaseCompare(command, "exec")) {
         log_unimpl(_("FsCommand exec called with argument %s"), args);
         return;
     }
 
     // FSCommand allowscale
-    if (noCaseCompare(command, "allowscale"))
-    {
+    if (noCaseCompare(command, "allowscale")) {
         //log_debug("allowscale: %s", args);
         if (noCaseCompare(args, "true")) _gui.allowScale(true);
-        else
-        {
-                if (strtol(args.c_str(), NULL, 0)) _gui.allowScale(true);
-                else _gui.allowScale(false);
+        else {
+            if (strtol(args.c_str(), NULL, 0)) _gui.allowScale(true);
+            else _gui.allowScale(false);
         }
         return;
     }
 
     // FSCommand trapallkeys
-    if (noCaseCompare(command, "trapallkeys"))
-    {
+    if (noCaseCompare(command, "trapallkeys")) {
         log_unimpl(_("FsCommand trapallkeys called with argument %s"), args);
         return;
     }
@@ -798,6 +860,10 @@ Player::getGui()
     return createAOS4Gui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
+#ifdef GUI_HAIKU
+    return createHaikuGui(_windowID, _scale, _doLoop, *_runResources);
+#endif
+
 #ifdef GUI_DUMP
     return createDumpGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
@@ -807,8 +873,7 @@ Player::getGui()
 
 Player::~Player()
 {
-    if (_movieDef.get())
-    {
+    if (_movieDef.get()) {
         log_debug("~Player - _movieDef refcount: %d (1 will be dropped "
                 "now)", _movieDef->get_ref_count());
     }
