@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,12 +25,15 @@
 
 #include "dsodefs.h" // for DSOEXPORT
 #include "gui.h" // for inheritance
+#include <string>
 #include <fstream>
 #include <boost/scoped_array.hpp>
 
 namespace gnash
 {
 
+class Renderer_agg_base;
+
 typedef bool (*callback_t)(void*, int, void *data);
 
 class DSOEXPORT DumpGui : public Gui
@@ -41,10 +45,11 @@ class DSOEXPORT DumpGui : public Gui
     bool createMenu() { return true; }
     bool createMenuBar() { return true; }
     bool createWindow(int width, int height);
-    bool createWindow(const char* /*title*/, int width, int height)
+    bool createWindow(const char* /*title*/, int width, int height,
+            int /*x*/, int /*y*/)
         { return createWindow(width, height); }
     bool init(int argc, char **argv[]);
-    void quit();
+    virtual void quitUI();
     void renderBuffer() {return; }
     void render() { return; }
     void render(int /*minx*/, int /*miny*/, int /*maxx*/, int /*maxy*/)
@@ -54,7 +59,7 @@ class DSOEXPORT DumpGui : public Gui
     void setTimeout(unsigned int timeout);
     bool setupEvents() { return true; }
     void setFullscreen() { return; }
-    void setInvalidatedRegion(const rect& /*bounds*/) { return; }
+    void setInvalidatedRegion(const SWFRect& /*bounds*/) { return; }
     void setInvalidatedRegions(const InvalidatedRanges& /*ranges*/) { return; }
     void setCursor(gnash_cursor_type /*newcursor*/) { return; }
     void setRenderHandlerSize(int width, int height);
@@ -64,7 +69,8 @@ class DSOEXPORT DumpGui : public Gui
     void writeFrame();
 
 private:
-    Renderer *_agg_renderer;      /* pointer to AGG renderer */
+    
+    Renderer_agg_base* _agg_renderer;
 
     // A buffer to hold the actual image data. A boost::scoped_array
     // is destroyed on reset and when it goes out of scope (including on
