@@ -1,6 +1,7 @@
 // gui.cpp:  Top level GUI for SWF player, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,15 +29,21 @@
 #include "movie_root.h"
 #include "VM.h"
 #include "DisplayObject.h"
+#include "tu_file.h"
+#include "gnash.h"
+#include "RunResources.h"
 
 #ifdef GNASH_FPS_DEBUG
 #include "ClockTime.h"
 #include <boost/format.hpp>
 #endif
 
+#include <vector>
 #include <cstdio>
 #include <cstring>
-#include <algorithm> // std::max, std::min
+#include <algorithm> 
+#include <boost/algorithm/string/replace.hpp>
+#include <boost/lexical_cast.hpp>
 
 /// Define this to make sure each frame is fully rendered from ground up
 /// even if no motion has been detected in the movie.
@@ -71,7 +78,6 @@
 //#define SKIP_RENDERING_IF_LATE 1
 
 
-
 namespace gnash {
 
 Gui::Gui(RunResources& r) :
@@ -107,7 +113,6 @@ Gui::Gui(RunResources& r) :
     //       before and destroyed after _virtualClock !
     ,_systemClock()
     ,_virtualClock(_systemClock)
-
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
     ,_xpointer(0)
     ,_ypointer(0)
@@ -152,7 +157,6 @@ Gui::Gui(unsigned long xid, float scale, bool loop, RunResources& r)
     //       before and destroyed after _virtualClock !
     ,_systemClock()
     ,_virtualClock(_systemClock)
-
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
     ,_xpointer(0)
     ,_ypointer(0)
@@ -180,12 +184,31 @@ Gui::setFullscreen()
     log_unimpl(_("Fullscreen not yet supported in this GUI"));
 }
 
+void
+Gui::resizeWindow(int /*width*/, int /*height*/)
+{
+    log_unimpl(_("Window resize not yet supported in this GUI"));
+}
+
 void
 Gui::unsetFullscreen()
 {
     log_unimpl(_("Fullscreen not yet supported in this GUI"));
 }
 
+void
+Gui::quit()
+{
+    log_debug(__PRETTY_FUNCTION__);
+
+    // Take a screenshot of the last frame if required.
+    if (_screenShotter.get()) {
+        _screenShotter->last();
+    }
+    
+    quitUI();
+}
+
 void
 Gui::hideMenu()
 {
@@ -208,14 +231,13 @@ Gui::showMenu(bool /* show */)
 void
 Gui::allowScale(bool allow)
 {
-	if ( ! _stage )
-	{
-		log_error("Gui::allowScale called before a movie_root was available");
-		return;
-	}
-
-	if ( allow ) _stage->setStageScaleMode(movie_root::showAll);
-	else _stage->setStageScaleMode(movie_root::noScale);
+    if (!_stage) {
+        log_error("Gui::allowScale called before a movie_root was available");
+        return;
+    }
+    
+    if (allow) _stage->setStageScaleMode(movie_root::SCALEMODE_SHOWALL);
+    else _stage->setStageScaleMode(movie_root::SCALEMODE_NOSCALE);
 }
 
 void
@@ -241,164 +263,154 @@ Gui::restart()
 void
 Gui::updateStageMatrix()
 {
-	if ( ! VM::isInitialized() )
-	{
-		// When VM initializes, we'll get a call to resize_view, which
-		// would call us again.
-		//log_debug("Can't update stage matrix till VM is initialized");
-		return;
-	}
+    if ( ! VM::isInitialized() ) {
+        // When VM initializes, we'll get a call to resize_view, which
+        // would call us again.
+        //log_debug("Can't update stage matrix till VM is initialized");
+        return;
+    }
+    
+    assert(_stage); // when VM is initialized this should hold
+    
+    float swfwidth = _movieDef->get_width_pixels();
+    float swfheight = _movieDef->get_height_pixels();
+    
+    // Fetch scale mode
+    movie_root::ScaleMode scaleMode = _stage->getStageScaleMode();
 
-	assert(_stage); // when VM is initialized this should hold
-
-	float swfwidth = _movieDef->get_width_pixels();
-	float swfheight = _movieDef->get_height_pixels();
-
-	// Fetch scale mode
-	movie_root::ScaleMode scaleMode = _stage->getStageScaleMode();
-	switch (scaleMode)
-	{
-		case movie_root::noScale:
-			_xscale = _yscale = 1.0f;
-			break;
-
-		case movie_root::showAll:
-		{
-		
-			// set new scale value ( user-pixel / pseudo-pixel ). Do
-			// not divide by zero, or we end up with an invalid
-			// stage matrix that returns nan values.			
-			_xscale = (swfwidth == 0.0f) ? 1.0f : _width / swfwidth;
-			_yscale = (swfheight == 0.0f) ? 1.0f : _height / swfheight;
-			
-			// Scale proportionally, using smallest scale
-			if (_xscale < _yscale) _yscale = _xscale;
-			else if (_yscale < _xscale) _xscale = _yscale;
-
-			break;
-		}
-
-		case movie_root::noBorder:
-		{
-
-			// set new scale value ( user-pixel / pseudo-pixel )
-			_xscale = (swfwidth == 0.0f) ? 1.0f : _width / swfwidth;
-			_yscale = (swfheight == 0.0f) ? 1.0f : _height / swfheight;
-			
-			// Scale proportionally, using biggest scale
-			if (_xscale > _yscale) _yscale = _xscale;
-			else if (_yscale > _xscale) _xscale = _yscale;
-
-			break;
-		}
-
-		case movie_root::exactFit:
-		{
-			// NOTE: changing aspect ratio is valid!
-			_xscale = (swfwidth == 0.0f) ? 1.0f : _width / swfwidth;
-			_yscale = (swfheight == 0.0f) ? 1.0f : _height / swfheight;
-			//LOG_ONCE( log_unimpl("Stage.scaleMode=exactFit") );
-			break;
-		}
-
-		default:
-		{
-			log_error("Invalid scaleMode %d", scaleMode);
-			break;
-		}
-	}
-
-	_xoffset=0;
-	_yoffset=0;
-
-	// Fetch align mode
-	movie_root::StageAlign align = _stage->getStageAlignment();
-	movie_root::StageHorizontalAlign halign = align.first;
-	movie_root::StageVerticalAlign valign = align.second;
-
-	// Handle horizontal alignment
-	switch ( halign )
-	{
-		case movie_root::STAGE_H_ALIGN_L:
-		{
-			// _xoffset=0 is fine
-			break;
-		}
-
-		case movie_root::STAGE_H_ALIGN_R:
-		{
-			// Offsets in pixels
-			float defWidth = swfwidth *= _xscale;
-			float diffWidth = _width-defWidth;
-			_xoffset = diffWidth;
-			break;
-		}
-
-		case movie_root::STAGE_V_ALIGN_C:
-		{
-			// Offsets in pixels
-			float defWidth = swfwidth *= _xscale;
-			float diffWidth = _width-defWidth;
-			_xoffset = diffWidth/2.0;
-			break;
-		}
-
-		default:
-		{
-			log_error("Invalid horizontal align %d", valign);
-			break;
-		}
-	}
-
-	// Handle vertical alignment
-	switch ( valign )
-	{
-		case movie_root::STAGE_V_ALIGN_T:
-		{
-			// _yoffset=0 is fine
-			break;
-		}
-
-		case movie_root::STAGE_V_ALIGN_B:
-		{
-			float defHeight = swfheight *= _yscale;
-			float diffHeight = _height-defHeight;
-			_yoffset = diffHeight;
-			break;
-		}
-
-		case movie_root::STAGE_V_ALIGN_C:
-		{
-			float defHeight = swfheight *= _yscale;
-			float diffHeight = _height-defHeight;
-			_yoffset = diffHeight/2.0;
-			break;
-		}
-
-		default:
-		{
-			log_error("Invalid vertical align %d", valign);
-			break;
-		}
-	}
-
-	//log_debug("updateStageMatrix: scaleMode:%d, valign:%d, halign:%d",
-	//scaleMode, valign, halign);
-
-	// TODO: have a generic set_matrix ?
-	if (_renderer.get()) {
-		_renderer->set_scale(_xscale, _yscale);
-		_renderer->set_translation(_xoffset, _yoffset);
-	}
-	else
-	{
-		//log_debug("updateStageMatrix: could not signal updated stage
-		//matrix to renderer (no renderer registered)");
-	}
-
-	// trigger redraw
-	//_redraw_flag |= (_width!=width) || (_height!=height);
-	_redraw_flag = true; // this fixes bug #21971
+    switch (scaleMode) {
+        case movie_root::SCALEMODE_NOSCALE:
+            _xscale = _yscale = 1.0f;
+            break;
+        
+        case movie_root::SCALEMODE_SHOWALL:
+            // set new scale value ( user-pixel / pseudo-pixel ). Do
+            // not divide by zero, or we end up with an invalid
+            // stage matrix that returns nan values.			
+            _xscale = (swfwidth == 0.0f) ? 1.0f : _width / swfwidth;
+            _yscale = (swfheight == 0.0f) ? 1.0f : _height / swfheight;
+            
+            // Scale proportionally, using smallest scale
+            if (_xscale < _yscale) {
+                _yscale = _xscale;
+            } 
+            else if (_yscale < _xscale) {
+                _xscale = _yscale;
+            }
+            break;
+        
+        case movie_root::SCALEMODE_NOBORDER:
+            // set new scale value ( user-pixel / pseudo-pixel )
+            _xscale = (swfwidth == 0.0f) ? 1.0f : _width / swfwidth;
+            _yscale = (swfheight == 0.0f) ? 1.0f : _height / swfheight;
+            
+            // Scale proportionally, using biggest scale
+            if (_xscale > _yscale) {
+                _yscale = _xscale;
+            } 
+            else if (_yscale > _xscale) {
+                _xscale = _yscale;
+            }
+            break;
+        
+        case movie_root::SCALEMODE_EXACTFIT:
+            // NOTE: changing aspect ratio is valid!
+            _xscale = (swfwidth == 0.0f) ? 1.0f : _width / swfwidth;
+            _yscale = (swfheight == 0.0f) ? 1.0f : _height / swfheight;
+            break;
+        
+        default:
+            log_error("Invalid scaleMode %d", scaleMode);
+            break;
+    }
+    
+    _xoffset=0;
+    _yoffset=0;
+    
+    // Fetch align mode
+    movie_root::StageAlign align = _stage->getStageAlignment();
+    movie_root::StageHorizontalAlign halign = align.first;
+    movie_root::StageVerticalAlign valign = align.second;
+    
+    // Handle horizontal alignment
+    switch ( halign ) {
+    case movie_root::STAGE_H_ALIGN_L:
+        {
+            // _xoffset=0 is fine
+            break;
+        }
+        
+    case movie_root::STAGE_H_ALIGN_R:
+        {
+            // Offsets in pixels
+            float defWidth = swfwidth *= _xscale;
+            float diffWidth = _width-defWidth;
+            _xoffset = diffWidth;
+            break;
+        }
+        
+    case movie_root::STAGE_V_ALIGN_C:
+        {
+            // Offsets in pixels
+            float defWidth = swfwidth *= _xscale;
+            float diffWidth = _width-defWidth;
+            _xoffset = diffWidth/2.0;
+            break;
+        }
+        
+    default:
+        {
+            log_error("Invalid horizontal align %d", valign);
+            break;
+        }
+    }
+    
+    // Handle vertical alignment
+    switch ( valign ) {
+    case movie_root::STAGE_V_ALIGN_T:
+        {
+            // _yoffset=0 is fine
+            break;
+        }
+        
+    case movie_root::STAGE_V_ALIGN_B:
+        {
+            float defHeight = swfheight *= _yscale;
+            float diffHeight = _height-defHeight;
+            _yoffset = diffHeight;
+            break;
+        }
+        
+    case movie_root::STAGE_V_ALIGN_C:
+        {
+            float defHeight = swfheight *= _yscale;
+            float diffHeight = _height-defHeight;
+            _yoffset = diffHeight/2.0;
+            break;
+        }
+        
+    default:
+        {
+            log_error("Invalid vertical align %d", valign);
+            break;
+        }
+    }
+    
+    //log_debug("updateStageMatrix: scaleMode:%d, valign:%d, halign:%d",
+    //scaleMode, valign, halign);
+    
+    // TODO: have a generic set_matrix ?
+    if (_renderer.get()) {
+        _renderer->set_scale(_xscale, _yscale);
+        _renderer->set_translation(_xoffset, _yoffset);
+    } else {
+        //log_debug("updateStageMatrix: could not signal updated stage
+        //matrix to renderer (no renderer registered)");
+    }
+    
+    // trigger redraw
+    //_redraw_flag |= (_width!=width) || (_height!=height);
+    _redraw_flag = true; // this fixes bug #21971
 }
 
 
@@ -412,9 +424,8 @@ Gui::resize_view(int width, int height)
 	if ( VM::isInitialized() )
 	{
 
-		if ( _stage && _started )
-		{
-			_stage->set_display_viewport(0, 0, width, height);
+		if (_stage && _started) {
+			_stage->setDimensions(width, height);
 		}
 
 	}
@@ -445,7 +456,7 @@ Gui::toggleSound()
 
 
 void
-Gui::notify_mouse_moved(int ux, int uy) 
+Gui::notifyMouseMove(int ux, int uy) 
 {
 	movie_root* m = _stage;
 
@@ -463,7 +474,7 @@ Gui::notify_mouse_moved(int ux, int uy)
 	log_debug(_("mouse @ %d,%d"), x, y);
 #endif
 
-	if ( m->notify_mouse_moved(x, y) )
+	if ( m->mouseMoved(x, y) )
 	{
 		// any action triggered by the
 		// event required screen refresh
@@ -500,17 +511,31 @@ Gui::notify_mouse_moved(int ux, int uy)
 }
 
 void
-Gui::notify_mouse_clicked(bool mouse_pressed, int mask) 
+Gui::notifyMouseWheel(int delta)
 {
 	movie_root* m = _stage;
 	assert(m);
 
-    if ( ! _started ) return;
+    if (!_started) return;
+    if (_stopped) return;
 
-    if ( _stopped ) return;
+	if (m->mouseWheel(delta)) {
+		// any action triggered by the
+		// event required screen refresh
+		display(m);
+	}
+} 
 
-	if ( m->notify_mouse_clicked(mouse_pressed, mask) )
-	{
+void
+Gui::notifyMouseClick(bool mouse_pressed) 
+{
+	movie_root* m = _stage;
+	assert(m);
+
+    if (!_started) return;
+    if (_stopped) return;
+
+	if (m->mouseClick(mouse_pressed)) {
 		// any action triggered by the
 		// event required screen refresh
 		display(m);
@@ -535,139 +560,129 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 {
 
 	/* Handle GUI shortcuts */
-	if (pressed)
-	{
-		if (k == gnash::key::ESCAPE) {
-			if (isFullscreen()) {
-				_stage->setStageDisplayState(movie_root::DISPLAYSTATE_NORMAL);
-			}
-		}
-		
-		if (modifier & gnash::key::GNASH_MOD_CONTROL)
-		{
-			switch(k)
-			{
-				case gnash::key::r:
-				case gnash::key::R:
-					restart();
-					break;
-				case gnash::key::p:
-				case gnash::key::P:
-					pause();
-					break;
-				case gnash::key::l:
-				case gnash::key::L:
-					refreshView();
-					break;
-				case gnash::key::q:
-				case gnash::key::Q:
-				case gnash::key::w:
-				case gnash::key::W:
-					quit();
-					break;
-				case gnash::key::f:
-				case gnash::key::F:
-					toggleFullscreen();
-					break;
-				case gnash::key::h:
-				case gnash::key::H:
-					showUpdatedRegions(!showUpdatedRegions());
-					break;
-				case gnash::key::MINUS:
-				{
-					// Max interval allowed: 1 second (1FPS)
-					unsigned int ni = std::min(_interval+2, 1000u);
-					setInterval(ni);
-					break;
-                }
-				case gnash::key::PLUS:
-				{
-					// Min interval allowed: 1/100 second (100FPS)
-					unsigned int ni = std::max(_interval-2, 10u);
-					setInterval(ni);
-					break;
-                                }
-				case gnash::key::EQUALS:
-				{
-					if ( _stage )
-					{
-						float fps = _stage->frameRate();
-						// Min interval allowed: 1/100 second (100FPS)
-						unsigned int ni = 1000.0/fps;
-						setInterval(ni);
-					}
-					break;
-                                }
-				default:
-					break;
-			}
-
+    if (pressed) {
+        if (k == gnash::key::ESCAPE) {
+            if (isFullscreen()) {
+                _stage->setStageDisplayState(movie_root::DISPLAYSTATE_NORMAL);
+            }
+        }
+	
+        if (modifier & gnash::key::GNASH_MOD_CONTROL) {
+            switch (k) {
+              case gnash::key::o:
+              case gnash::key::O:
+                  takeScreenShot();
+                  break;
+              case gnash::key::r:
+              case gnash::key::R:
+                  restart();
+                  break;
+              case gnash::key::p:
+              case gnash::key::P:
+                  pause();
+                  break;
+              case gnash::key::l:
+              case gnash::key::L:
+                  refreshView();
+                  break;
+              case gnash::key::q:
+              case gnash::key::Q:
+              case gnash::key::w:
+              case gnash::key::W:
+                  quit();
+                  break;
+              case gnash::key::f:
+              case gnash::key::F:
+                  toggleFullscreen();
+                  break;
+              case gnash::key::h:
+              case gnash::key::H:
+                  showUpdatedRegions(!showUpdatedRegions());
+                  break;
+              case gnash::key::MINUS:
+              {
+                  // Max interval allowed: 1 second (1FPS)
+                  const size_t ni = std::min<size_t>(_interval + 2, 1000u);
+                  setInterval(ni);
+                  break;
+              }
+              case gnash::key::PLUS:
+              {
+                  // Min interval allowed: 1/100 second (100FPS)
+                  const size_t ni = std::max<size_t>(_interval - 2, 10u);
+                  setInterval(ni);
+                  break;
+              }
+              case gnash::key::EQUALS:
+              {
+                  if (_stage) {
+                      const float fps = _stage->frameRate();
+                      // Min interval allowed: 1/100 second (100FPS)
+                      const size_t ni = 1000.0/fps;
+                      setInterval(ni);
+                  }
+                  break;
+              }
+              default:
+                  break;
+            }
+            
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS
-			if ( _keyboardMouseMovements )
-			{
-				int step = _keyboardMouseMovementsStep; 
-				if (modifier & gnash::key::GNASH_MOD_SHIFT) step*=5; // x5 if SHIFT is pressed
-				switch(k)
-				{
-					case gnash::key::UP:
-					{
-						int newx = _xpointer;
-						int newy = _ypointer-step;
-						//log_debug("(theoretically) From %d,%d to %d,%d (step %d)", _xpointer, _ypointer, newx, newy, step);
-						if ( newy < 0 ) newy=0;
-						//log_debug("From %d,%d to %d,%d", _xpointer, _ypointer, newx, newy);
-						notify_mouse_moved(newx, newy);
-						break;
-					}
-					case gnash::key::DOWN:
-					{
-						int newx = _xpointer;
-						int newy = _ypointer+step;
-						//log_debug("(theoretically) From %d,%d to %d,%d (step %d)", _xpointer, _ypointer, newx, newy, step);
-						if ( newy >= _height ) newy = _height-1;
-						//log_debug("From %d,%d to %d,%d", _xpointer, _ypointer, newx, newy);
-						notify_mouse_moved(newx, newy);
-						break;
-					}
-					case gnash::key::LEFT:
-					{
-						int newx = _xpointer-step;
-						int newy = _ypointer;
-						//log_debug("(theoretically) From %d,%d to %d,%d (step %d)", _xpointer, _ypointer, newx, newy, step);
-						if ( newx < 0 ) newx = 0;
-						//log_debug("From %d,%d to %d,%d", _xpointer, _ypointer, newx, newy);
-						notify_mouse_moved(newx, newy);
-						break;
-					}
-					case gnash::key::RIGHT:
-					{
-						int newy = _ypointer;
-						int newx = _xpointer+step;
-						//log_debug("(theoretically) From %d,%d to %d,%d (step %d)", _xpointer, _ypointer, newx, newy, step);
-						if ( newx >= _width ) newx = _width-1;
-						//log_debug("From %d,%d to %d,%d", _xpointer, _ypointer, newx, newy);
-						notify_mouse_moved(newx, newy);
-						break;
-					}
-					default:
-						break;
-				}
-			}
+            if ( _keyboardMouseMovements ) {
+                int step = _keyboardMouseMovementsStep; 
+                // x5 if SHIFT is pressed
+                if (modifier & gnash::key::GNASH_MOD_SHIFT) step *= 5; 
+                switch (k) {
+                  case gnash::key::UP:
+                  {
+                      int newx = _xpointer;
+                      int newy = _ypointer-step;
+                      if ( newy < 0 ) newy=0;
+                      notifyMouseMove(newx, newy);
+                      break;
+                  }
+                  case gnash::key::DOWN:
+                  {
+                      int newx = _xpointer;
+                      int newy = _ypointer+step;
+                      if ( newy >= _height ) newy = _height-1;
+                      notifyMouseMove(newx, newy);
+                      break;
+                  }
+                  case gnash::key::LEFT:
+                  {
+                      int newx = _xpointer-step;
+                      int newy = _ypointer;
+                      if ( newx < 0 ) newx = 0;
+                      notifyMouseMove(newx, newy);
+                      break;
+                  }
+                  case gnash::key::RIGHT:
+                  {
+                      const int newy = _ypointer;
+                      int newx = _xpointer + step;
+                      if ( newx >= _width ) newx = _width-1;
+                      notifyMouseMove(newx, newy);
+                      break;
+                  }
+                  default:
+                      break;
+                }
+            }
 #endif // ENABLE_KEYBOARD_MOUSE_MOVEMENTS
-		}
-	}
-
-    if ( ! _started ) return;
-
-    if ( _stopped ) return;
-
-	if ( _stage->notify_key_event(k, pressed) )
-	{
-		// any action triggered by the
-		// event required screen refresh
-		display(_stage);
-	}
-
+        }
+    }
+    
+    if (!_started) return;
+    
+    if (_stopped) return;
+    
+    if (_stage->keyEvent(k, pressed)) {
+        // any action triggered by the
+        // event required screen refresh
+        display(_stage);
+    }
+    
 }
 
 bool
@@ -786,7 +801,8 @@ Gui::display(movie_root* m)
 		
 			for (size_t rno = 0; rno < changed_ranges.size(); rno++) {
 			
-				geometry::Range2d<float> bounds = changed_ranges.getRange(rno);
+				const geometry::Range2d<int>& bounds = 
+					changed_ranges.getRange(rno);
 
 				point corners[4];
 				float xmin = bounds.getMinX();
@@ -865,43 +881,39 @@ Gui::stop()
 void
 Gui::pause()
 {
-    if ( _stopped )
-    {
+    if (_stopped) {
         play();
+        return;
     }
-    else
-    {
-        // TODO: call stop() instead ?
-        // The only thing I see is that ::stop exits full-screen,
-        // but I'm not sure that's intended behaviour
 
-        // @todo since we registered the sound handler, shouldn't we know
-        //       already what it is ?!
-    	sound::sound_handler* s = _stage->runResources().soundHandler();
-    	if ( s ) s->pause();
-        _stopped = true;
+    // TODO: call stop() instead ?
+    // The only thing I see is that ::stop exits full-screen,
+    // but I'm not sure that's intended behaviour
 
-        log_debug("Pausing virtual clock");
-        _virtualClock.pause();
+    // @todo since we registered the sound handler, shouldn't we know
+    //       already what it is ?!
+    sound::sound_handler* s = _stage->runResources().soundHandler();
+    if (s) s->pause();
+    _stopped = true;
 
-        stopHook();
-    }
+    log_debug("Pausing virtual clock");
+    _virtualClock.pause();
+
+    stopHook();
 }
 
 void
 Gui::start()
 {
     assert ( ! _started );
-    if ( _stopped )
-    {
+    if (_stopped) {
         log_debug("Gui is in stop mode, won't start application");
         return;
     }
 
-    std::auto_ptr<Movie> mr ( _movieDef->createMovie() );
-    mr->setVariables(_flashVars);
-
-    _stage->setRootMovie( mr.release() ); // will construct the instance
+    // Initializes the stage with a Movie and the passed flash vars and
+    // Scriptable vars for ExternalInterface.
+    _stage->init(_movieDef.get(), _flashVars, _scriptableVars);
 
     bool background = true; // ??
     _stage->set_background_alpha(background ? 1.0f : 0.05f);
@@ -924,48 +936,53 @@ bool
 Gui::advanceMovie()
 {
 
-	if ( isStopped() ) return true;
+    if (isStopped()) {
+        return true;
+    }
 
-    if ( ! _started ) start();
-  
-
-
-	gnash::movie_root* m = _stage;
-	
-// Define REVIEW_ALL_FRAMES to have *all* frames
-// consequencially displaied. Useful for debugging.
-//#define REVIEW_ALL_FRAMES 1
+    if (!_started) {
+        start();
+    }
 
+    gnash::movie_root* m = _stage;
+    
+    // Define REVIEW_ALL_FRAMES to have *all* frames
+    // consequentially displayed. Useful for debugging.
+    //#define REVIEW_ALL_FRAMES 1
+    
 #ifndef REVIEW_ALL_FRAMES
-	// Advance movie by one frame
-	bool advanced = m->advance();
+    // Advance movie by one frame
+    bool advanced = m->advance();
 #else
-	size_t cur_frame = m->getRootMovie()->get_current_frame();
-	size_t tot_frames = m->getRootMovie()->get_frame_count();
-	bool advanced = m->advance();
-	m->getRootMovie.ensureFrameLoaded(tot_frames);
-	m->goto_frame(cur_frame+1);
+    const size_t cur_frame = m->getRootMovie()->get_current_frame();
+    const size_t tot_frames = m->getRootMovie()->get_frame_count();
+    const bool advanced = m->advance();
+    
+    m->getRootMovie.ensureFrameLoaded(tot_frames);
+    m->goto_frame(cur_frame + 1);
     m->set_play_state(gnash::MovieClip::PLAYSTATE_PLAY);
-	log_debug(_("Frame %d"), m->get_current_frame());
+    log_debug(_("Frame %d"), m->get_current_frame());
 #endif
-
+    
 #ifdef GNASH_FPS_DEBUG
-	if ( advanced ) fpsCounterTick(); // will be a no-op if fps_timer_interval is zero
+    // will be a no-op if fps_timer_interval is zero
+    if (advanced) {
+	fpsCounterTick();
+    }
 #endif
-
-
-
+    
+    
     // TODO: ask stage about doDisplay ?
     // - if it didn't advance might need to check updateAfterEvent
     bool doDisplay = true;
-
+    
 #ifdef SKIP_RENDERING_IF_LATE
     // We want to skip rendering IFF it's time to advance again.
     // We'll ask the stage about it
-    if ( _stage->timeToNextFrame() <= 0 )
-    {
-        if ( doDisplay ) // or should it be if advanced ?
-        {
+    if (_stage->timeToNextFrame() <= 0) {
+
+        // or should it be if advanced ?
+        if (doDisplay) {
             // TODO: take note of a frame drop (count them)
             //log_debug("Frame rendering dropped due to being late");
 #ifdef GNASH_FPS_DEBUG
@@ -978,28 +995,68 @@ Gui::advanceMovie()
 
 	if (doDisplay) display(m);
 
-	if ( ! loops() )
-	{
+	if (!loops()) {
 		size_t curframe = m->get_current_frame(); // can be 0 on malformed SWF
 		const gnash::MovieClip& si = m->getRootMovie();
-		if (curframe + 1 >= si.get_frame_count())
-		{
+		if (curframe + 1 >= si.get_frame_count()) {
 			quit(); 
 		}
 	}
 
-    /// Quit if we've reached the advance limit.
-    if (_maxAdvances && advanced && (_advances++ > _maxAdvances))
-    {
-        quit();
+    if (_screenShotter.get()) {
+        _screenShotter->screenShot(_advances);
+    }
+
+    // Only increment advances and check for exit condition when we've
+    // really changed frame.
+    if (advanced) {
+        /// Quit if we've reached the frame advance limit.
+        if (_maxAdvances && (_advances > _maxAdvances)) {
+            quit();
+        }
+        ++_advances;
     }
 
 	return true;
 }
 
+void
+Gui::takeScreenShot()
+{
+    if (!_screenShotter.get()) {
+        // If no ScreenShotter exists, none was requested at startup.
+        // We use a default filename pattern.
+        URL url(_runResources.baseURL());
+        std::string::size_type p = url.path().rfind('/');
+        const std::string& name = (p == std::string::npos) ? url.path() :
+            url.path().substr(p + 1);
+        const std::string& filename = "screenshot-" + name + "-%f";
+        _screenShotter.reset(new ScreenShotter(_renderer, filename));
+    }
+    assert (_screenShotter.get());
+    _screenShotter->now();
+}
+
+void
+Gui::requestScreenShots(const ScreenShotter::FrameList& l, bool last,
+        const std::string& filename)
+{
+    // Nothing to do if there is no renderer or if no frames should be
+    // saved.
+    if (!_renderer.get() || (l.empty() && !last)) {
+	return;
+    }
+
+    _screenShotter.reset(new ScreenShotter(_renderer, filename));
+    if (last) _screenShotter->lastFrame();
+    _screenShotter->setFrames(l);
+
+}
+
 void
 Gui::setCursor(gnash_cursor_type /*newcursor*/)
 {
+    /* do nothing */
 }
 
 bool
@@ -1009,25 +1066,28 @@ Gui::want_redraw()
 }
 
 void
-Gui::setInvalidatedRegion(const rect& /*bounds*/)
+Gui::setInvalidatedRegion(const SWFRect& /*bounds*/)
 {
+    /* do nothing */
 }
 
 void
 Gui::setInvalidatedRegions(const InvalidatedRanges& ranges)
 {
-	// fallback to single regions
-	geometry::Range2d<float> full = ranges.getFullArea();
-	
-	rect bounds;
-	
-	if (full.isFinite())
-		bounds = rect(full.getMinX(), full.getMinY(), full.getMaxX(), full.getMaxY());
-	else
-	if (full.isWorld())
-		bounds.set_world();
-	
-	setInvalidatedRegion(bounds);
+    // fallback to single regions
+    geometry::Range2d<int> full = ranges.getFullArea();
+    
+    SWFRect bounds;
+    
+    if (full.isFinite()) {
+        bounds = SWFRect(full.getMinX(), full.getMinY(),
+                full.getMaxX(), full.getMaxY());
+    }
+    else if (full.isWorld()) {
+        bounds.set_world();
+    }
+    
+    setInvalidatedRegion(bounds);
 }
 
 #ifdef USE_SWFTREE
@@ -1037,8 +1097,7 @@ Gui::getMovieInfo() const
 {
     std::auto_ptr<InfoTree> tr;
 
-    if ( ! VM::isInitialized() )
-    {
+    if (! VM::isInitialized()) {
         return tr;
     }
 
@@ -1066,8 +1125,7 @@ Gui::getMovieInfo() const
     // which relies on the availability of a _rootMovie for doing
     // it's work, while we don't set it if we didn't start..
     // 
-    if ( ! _started )
-    {
+    if (! _started) {
         topIter = tr->insert(topIter, StringPair("Stage properties", 
                     "not constructed yet"));
         return tr;
@@ -1083,34 +1141,29 @@ Gui::getMovieInfo() const
 
     const DisplayObject* ch;
     ch = stage.getActiveEntityUnderPointer();
-    if ( ch )
-    {
-	    std::stringstream ss;
-	    ss << ch->getTarget() << " (" + typeName(*ch)
-            << " - id:" << ch->get_id()
-            << " - depth:" << ch->get_depth()
-            << " - useHandCursor:" << ch->allowHandCursor()
-            << ")";
+    if (ch) {
+        std::stringstream ss;
+        ss << ch->getTarget() << " (" + typeName(*ch)
+           << " - depth:" << ch->get_depth()
+           << " - useHandCursor:" << ch->allowHandCursor()
+           << ")";
     	firstLevelIter = tr->append_child(topIter, StringPair("Active entity under mouse pointer", ss.str()));
     }
 
     ch = stage.getEntityUnderPointer();
-    if ( ch )
-    {
-	    std::stringstream ss;
-	    ss << ch->getTarget() << " (" + typeName(*ch) 
-               << " - id:" << ch->get_id() << " - depth:" << ch->get_depth()
-               << ")";
+    if (ch) {
+        std::stringstream ss;
+        ss << ch->getTarget() << " (" + typeName(*ch) 
+           << " - depth:" << ch->get_depth()
+           << ")";
 	firstLevelIter = tr->append_child(topIter, StringPair("Topmost entity under mouse pointer", ss.str()));
     }
-
+    
     ch = stage.getDraggingCharacter();
-    if ( ch ) 
-    {
-	    std::stringstream ss;
-	    ss << ch->getTarget() << " (" + typeName(*ch) 
-               << " - id:" << ch->get_id()
-               << " - depth:" << ch->get_depth() << ")";
+    if (ch) {
+        std::stringstream ss;
+        ss << ch->getTarget() << " (" + typeName(*ch) 
+           << " - depth:" << ch->get_depth() << ")";
     	firstLevelIter = tr->append_child(topIter, StringPair("Dragging character: ", ss.str()));
     }
 
@@ -1122,8 +1175,7 @@ Gui::getMovieInfo() const
     GC::get().countCollectables(cc);
     
     const std::string lbl = "GC managed ";
-    for (GC::CollectablesCount::iterator i=cc.begin(), e=cc.end(); i!=e; ++i)
-    {
+    for (GC::CollectablesCount::iterator i=cc.begin(), e=cc.end(); i!=e; ++i) {
         const std::string& typ = i->first;
         std::ostringstream ss;
         ss << i->second;
@@ -1147,9 +1199,8 @@ Gui::fpsCounterTick()
   // frame count on exit is still valid
   ++fps_counter_total;
 
-  if ( ! fps_timer_interval )
-  {
-	  return;
+  if (! fps_timer_interval) {
+      return;
   }
 
   boost::uint64_t current_timer = clocktime::getTicks();
@@ -1216,12 +1267,19 @@ Gui::fpsCounterTick()
 void
 Gui::addFlashVars(Gui::VariableMap& from)
 {
-    for (VariableMap::iterator i=from.begin(), ie=from.end(); i!=ie; ++i)
-    {
+    for (VariableMap::iterator i=from.begin(), ie=from.end(); i!=ie; ++i) {
         _flashVars[i->first] = i->second;
     }
 }
 
+void
+Gui::addScriptableVar(const std::string &name, const std::string &value)
+{
+    log_debug("Adding scriptable variable \"%s\" = %s",
+              name, value);
+    _scriptableVars[name] = value;
+}
+
 void
 Gui::setMovieDefinition(movie_definition* md)
 {
@@ -1240,31 +1298,105 @@ Gui::setStage(movie_root* stage)
 bool
 Gui::yesno(const std::string& question)
 {
-    log_error("This gui didn't override 'yesno', assuming 'yes' answer to question: %s", question);
+    log_error("This gui didn't override 'yesno', assuming 'yes' answer to "
+            "question: %s", question);
     return true;
 }
 
 void
 Gui::setQuality(Quality q)
 {
-	if ( ! _stage )
-	{
-		log_error("Gui::setQuality called before a movie_root was available");
-		return;
-	}
+    if (!_stage) {
+	log_error("Gui::setQuality called before a movie_root was available");
+	return;
+    }
     _stage->setQuality(q);
 }
 
 Quality
 Gui::getQuality() const
 {
-	if ( ! _stage )
-	{
-		log_error("Gui::getQuality called before a movie_root was available");
-		return QUALITY_HIGH; // just a guess..
-	}
+    if (!_stage) {
+	log_error("Gui::getQuality called before a movie_root was available");
+	// just a guess..
+	return QUALITY_HIGH;
+    }
     return _stage->getQuality();
 }
 
+void
+Gui::setFDCallback(int fd, boost::function<void ()> callback)
+{
+    log_debug("Setting callback for fd #%d", fd);
+    
+    _fd_callbacks[fd] = callback;
+
+    watchFD(fd);
+}
+
+
+void
+Gui::callCallback(int fd)
+{
+    std::map<int, boost::function<void ()> >::iterator it = _fd_callbacks.find(fd);
+
+    if (it == _fd_callbacks.end()) {
+        log_error("Attempted to call a callback for an unregistered fd.");
+        return;
+    }
+
+    boost::function<void()>& f = it->second;
+
+    f();
+}
+
+void
+ScreenShotter::saveImage(const std::string& id) const
+{
+    // Replace all "%f" in the filename with the frameAdvance.
+    std::string outfile(_fileName);
+    boost::replace_all(outfile, "%f", id);
+    
+    FILE* f = std::fopen(outfile.c_str(), "wb");
+    if (f) {
+        boost::shared_ptr<IOChannel> t(new tu_file(f, true));
+        _renderer->renderToImage(t, GNASH_FILETYPE_PNG);
+    } else {
+        log_error("Failed to open screenshot file \"%s\"!", outfile);
+    }
+}
+
+void
+ScreenShotter::screenShot(size_t frameAdvance)
+{
+    // Save an image if an spontaneous screenshot was requested or the
+    // frame is in the list of requested frames.
+    if (_immediate || std::binary_search(_frames.begin(), _frames.end(),
+                frameAdvance)) {
+        saveImage(boost::lexical_cast<std::string>(frameAdvance));
+        _immediate = false;
+    }
+}
+
+void
+ScreenShotter::last() const
+{
+    if (_last) {
+	saveImage("last");
+    }
+}
+
+void
+ScreenShotter::setFrames(const FrameList& frames)
+{
+    _frames = frames;
+    std::sort(_frames.begin(), _frames.end());
+}
+
 // end of namespace
 }
+
+// local Variables:
+// mode: C++
+// indent-tabs-mode: nil
+// End:
