@@ -39,12 +39,8 @@
 #include "smart_ptr.h"
 #include "DisplayObject.h"
 #include "Global_as.h"
-
-#ifdef USE_FFMPEG
-# include "MediaHandlerFfmpeg.h"
-#elif defined(USE_GST)
-# include "MediaHandlerGst.h"
-#endif
+#include "NamingPolicy.h"
+#include "StreamProvider.h"
 
 enum
 {
@@ -59,7 +55,7 @@ struct _GnashView {
     const gchar *uri;
     guint advance_timer;
 
-	std::auto_ptr<gnash::media::MediaHandler> media_handler;
+    boost::shared_ptr<gnash::media::MediaHandler> media_handler;
     boost::shared_ptr<gnash::sound::sound_handler> sound_handler;
 
     /// Handlers (for sound etc) for a libcore run.
@@ -215,33 +211,25 @@ gnash_view_init(GnashView *view)
     gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
     dbglogfile.setVerbosity(3);
 
-    // Init media
-#ifdef USE_FFMPEG
-    view->media_handler.reset( new gnash::media::ffmpeg::MediaHandlerFfmpeg() );
-    gnash::media::MediaHandler::set(view->media_handler);
-#elif defined(USE_GST)
-    view->media_handler.reset( new gnash::media::gst::MediaHandlerGst() );
-    gnash::media::MediaHandler::set(view->media_handler);
-#else
-    log_error(_("No media support compiled in"));
-#endif    
+    // Use the default media handler.
+    // TODO: allow setting this.
+    view->media_handler.reset(gnash::media::MediaFactory::instance().get(""));
 
     // Init sound
 #ifdef SOUND_SDL
     try {
-        view->sound_handler.reset(gnash::sound::create_sound_handler_sdl(""));
+        view->sound_handler.reset(gnash::sound::create_sound_handler_sdl(
+                view->media_handler.get(), ""));
     } catch (gnash::SoundException& ex) {
         gnash::log_error(_("Could not create sound handler: %s."
                            " Will continue w/out sound."), ex.what());
     }
-#elif defined(SOUND_GST)
-    view->sound_handler.reset(media::create_sound_handler_gst());
-#else
     gnash::log_error(_("Sound requested but no sound support compiled in"));
 #endif
 
     view->canvas = GNASH_CANVAS(gnash_canvas_new());
-    gnash_canvas_setup(view->canvas, 0, NULL);
+    std::string nullstr;
+    gnash_canvas_setup(view->canvas, nullstr, nullstr, 0, NULL);
     gtk_container_add (GTK_CONTAINER (view), GTK_WIDGET(view->canvas));
     gtk_widget_show (GTK_WIDGET(view->canvas));
 
@@ -271,7 +259,7 @@ gnash_view_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
     gtk_widget_size_allocate (GTK_BIN(widget)->child, allocation);
 
     if( view->stage.get() != NULL) {
-    	view->stage->set_display_viewport(0, 0, allocation->width, allocation->height);
+    	view->stage->setDimensions(allocation->width, allocation->height);
 
         boost::shared_ptr<gnash::Renderer> renderer = gnash_canvas_get_renderer(view->canvas);
         float xscale = allocation->width / view->movie_definition->get_width_pixels();
@@ -317,7 +305,7 @@ key_press_event_cb(GtkWidget */*widget*/, GdkEventKey *event, gpointer data)
     gnash::key::code c = gdk_to_gnash_key(event->keyval);
     
     if (c != gnash::key::INVALID) {
-        if( view->stage->notify_key_event(c, true) )
+        if( view->stage->keyEvent(c, true) )
             gnash_view_display(view);
         return TRUE;
     }
@@ -336,7 +324,7 @@ key_release_event_cb(GtkWidget */*widget*/, GdkEventKey *event, gpointer data)
     gnash::key::code c = gdk_to_gnash_key(event->keyval);
     
     if (c != gnash::key::INVALID) {
-        if( view->stage->notify_key_event(c, false) )
+        if( view->stage->keyEvent(c, false) )
             gnash_view_display(view);
         return TRUE;
     }
@@ -358,22 +346,21 @@ button_press_event_cb(GtkWidget */*widget*/, GdkEventButton *event, gpointer dat
 
     gtk_widget_grab_focus(GTK_WIDGET(view->canvas));
 
-    int	mask = 1 << (event->button - 1);
-    view->stage->notify_mouse_clicked(true, mask);
+    view->stage->mouseClick(true);
 
     return TRUE;
 }
 
 static gboolean
-button_release_event_cb(GtkWidget */*widget*/, GdkEventButton *event, gpointer data)
+button_release_event_cb(GtkWidget* /*widget*/, GdkEventButton* /*event*/,
+        gpointer data)
 {
     GNASH_REPORT_FUNCTION;
     GnashView *view = GNASH_VIEW(data);
     if (view->stage.get() == NULL)
         return FALSE;
 
-    int	mask = 1 << (event->button - 1);
-    view->stage->notify_mouse_clicked(false, mask);
+    view->stage->mouseClick(false);
 
     return TRUE;
 }
@@ -394,7 +381,7 @@ motion_notify_event_cb(GtkWidget */*widget*/, GdkEventMotion *event, gpointer da
 	// A stage pseudopixel is user pixel / _yscale high
 	boost::int32_t y = event->y / yscale;
 
-	if ( view->stage->notify_mouse_moved(x, y) )
+	if ( view->stage->mouseMoved(x, y) )
 	{
 		// any action triggered by the
 		// event required screen refresh
@@ -481,7 +468,8 @@ gnash_view_load_movie(GnashView *view, const gchar *uri)
     std::map<std::string, std::string> variables;
     gnash::URL::parse_querystring(url.querystring(), variables);
 
-    gnash::Movie* m = view->stage->init(view->movie_definition.get(), variables);
+    gnash::Movie* m = view->stage->init(view->movie_definition.get(), variables,
+                                        variables);
     view->movie = m;
 
     view->stage->set_background_alpha(1.0f);
