@@ -1,6 +1,7 @@
 // GnashImageSystemIOHeaders.h: Compatibility IO header for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -34,7 +35,6 @@
 /// @todo A configure 'test' (a #define in gnashconfig.h) might be a better
 ///       way of checking for compiler.
 
-#include "gnashconfig.h"
 #if !defined(_MSC_VER)
 # include <unistd.h>
 #else
