@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,20 +16,14 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
 #ifndef GNASH_SIMPLEBUFFER_H
 #define GNASH_SIMPLEBUFFER_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
 
-//#include "dsodefs.h" // for DSOEXPORT (not used)
-
-#include <cassert> // for assert
 #include <boost/cstdint.hpp> // for boost::uint8_t
 #include <algorithm> // for std::copy
 #include <boost/scoped_array.hpp>
+#include <cassert>
 
 
 namespace gnash {
