@@ -65,7 +65,7 @@ public:
 
     DepthEquals(int depth) : _depth(depth) {}
 
-    bool operator() (const DisplayItem& item) {
+    bool operator() (const DisplayObject* item) {
         if (!item) return false;
         return item->get_depth() == _depth;
     }
@@ -76,14 +76,14 @@ private:
 
 struct DepthGreaterThan
 {
-    bool operator()(const DisplayItem& a, const DisplayItem& b) {
+    bool operator()(const DisplayObject* a, const DisplayObject* b) {
         return a->get_depth() > b->get_depth();
     }
 };
 
 struct DepthLessThan
 {
-    bool operator()(const DisplayItem& a, const DisplayItem& b) {
+    bool operator()(const DisplayObject* a, const DisplayObject* b) {
         return a->get_depth() < b->get_depth();
     }
 };
@@ -94,7 +94,7 @@ public:
 
     DepthGreaterOrEqual(int depth) : _depth(depth) {}
 
-    bool operator() (const DisplayItem& item) {
+    bool operator() (const DisplayObject* item) {
         if (!item) return false;
         return item->get_depth() >= _depth;
     }
@@ -106,31 +106,34 @@ private:
 class NameEquals
 {
 public:
-    NameEquals(const std::string& name) : _name(name) {}
+    NameEquals(string_table& st, string_table::key name, bool caseless)
+        :
+        _st(st),
+        _caseless(caseless),
+        _name(caseless ? _st.noCase(name) : name)
+    {}
 
-    bool operator() (const DisplayItem& item) {
+    bool operator() (const DisplayObject* item) {
         assert (item);
-        return item->get_name() == _name;
+
+        // TODO: this is necessary because destroy() is called in
+        // movie_root, leaving destroyed items on the DisplayList. They
+        // shouldn't be found. A better fix would be to stop destroying
+        // objects there and add to the invariant that there are never
+        // destroyed DisplayObjects in the DisplayList.
+        if (item->isDestroyed()) return false;
+        
+        const string_table::key itname =
+            _caseless ? _st.noCase(item->get_name()) : item->get_name();
+
+        return itname == _name;
+
     }
 
 private:
-    const std::string& _name;
-};
-
-
-class NameEqualsNoCase
-{
-public:
-    NameEqualsNoCase(const std::string& name) : _name(name) {}
-
-    bool operator() (const DisplayItem& item) {
-        assert (item);
-        return _noCaseEquals(item->get_name(), _name);
-    }
-
-private:
-    const std::string& _name;
-    StringNoCaseEqual _noCaseEquals;
+    string_table& _st;
+    const bool _caseless;
+    const string_table::key _name;
 };
 
 } // anonymous namespace
@@ -155,15 +158,18 @@ DisplayList::getNextHighestDepth() const
 }
 
 DisplayObject*
-DisplayList::getDisplayObjectAtDepth(int depth)
+DisplayList::getDisplayObjectAtDepth(int depth) const
 {
     testInvariant();
 
-    for (iterator it = _charsByDepth.begin(), itEnd = _charsByDepth.end();
+    for (const_iterator it = _charsByDepth.begin(), itEnd = _charsByDepth.end();
         it != itEnd; ++it) {
 
         DisplayObject* ch = *it;
 
+        // Should not be there!
+        if (ch->isDestroyed()) continue;
+
         // found
         if (ch->get_depth() == depth) return ch;
 
@@ -177,14 +183,15 @@ DisplayList::getDisplayObjectAtDepth(int depth)
 
 
 DisplayObject*
-DisplayList::getDisplayObjectByName(const std::string& name)
+DisplayList::getDisplayObjectByName(string_table& st, string_table::key name,
+        bool caseless) const
 {
     testInvariant();
 
-    const container_type::iterator e = _charsByDepth.end();
+    const container_type::const_iterator e = _charsByDepth.end();
 
     container_type::const_iterator it =
-        std::find_if( _charsByDepth.begin(), e, NameEquals(name));
+        std::find_if(_charsByDepth.begin(), e, NameEquals(st, name, caseless));
 
     if (it == e) return 0;
     
@@ -192,24 +199,8 @@ DisplayList::getDisplayObjectByName(const std::string& name)
 
 }
 
-DisplayObject*
-DisplayList::getDisplayObjectByName_i(const std::string& name)
-{
-    testInvariant();
-
-    const container_type::iterator e = _charsByDepth.end();
-
-    container_type::const_iterator it =
-        std::find_if( _charsByDepth.begin(), e, NameEqualsNoCase(name));
-
-    if ( it == e ) return NULL;
-    
-    return *it;
-}
-
 void
-DisplayList::placeDisplayObject(DisplayObject* ch, int depth,
-        as_object* initObj)
+DisplayList::placeDisplayObject(DisplayObject* ch, int depth)
 {
     assert(!ch->unloaded());
     ch->set_invalidated();
@@ -221,7 +212,7 @@ DisplayList::placeDisplayObject(DisplayObject* ch, int depth,
 
     if (it == _charsByDepth.end() || (*it)->get_depth() != depth) {
         // add the new char
-        _charsByDepth.insert(it, DisplayItem(ch));
+        _charsByDepth.insert(it, ch);
     }
     else {
         // remember bounds of old char
@@ -232,7 +223,7 @@ DisplayList::placeDisplayObject(DisplayObject* ch, int depth,
         DisplayObject* oldCh = *it;
 
         // replace existing char (before calling unload!)
-        *it = DisplayItem(ch);
+        *it = ch;
     
         if (oldCh->unload()) {
             // reinsert removed DisplayObject if needed
@@ -244,9 +235,6 @@ DisplayList::placeDisplayObject(DisplayObject* ch, int depth,
         ch->extend_invalidated_bounds(old_ranges);                
     }
 
-    // Give life to this instance
-    ch->stagePlacementCallback(initObj);
-
     testInvariant();
 }
 
@@ -260,9 +248,9 @@ DisplayList::add(DisplayObject* ch, bool replace)
                 DepthGreaterOrEqual(depth));
 
     if (it == _charsByDepth.end() || (*it)->get_depth() != depth) {
-        _charsByDepth.insert(it, DisplayItem(ch));
+        _charsByDepth.insert(it, ch);
     }
-    else if (replace) *it = DisplayItem(ch);
+    else if (replace) *it = ch;
 
     testInvariant();
 }
@@ -283,10 +271,8 @@ DisplayList::replaceDisplayObject(DisplayObject* ch, int depth,
         std::find_if(_charsByDepth.begin(), _charsByDepth.end(),
             DepthGreaterOrEqual(depth));
 
-    DisplayItem di(ch);
-
     if (it == _charsByDepth.end() || (*it)->get_depth() != depth) {
-        _charsByDepth.insert(it, di);
+        _charsByDepth.insert(it, ch);
     }
     else {
         // Make a copy (before replacing)
@@ -308,7 +294,7 @@ DisplayList::replaceDisplayObject(DisplayObject* ch, int depth,
         oldch->add_invalidated_bounds(old_ranges, true);        
 
         // replace existing char (before calling unload)
-        *it = di;
+        *it = ch;
 
         // Unload old char
         if (oldch->unload()) {
@@ -320,16 +306,13 @@ DisplayList::replaceDisplayObject(DisplayObject* ch, int depth,
         // extend invalidated bounds
         // WARNING: when a new Button DisplayObject is added,
         //          the invalidated bounds computation will likely
-        //          be bogus, as the actual DisplayObject shown is not instantiated
-        //          until ::stagePlacementCallback for buttons (I'd
-        //          say this is a bug in Button). 
+        //          be bogus, as the actual DisplayObject shown is not
+        //          instantiated until stagePlacementCallback for buttons
+        //          (I'd say this is a bug in Button). 
         ch->extend_invalidated_bounds(old_ranges);                
 
     }
 
-    // Give life to this instance
-    ch->stagePlacementCallback();
-
     testInvariant();
 }
     
@@ -455,7 +438,7 @@ DisplayList::swapDepths(DisplayObject* ch1, int newdepth)
     // Found another DisplayObject at the given depth
     if (it2 != _charsByDepth.end() && (*it2)->get_depth() == newdepth)
     {
-        DisplayItem ch2 = *it2;
+        DisplayObject* ch2 = *it2;
 
         ch2->set_depth(srcdepth);
 
@@ -545,9 +528,6 @@ DisplayList::insertDisplayObject(DisplayObject* obj, int index)
         ++index, ++it;
     }
 
-    // Give life to this instance
-    obj->stagePlacementCallback();
-
     testInvariant();
 
 }
@@ -576,9 +556,6 @@ DisplayList::addDisplayObject(DisplayObject* obj)
     // Insert the DisplayObject at the end
     _charsByDepth.insert(_charsByDepth.end(), obj);
 
-    // Give life to this instance
-    obj->stagePlacementCallback();
-
     testInvariant();
 
 }
@@ -587,37 +564,41 @@ DisplayList::addDisplayObject(DisplayObject* obj)
 bool
 DisplayList::unload()
 {
-    //GNASH_REPORT_FUNCTION;
-
     testInvariant();
 
-    // Should we start looking from beginNonRemoved ?
-    // If I try, I get a failure in swfdec/gotoframe.swf
-    for (iterator it = _charsByDepth.begin(), itEnd = _charsByDepth.end();
-            it != itEnd; )
+    bool unloadHandler = false;
+
+    // All children with an unload handler should be unloaded. As soon as
+    // the first unload handler is encountered, subsequent children should
+    // not be destroyed or removed from the display list. This affects
+    // children without an unload handler.
+    for (iterator it = beginNonRemoved(_charsByDepth),
+            itEnd = _charsByDepth.end(); it != itEnd; )
     {
         // make a copy
-        DisplayItem di = *it;
+        DisplayObject* di = *it;
 
-        // skip if already unloaded
-        if (di->unloaded()) {
-            // TODO: call di->destroy(); ?
+        // Destroyed objects should not be there!
+        assert(!di->isDestroyed());
+
+        // Destroy those with a handler anyway?
+        if (di->unload()) {
+            unloadHandler = true;
             ++it;
             continue;
         }
 
-        if (!di->unload()) {
-            // no event handler queued, we remove
-            // will be destroyed on next iteration, or by unload
-            // handler ? we don't want soft-ref to rebind here
-            it = _charsByDepth.erase(it); 
+        if (!unloadHandler) {
+            di->destroy();
+            it = _charsByDepth.erase(it);
         }
         else ++it;
+
     }
 
     testInvariant();
 
-    return ! _charsByDepth.empty();
+    return unloadHandler;
 
 }
 
@@ -633,7 +614,7 @@ DisplayList::destroy()
             it != itEnd; ) {
 
         // make a copy
-        DisplayItem di = *it;
+        DisplayObject* di = *it;
 
         // skip if already unloaded
         if ( di->isDestroyed() ) {
@@ -662,6 +643,7 @@ DisplayList::display(Renderer& renderer)
     for (iterator endIt = _charsByDepth.end(); it != endIt; ++it)
     {
         DisplayObject* ch = *it;
+        assert(!ch->isDestroyed());
 
         DisplayObject* mask = ch->getMask();
         if (mask && ch->visible() && ! mask->unloaded())
@@ -684,7 +666,7 @@ DisplayList::display(Renderer& renderer)
         // Don't display dynamic masks
         if (ch->isDynamicMask()) continue;
 
-        assert(! ch->unloaded() ); // we don't advance unloaded chars
+        assert(!ch->unloaded()); // we don't advance unloaded chars
 
         // Check if this charater or any of its parents is a mask.
         // Characters acting as masks should always be rendered to the
@@ -754,9 +736,11 @@ DisplayList::dump() const
     for (const_iterator it = _charsByDepth.begin(),
             endIt = _charsByDepth.end(); it != endIt; ++it) {
 
-        const DisplayItem& dobj = *it;
-        log_debug(_("Item %d at depth %d (char name %s, type %s)"),
-            num, dobj->get_depth(), dobj->get_name(), typeName(*dobj));
+        const DisplayObject* dobj = *it;
+        log_debug(_("Item %d(%s) at depth %d (char name %s, type %s)"
+                    "Destroyed: %s, unloaded: %s"),
+            num, dobj, dobj->get_depth(), dobj->get_name(), typeName(*dobj),
+            dobj->isDestroyed(), dobj->unloaded());
         num++;
     }
 }
@@ -797,7 +781,7 @@ DisplayList::add_invalidated_bounds(InvalidatedRanges& ranges, bool force)
 
     iterator it = beginNonRemoved(_charsByDepth);
     for (iterator endIt = _charsByDepth.end(); it != endIt; ++it) {
-        DisplayItem& dobj = *it;
+        DisplayObject* dobj = *it;
         
 #ifndef GNASH_USE_GC
         assert(dobj->get_ref_count() > 0);
@@ -1054,6 +1038,7 @@ void
 DisplayList::reinsertRemovedCharacter(DisplayObject* ch)
 {
     assert(ch->unloaded());
+    assert(!ch->isDestroyed());
     testInvariant();
 
     // TODO: have this done by DisplayObject::unload() instead ?
@@ -1066,7 +1051,7 @@ DisplayList::reinsertRemovedCharacter(DisplayObject* ch)
         std::find_if(_charsByDepth.begin(), _charsByDepth.end(),
                 DepthGreaterOrEqual(newDepth));
 
-    _charsByDepth.insert(it, DisplayItem(ch));
+    _charsByDepth.insert(it, ch);
 
     testInvariant();
 }
@@ -1142,7 +1127,7 @@ operator<< (std::ostream& os, const DisplayList& dl)
     for (DisplayList::const_iterator it = dl._charsByDepth.begin(),
             itEnd = dl._charsByDepth.end(); it != itEnd; ++it) {
 
-        const DisplayItem& item = *it; 
+        const DisplayObject* item = *it; 
         if (it != dl._charsByDepth.begin()) os << " | ";
         os << " name:" << item->get_name()
            << " depth:" << item->get_depth();
