@@ -24,9 +24,7 @@
 #ifndef GNASH_FONT_H
 #define GNASH_FONT_H
 
-#include "smart_ptr.h" // GNASH_USE_GC
-#include "ExportableResource.h" 
-
+#include <string>
 #include <boost/scoped_ptr.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/cstdint.hpp>
@@ -34,6 +32,8 @@
 #include <vector>
 #include <map>
 
+#include "ref_counted.h"
+
 namespace gnash {
     class FreetypeGlyphsProvider;
     namespace SWF {
@@ -51,7 +51,8 @@ namespace gnash {
 class kerning_pair
 {
 public:
-    boost::uint16_t    m_char0, m_char1;
+    boost::uint16_t m_char0;
+    boost::uint16_t m_char1;
 
     bool operator==(const kerning_pair& k) const
     {
@@ -84,7 +85,9 @@ operator< (const kerning_pair& p1, const kerning_pair& p2)
 /// font simultaneously means that callers must themselves ensure they
 /// specify which font they require. Failure to do this consistently may mean
 /// callers end up with the wrong information about a font.
-class Font : public ExportableResource
+//
+/// TODO: check whether it really needs to be ref_counted.
+class Font : public ref_counted
 {
 public:
 
@@ -103,7 +106,7 @@ public:
     ///
     /// @param italic
     ///    Whether to use the italic variant of the font.
-    Font(const std::string& name, bool bold=false, bool italic=false);
+    Font(const std::string& name, bool bold = false, bool italic = false);
 
     ~Font();
 
