@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,8 +24,6 @@
 #include "StreamProvider.h" // for inlines
 #include "URL.h" // for inlines
 
-#include "log.h"
-#include "IOChannel.h"
 
 #include <string>
 #include <map>
