@@ -1,6 +1,7 @@
 // CustomActions.cpp:  ActionScript CustomActions class, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,17 +18,12 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "CustomActions.h"
 #include "as_object.h" // for inheritance
 #include "log.h"
 #include "fn_call.h"
 #include "smart_ptr.h" // for boost intrusive_ptr
 #include "builtin_function.h" // need builtin_function
-#include "Object.h" // for getObjectInterface
 #include "Global_as.h"
 
 namespace gnash {
@@ -56,11 +52,11 @@ namespace {
 void
 attachCustomActionsInterface(as_object& o)
 {
-    Global_as* gl = getGlobal(o);
-	o.init_member("get", gl->createFunction(customactions_get));
-	o.init_member("install", gl->createFunction(customactions_install));
-	o.init_member("list", gl->createFunction(customactions_list));
-	o.init_member("uninstall", gl->createFunction(customactions_uninstall));
+    Global_as& gl = getGlobal(o);
+	o.init_member("get", gl.createFunction(customactions_get));
+	o.init_member("install", gl.createFunction(customactions_install));
+	o.init_member("list", gl.createFunction(customactions_list));
+	o.init_member("uninstall", gl.createFunction(customactions_uninstall));
 }
 
 as_value
