@@ -1,6 +1,7 @@
 // Error_as.h:  ActionScript "Error" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,14 +21,10 @@
 #ifndef GNASH_ASOBJ_ERROR_H
 #define GNASH_ASOBJ_ERROR_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 namespace gnash {
 
 class as_object;
-class ObjectURI;
+struct ObjectURI;
 
 /// Initialize the global Error class
 void Error_class_init(as_object& global, const ObjectURI& uri);
