@@ -40,50 +40,31 @@
 #include "Selection_as.h"
 #include "Number_as.h"
 #include "Math_as.h"
-#include "flash/accessibility/Accessibility_as.h"
-#include "flash/ui/ContextMenu_as.h"
-#include "flash/ui/ContextMenuItem_as.h"
-#include "flash/ui/Keyboard_as.h"
-#include "flash/ui/Mouse_as.h"
-#include "flash/media/Microphone_as.h"
-#include "flash/media/Sound_as.h"
-#include "flash/media/Camera_as.h"
-#include "flash/display/InteractiveObject_as.h"
-#include "flash/display/DisplayObjectContainer_as.h"
-#include "flash/display/DisplayObject_as.h"
-#include "flash/display/Stage_as.h"
-#include "flash/display/MovieClip_as.h"
-#include "flash/display/Shape_as.h"
-#include "flash/display/Sprite_as.h"
-#include "flash/display/Bitmap_as.h"
-#include "flash/events/Event_as.h"
-#include "flash/events/EventDispatcher_as.h"
+#include "Accessibility_as.h"
+#include "ContextMenu_as.h"
+#include "ContextMenuItem_as.h"
+#include "Key_as.h"
+#include "Mouse_as.h"
+#include "Microphone_as.h"
+#include "Sound_as.h"
+#include "Camera_as.h"
+#include "Stage_as.h"
+#include "MovieClip_as.h"
 #include "flash/filters/BitmapFilter_as.h"
 #include "flash/geom/ColorTransform_as.h"
-#include "flash/net/LocalConnection_as.h"
-#include "flash/net/XMLSocket_as.h"
-#include "flash/net/SharedObject_as.h"
-#include "flash/net/NetConnection_as.h"
-#include "flash/net/NetStream_as.h"
-#include "flash/system/System_as.h"
-#include "flash/text/TextSnapshot_as.h"
-#include "flash/text/TextFieldAutoSize_as.h"
-#include "flash/text/Font_as.h"
-#include "flash/text/FontStyle_as.h"
-#include "flash/text/AntiAliasType_as.h"
-#include "flash/text/CSMSettings_as.h"
-#include "flash/text/GridFitType_as.h"
-#include "flash/text/StaticText_as.h"
-#include "flash/text/StyleSheet_as.h"
-#include "flash/text/TextColorType_as.h"
-#include "flash/text/TextDisplayMode_as.h"
-#include "flash/text/TextFieldType_as.h"
-#include "flash/text/TextFormatAlign_as.h"
-#include "flash/text/TextFormat_as.h"
-#include "flash/text/TextLineMetrics_as.h"
+#include "LocalConnection_as.h"
+#include "XMLSocket_as.h"
+#include "SharedObject_as.h"
+#include "NetConnection_as.h"
+#include "NetStream_as.h"
+#include "System_as.h"
+#include "TextSnapshot_as.h"
+#include "TextField_as.h"
+#include "TextFormat_as.h"
 #include "flash/text/TextRenderer_as.h"
-#include "flash/xml/XMLDocument_as.h"
-#include "flash/xml/XMLNode_as.h"
+#include "XML_as.h"
+#include "XMLNode_as.h"
+#include "flash/external/ExternalInterface_as.h"
 #include "MovieClipLoader.h"
 #include "movie_definition.h"
 #include "Video.h"
@@ -92,7 +73,6 @@
 #include "Timers.h"
 #include "URL.h" 
 #include "builtin_function.h"
-#include "TextField.h"
 #include "rc.h"
 #include "ClassHierarchy.h"
 #include "namedStrings.h"
@@ -101,11 +81,9 @@
 #include "fn_call.h"
 #include "Button.h"
 #include "Globals.h"
-#include "int_as.h"
 #include "LoadVars_as.h"
-#include "Namespace_as.h"
-#include "QName_as.h"
 #include "Object.h"
+#include "LoadableObject.h"
 
 #include <map>
 #include <limits> 
@@ -133,10 +111,6 @@ namespace {
 
     const ClassHierarchy::NativeClasses& avm1Classes();
 
-    // These functions are for AVM2 only.
-    const ClassHierarchy::NativeClasses& avm2Classes();
-    ObjectURI knownClass(string_table::key key);
-
     as_value global_trace(const fn_call& fn);
     as_value global_isNaN(const fn_call& fn);
     as_value global_isfinite(const fn_call& fn);
@@ -265,8 +239,8 @@ AVM1Global::markReachableResources() const
 
 AVM1Global::AVM1Global(VM& vm)
     :
+    Global_as(vm),
     _classes(this, &_et),
-    _vm(vm),
     _objectProto(new as_object(*this))
 {
 }
@@ -290,29 +264,32 @@ AVM1Global::registerClasses()
     as_value nullVal; nullVal.set_null();
     init_member("o", nullVal, PropFlags::dontEnum);
 
+
+    VM& vm = getVM();
+
     // _global functions.            
     // These functions are only available in SWF6+, but this is just
     // because SWF5 or lower did not have a "_global"
     // reference at all.
     init_member("ASnative", createFunction(global_asnative));
     init_member("ASconstructor", createFunction(global_asconstructor));
-    init_member("ASSetPropFlags", _vm.getNative(1, 0));
-    init_member("ASSetNative", _vm.getNative(4, 0));
-    init_member("ASSetNativeAccessor", _vm.getNative(4, 1));
+    init_member("ASSetPropFlags", vm.getNative(1, 0));
+    init_member("ASSetNative", vm.getNative(4, 0));
+    init_member("ASSetNativeAccessor", vm.getNative(4, 1));
     init_member("AsSetupError", createFunction(global_assetuperror));
-    init_member("updateAfterEvent", _vm.getNative(9, 0));
-    init_member("trace", _vm.getNative(100, 4));
+    init_member("updateAfterEvent", vm.getNative(9, 0));
+    init_member("trace", vm.getNative(100, 4));
 
-    init_member("setInterval", _vm.getNative(250, 0));
-    init_member("clearInterval", _vm.getNative(250, 1));
-    init_member("setTimeout", _vm.getNative(250, 2));
+    init_member("setInterval", vm.getNative(250, 0));
+    init_member("clearInterval", vm.getNative(250, 1));
+    init_member("setTimeout", vm.getNative(250, 2));
  
     // This is an odd function with no properties. There ought to be
     // a better way of implementing this. See also TextFormat.getTextExtent.
     as_function* edc = createFunction(global_enableDebugConsole);
     edc->clearProperties();
     init_member("enableDebugConsole", edc);
-    init_member("showRedrawRegions", _vm.getNative(1021, 1));
+    init_member("showRedrawRegions", vm.getNative(1021, 1));
     
     string_table& st = getStringTable(*this);
     init_member("clearTimeout", getMember(st.find("clearInterval")));
@@ -323,7 +300,7 @@ AVM1Global::registerClasses()
     const string_table::key NS_FLASH = st.find("flash");
     flash_package_init(*this, NS_FLASH); 
 
-    const int version = _vm.getSWFVersion();
+    const int version = vm.getSWFVersion();
 
     switch (version)
     {
@@ -339,12 +316,12 @@ AVM1Global::registerClasses()
         
             // This is surely not correct, but they are not available
             // in SWF4
-            init_member("escape", _vm.getNative(100, 0));
-            init_member("unescape", _vm.getNative(100, 1));
-            init_member("parseInt", _vm.getNative(100, 2));
-            init_member("parseFloat", _vm.getNative(100, 3));
-            init_member("isNaN", _vm.getNative(200, 18));
-            init_member("isFinite", _vm.getNative(200, 19));
+            init_member("escape", vm.getNative(100, 0));
+            init_member("unescape", vm.getNative(100, 1));
+            init_member("parseInt", vm.getNative(100, 2));
+            init_member("parseFloat", vm.getNative(100, 3));
+            init_member("isNaN", vm.getNative(200, 18));
+            init_member("isFinite", vm.getNative(200, 19));
 
             init_member("NaN", as_value(NaN));
             init_member("Infinity", as_value(
@@ -383,107 +360,6 @@ AVM1Global::loadExtensions()
 
 }
 
-#ifdef ENABLE_AVM2
-
-AVM2Global::AVM2Global(abc::Machine& /*machine*/, VM& vm)
-    :
-    _classes(this, 0),
-    _vm(vm),
-    _objectProto(new as_object(*this))
-{
-}
-
-void
-AVM2Global::registerClasses()
-{
-   
-    const string_table::key NS_GLOBAL(0);
-    
-    initObjectClass(_objectProto, *this,
-            ObjectURI(NSV::CLASS_OBJECT, NS_GLOBAL)); 
-
-    function_class_init(*this, ObjectURI(NSV::CLASS_FUNCTION, NS_GLOBAL));
-    string_class_init(*this, ObjectURI(NSV::CLASS_STRING, NS_GLOBAL)); 
-    array_class_init(*this, ObjectURI(NSV::CLASS_ARRAY, NS_GLOBAL)); 
-    
-    init_member("trace", createFunction(global_trace));
-    init_member("escape", createFunction(global_escape));
-
-    _classes.declareAll(avm2Classes());
-    _classes.getGlobalNs()->stubPrototype(_classes, NSV::CLASS_FUNCTION);
-    _classes.getGlobalNs()->getScript(NSV::CLASS_FUNCTION)->setDeclared();
-    _classes.getGlobalNs()->stubPrototype(_classes, NSV::CLASS_OBJECT);
-    _classes.getGlobalNs()->getScript(NSV::CLASS_OBJECT)->setDeclared();
-    _classes.getGlobalNs()->stubPrototype(_classes, NSV::CLASS_ARRAY);
-    _classes.getGlobalNs()->getScript(NSV::CLASS_ARRAY)->setDeclared();
-    _classes.getGlobalNs()->stubPrototype(_classes, NSV::CLASS_STRING);
-    _classes.getGlobalNs()->getScript(NSV::CLASS_STRING)->setDeclared();        
-
-}
-
-as_object*
-AVM2Global::createObject()
-{
-    as_object* obj = new as_object(*this);
-    obj->set_prototype(_objectProto);
-    return obj;
-}
-
-builtin_function*
-AVM2Global::createFunction(Global_as::ASFunction function)
-{
-    builtin_function* f = new builtin_function(*this, function);
-    f->init_member(NSV::PROP_CONSTRUCTOR,
-            as_function::getFunctionConstructor());
-    return f;
-}
-
-as_object*
-AVM2Global::createClass(Global_as::ASFunction ctor, as_object* prototype)
-{
-    // TODO: this should attach the function to the prototype as its
-    as_object* cl = new builtin_function(*this, ctor);
-    
-    if (prototype) {
-        prototype->init_member(NSV::PROP_CONSTRUCTOR, cl); 
-        cl->init_member(NSV::PROP_PROTOTYPE, prototype);
-    }
-    return cl;
-}
-
-as_object*
-AVM2Global::createString(const std::string& s)
-{
-    // What AVM2 does for createString is untested, so we do the same
-    // as AVM1 for now.
-    return constructObject(*this, s, NSV::CLASS_STRING);
-}
-
-as_object*
-AVM2Global::createNumber(double d)
-{
-    return constructObject(*this, d, NSV::CLASS_NUMBER);
-}
-
-as_object*
-AVM2Global::createBoolean(bool b)
-{
-    return constructObject(*this, b, NSV::CLASS_BOOLEAN);
-}
-
-/// This serves the purpose of hiding the Array_as type from the
-/// implementation, which at least enforces good behaviour from users.
-as_object*
-AVM2Global::createArray()
-{
-    as_object* array = new as_object(*this);
-    array->setArray();
-    array->init_member(NSV::PROP_CONSTRUCTOR, getMember(NSV::CLASS_ARRAY));
-    return array;
-}
-
-#endif
-
 namespace {
 
 const ClassHierarchy::NativeClasses&
@@ -497,248 +373,45 @@ avm1Classes()
     // An ObjectURI constructed without a namespace is in the global namespace.
     static const ClassHierarchy::NativeClasses s = boost::assign::list_of
 
-        (N(system_class_init, NSV::CLASS_SYSTEM, 0, 1))
-        (N(stage_class_init, NSV::CLASS_STAGE, 0, 1))
-        (N(movieclip_class_init, NSV::CLASS_MOVIE_CLIP, 0, 3))
-        (N(textfield_class_init, NSV::CLASS_TEXT_FIELD, 0, 3))
-        (N(math_class_init, NSV::CLASS_MATH, 0, 4))
-        (N(boolean_class_init, NSV::CLASS_BOOLEAN, 0, 5))
-        (N(button_class_init, NSV::CLASS_BUTTON, 0, 5))
-        (N(color_class_init, NSV::CLASS_COLOR, 0, 5))
-        (N(selection_class_init, NSV::CLASS_SELECTION, 0, 5))
-        (N(sound_class_init, NSV::CLASS_SOUND, 0, 5))
-        (N(xmlsocket_class_init, NSV::CLASS_XMLSOCKET, 0, 5))
-        (N(date_class_init, NSV::CLASS_DATE, 0, 5))
-        (N(xmlnode_class_init, NSV::CLASS_XMLNODE, 0, 5))
-        (N(xml_class_init, NSV::CLASS_XML, 0, 5))
-        (N(mouse_class_init, NSV::CLASS_MOUSE, 0, 5))
-        (N(number_class_init, NSV::CLASS_NUMBER, 0, 5))
-        (N(textformat_class_init, NSV::CLASS_TEXT_FORMAT, 0, 5))
-        (N(keyboard_class_init, NSV::CLASS_KEY, 0, 5))
-        (N(AsBroadcaster::init, NSV::CLASS_AS_BROADCASTER, 0, 5))
-        (N(textsnapshot_class_init, NSV::CLASS_TEXT_SNAPSHOT, 0, 5))
-        (N(video_class_init, NSV::CLASS_VIDEO, 0, 6))
-        (N(camera_class_init, NSV::CLASS_CAMERA, 0, 5))
-        (N(microphone_class_init, NSV::CLASS_MICROPHONE, 0, 5))
-        (N(sharedobject_class_init, NSV::CLASS_SHARED_OBJECT, 0, 5))
-        (N(loadvars_class_init, NSV::CLASS_LOAD_VARS, 0, 5))
-        (N(localconnection_class_init, NSV::CLASS_LOCALCONNECTION, 0, 6))
-        (N(customactions_class_init, NSV::CLASS_CUSTOM_ACTIONS, 0, 6))
-        (N(netconnection_class_init, NSV::CLASS_NET_CONNECTION, 0, 6))
-        (N(netstream_class_init, NSV::CLASS_NET_STREAM, 0, 6))
-        (N(contextmenu_class_init, NSV::CLASS_CONTEXTMENU, 0, 5))
-        (N(contextmenuitem_class_init, NSV::CLASS_CONTEXTMENUITEM, 0, 5))
-        (N(moviecliploader_class_init, NSV::CLASS_MOVIE_CLIP_LOADER, 0, 5))
-        (N(Error_class_init, NSV::CLASS_ERROR, 0, 5))
-        (N(accessibility_class_init, NSV::CLASS_ACCESSIBILITY, 0, 5));
+        (N(system_class_init, NSV::CLASS_SYSTEM, 1))
+        (N(stage_class_init, NSV::CLASS_STAGE, 1))
+        (N(movieclip_class_init, NSV::CLASS_MOVIE_CLIP, 3))
+        (N(textfield_class_init, NSV::CLASS_TEXT_FIELD, 3))
+        (N(math_class_init, NSV::CLASS_MATH, 4))
+        (N(boolean_class_init, NSV::CLASS_BOOLEAN, 5))
+        (N(button_class_init, NSV::CLASS_BUTTON, 5))
+        (N(color_class_init, NSV::CLASS_COLOR, 5))
+        (N(selection_class_init, NSV::CLASS_SELECTION, 5))
+        (N(sound_class_init, NSV::CLASS_SOUND, 5))
+        (N(xmlsocket_class_init, NSV::CLASS_XMLSOCKET, 5))
+        (N(date_class_init, NSV::CLASS_DATE, 5))
+        (N(xmlnode_class_init, NSV::CLASS_XMLNODE, 5))
+        (N(xml_class_init, NSV::CLASS_XML, 5))
+        (N(mouse_class_init, NSV::CLASS_MOUSE, 5))
+        (N(number_class_init, NSV::CLASS_NUMBER, 5))
+        (N(textformat_class_init, NSV::CLASS_TEXT_FORMAT, 5))
+        (N(key_class_init, NSV::CLASS_KEY, 5))
+        (N(AsBroadcaster::init, NSV::CLASS_AS_BROADCASTER, 5))
+        (N(textsnapshot_class_init, NSV::CLASS_TEXT_SNAPSHOT, 5))
+        (N(video_class_init, NSV::CLASS_VIDEO, 6))
+        (N(camera_class_init, NSV::CLASS_CAMERA, 5))
+        (N(microphone_class_init, NSV::CLASS_MICROPHONE, 5))
+        (N(sharedobject_class_init, NSV::CLASS_SHARED_OBJECT, 5))
+        (N(loadvars_class_init, NSV::CLASS_LOAD_VARS, 5))
+        (N(localconnection_class_init, NSV::CLASS_LOCALCONNECTION, 6))
+        (N(customactions_class_init, NSV::CLASS_CUSTOM_ACTIONS, 6))
+        (N(netconnection_class_init, NSV::CLASS_NET_CONNECTION, 6))
+        (N(netstream_class_init, NSV::CLASS_NET_STREAM, 6))
+        (N(contextmenu_class_init, NSV::CLASS_CONTEXTMENU, 5))
+        (N(contextmenuitem_class_init, NSV::CLASS_CONTEXTMENUITEM, 5))
+        (N(moviecliploader_class_init, NSV::CLASS_MOVIE_CLIP_LOADER, 5))
+        (N(Error_class_init, NSV::CLASS_ERROR, 5))
+        (N(accessibility_class_init, NSV::CLASS_ACCESSIBILITY, 5));
 
     return s;
 
 }
 
-#ifdef ENABLE_AVM2
-
-const ClassHierarchy::NativeClasses&
-avm2Classes()
-{
-
-    typedef ClassHierarchy::NativeClass N;
-
-    ObjectURI(*c)(string_table::key) = knownClass;
-
-    static const ClassHierarchy::NativeClasses s = boost::assign::list_of
-
-        // Global classes
-        (N(math_class_init, c(NSV::CLASS_MATH), c(NSV::CLASS_OBJECT), 4))
-        (N(boolean_class_init, c(NSV::CLASS_BOOLEAN), c(NSV::CLASS_OBJECT), 5))
-        (N(number_class_init, c(NSV::CLASS_NUMBER), c(NSV::CLASS_OBJECT), 5))
-        (N(int_class_init, c(NSV::CLASS_INT), c(NSV::CLASS_OBJECT), 5))
-        (N(namespace_class_init, c(NSV::CLASS_NAMESPACE),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(qname_class_init, c(NSV::CLASS_QNAME), c(NSV::CLASS_OBJECT), 5))
-        (N(date_class_init, c(NSV::CLASS_DATE), c(NSV::CLASS_OBJECT), 5))
-        (N(Error_class_init, c(NSV::CLASS_ERROR), c(NSV::CLASS_OBJECT), 5))
-
-        // System classes
-        (N(system_class_init, c(NSV::CLASS_SYSTEM), c(NSV::CLASS_OBJECT), 1))
-
-        // Display classes
-        (N(shape_class_init, c(NSV::CLASS_SHAPE),
-           c(NSV::CLASS_DISPLAYOBJECT), 3))
-        (N(displayobject_class_init, c(NSV::CLASS_DISPLAYOBJECT),
-           c(NSV::CLASS_EVENTDISPATCHER), 3))
-        (N(interactiveobject_class_init, c(NSV::CLASS_INTERACTIVEOBJECT),
-           c(NSV::CLASS_DISPLAYOBJECT), 3))
-        (N(displayobjectcontainer_class_init,
-           c(NSV::CLASS_DISPLAYOBJECTCONTAINER),
-           c(NSV::CLASS_INTERACTIVEOBJECT), 3))
-        (N(sprite_class_init, c(NSV::CLASS_SPRITE),
-           c(NSV::CLASS_DISPLAYOBJECTCONTAINER), 3))
-        (N(bitmap_class_init, c(NSV::CLASS_BITMAP),
-           c(NSV::CLASS_DISPLAYOBJECT), 3))
-        (N(movieclip_class_init, c(NSV::CLASS_MOVIE_CLIP),
-           c(NSV::CLASS_SPRITE), 3))
-        (N(stage_class_init, c(NSV::CLASS_STAGE),
-           c(NSV::CLASS_MOVIE_CLIP), 1))
-        (N(button_class_init, c(NSV::CLASS_SIMPLE_BUTTON),
-           c(NSV::CLASS_INTERACTIVEOBJECT), 5))
-
-        // Text classes
-        (N(textfield_class_init, c(NSV::CLASS_TEXT_FIELD),
-           c(NSV::CLASS_INTERACTIVEOBJECT), 3))
-        (N(textformat_class_init, c(NSV::CLASS_TEXT_FORMAT),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(textsnapshot_class_init, c(NSV::CLASS_TEXT_SNAPSHOT),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(textfieldautosize_class_init, c(NSV::CLASS_TEXTFIELDAUTOSIZE),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(font_class_init, c(NSV::CLASS_FONT), c(NSV::CLASS_OBJECT), 5))
-        (N(fontstyle_class_init, c(NSV::CLASS_FONTSTYLE),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(antialiastype_class_init, c(NSV::CLASS_ANTIALIASTYPE),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(csmsettings_class_init, c(NSV::CLASS_CSMTEXTSETTINGS),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(gridfittype_class_init, c(NSV::CLASS_GRIDFITTYPE),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(statictext_class_init, c(NSV::CLASS_STATICTEXT),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(stylesheet_class_init, c(NSV::CLASS_STYLESHEET),
-           c(NSV::CLASS_OBJECT), 5))
-#if 0
-        // This one isn't stubbed for some reason.
-        (N(textcolor_class_init, c(NSV::CLASS_TEXTCOLOR),
-           c(NSV::CLASS_OBJECT), 5))
-#endif
-        (N(textcolortype_class_init, c(NSV::CLASS_TEXTCOLORTYPE),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(textdisplaymode_class_init, c(NSV::CLASS_TEXTDISPLAYMODE),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(textfieldtype_class_init, c(NSV::CLASS_TEXTFIELDTYPE),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(textformatalign_class_init, c(NSV::CLASS_TEXTFORMATALIGN),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(textlinemetrics_class_init, c(NSV::CLASS_TEXTLINEMETRICS),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(textrenderer_class_init, c(NSV::CLASS_TEXTRENDERER),
-           c(NSV::CLASS_OBJECT), 5))
-
-        // Media classes
-        (N(sound_class_init, c(NSV::CLASS_SOUND), c(NSV::CLASS_OBJECT), 5))
-        (N(video_class_init, c(NSV::CLASS_VIDEO),
-           c(NSV::CLASS_DISPLAYOBJECT), 6))
-        (N(camera_class_init, c(NSV::CLASS_CAMERA), c(NSV::CLASS_OBJECT), 6))
-        (N(microphone_class_init, c(NSV::CLASS_MICROPHONE),
-           c(NSV::CLASS_OBJECT), 6))
-        // Net classes
-        (N(xmlsocket_class_init, c(NSV::CLASS_XMLSOCKET),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(sharedobject_class_init, c(NSV::CLASS_SHARED_OBJECT),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(localconnection_class_init, c(NSV::CLASS_LOCALCONNECTION),
-           c(NSV::CLASS_OBJECT), 6))
-        (N(netconnection_class_init, c(NSV::CLASS_NET_CONNECTION),
-           c(NSV::CLASS_OBJECT), 6))
-        (N(netstream_class_init, c(NSV::CLASS_NET_STREAM),
-           c(NSV::CLASS_OBJECT), 6))
-        // Error classes
-        // XML classes
-        (N(xmlnode_class_init, c(NSV::CLASS_XMLNODE),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(xml_class_init, c(NSV::CLASS_XML_DOCUMENT),
-           c(NSV::CLASS_OBJECT), 5))
-        // UI classes
-        (N(mouse_class_init, c(NSV::CLASS_MOUSE),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(keyboard_class_init, c(NSV::CLASS_KEYBOARD),
-           c(NSV::CLASS_OBJECT), 5))
-        (N(contextmenu_class_init, c(NSV::CLASS_CONTEXTMENU),
-           c(NSV::CLASS_OBJECT), 7))
-        (N(contextmenuitem_class_init, c(NSV::CLASS_CONTEXTMENUITEM),
-           c(NSV::CLASS_OBJECT), 5))
-        // Accessibility classes
-        (N(accessibility_class_init, c(NSV::CLASS_ACCESSIBILITY),
-           c(NSV::CLASS_OBJECT), 5))
-        // Event classes
-        (N(event_class_init, c(NSV::CLASS_EVENT), c(NSV::CLASS_OBJECT), 5))
-        (N(eventdispatcher_class_init, c(NSV::CLASS_EVENTDISPATCHER),
-           c(NSV::CLASS_OBJECT), 5));
-
-        return s;
-
-}
-
-ObjectURI
-knownClass(string_table::key key)
-{
-    typedef std::map<string_table::key, string_table::key> KnownClasses;
-
-    const string_table::key NS_GLOBAL(0);
-
-    static const KnownClasses knownClasses = boost::assign::map_list_of
-         (NSV::CLASS_OBJECT, NS_GLOBAL)
-         (NSV::CLASS_MATH, NS_GLOBAL)
-         (NSV::CLASS_BOOLEAN, NS_GLOBAL)
-         (NSV::CLASS_NUMBER, NS_GLOBAL)
-         (NSV::CLASS_INT, NS_GLOBAL)
-         (NSV::CLASS_NAMESPACE, NS_GLOBAL)
-         (NSV::CLASS_QNAME, NS_GLOBAL)
-         (NSV::CLASS_DATE, NS_GLOBAL)
-         (NSV::CLASS_ERROR, NS_GLOBAL)
-         (NSV::CLASS_SYSTEM, NSV::NS_FLASH_SYSTEM)
-         (NSV::CLASS_SHAPE, NSV::NS_FLASH_DISPLAY)
-         (NSV::CLASS_DISPLAYOBJECT, NSV::NS_FLASH_DISPLAY)
-         (NSV::CLASS_INTERACTIVEOBJECT, NSV::NS_FLASH_DISPLAY)
-         (NSV::CLASS_DISPLAYOBJECTCONTAINER, NSV::NS_FLASH_DISPLAY)
-         (NSV::CLASS_SPRITE, NSV::NS_FLASH_DISPLAY)
-         (NSV::CLASS_BITMAP, NSV::NS_FLASH_DISPLAY)
-         (NSV::CLASS_MOVIE_CLIP, NSV::NS_FLASH_DISPLAY)
-         (NSV::CLASS_STAGE, NSV::NS_FLASH_DISPLAY)
-         (NSV::CLASS_SIMPLE_BUTTON, NSV::NS_FLASH_DISPLAY)
-         (NSV::CLASS_TEXT_FIELD, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_TEXT_FORMAT, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_TEXT_SNAPSHOT, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_TEXTFIELDAUTOSIZE, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_FONT, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_FONTSTYLE, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_ANTIALIASTYPE, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_CSMTEXTSETTINGS, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_GRIDFITTYPE, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_STATICTEXT, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_STYLESHEET, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_TEXTCOLORTYPE, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_TEXTDISPLAYMODE, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_TEXTFIELDTYPE, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_TEXTFORMATALIGN, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_TEXTLINEMETRICS, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_TEXTRENDERER, NSV::NS_FLASH_TEXT)
-         (NSV::CLASS_SOUND, NSV::NS_FLASH_MEDIA)
-         (NSV::CLASS_VIDEO, NSV::NS_FLASH_MEDIA)
-         (NSV::CLASS_CAMERA, NSV::NS_FLASH_MEDIA)
-         (NSV::CLASS_MICROPHONE, NSV::NS_FLASH_MEDIA)
-         (NSV::CLASS_XMLSOCKET, NSV::NS_FLASH_NET)
-         (NSV::CLASS_SHARED_OBJECT, NSV::NS_FLASH_NET)
-         (NSV::CLASS_LOCALCONNECTION, NSV::NS_FLASH_NET)
-         (NSV::CLASS_NET_CONNECTION, NSV::NS_FLASH_NET)
-         (NSV::CLASS_NET_STREAM, NSV::NS_FLASH_NET)
-         (NSV::CLASS_XMLNODE, NSV::NS_FLASH_XML)
-         (NSV::CLASS_XML_DOCUMENT, NSV::NS_FLASH_XML)
-         (NSV::CLASS_MOUSE, NSV::NS_FLASH_UI)
-         (NSV::CLASS_KEYBOARD, NSV::NS_FLASH_UI)
-         (NSV::CLASS_SHAPE, NSV::NS_FLASH_UI)
-         (NSV::CLASS_CONTEXTMENU, NSV::NS_FLASH_UI)
-         (NSV::CLASS_CONTEXTMENUITEM, NSV::NS_FLASH_UI)
-         (NSV::CLASS_ACCESSIBILITY, NSV::NS_FLASH_ACCESSIBILITY)
-         (NSV::CLASS_EVENT, NSV::NS_FLASH_EVENTS)
-         (NSV::CLASS_EVENTDISPATCHER, NSV::NS_FLASH_EVENTS);
-
-    KnownClasses::const_iterator it = knownClasses.find(key);
-    assert(it != knownClasses.end());
-    return ObjectURI(key, it->second);
-}
-
-#endif
-
 as_value
 global_trace(const fn_call& fn)
 {
@@ -1010,11 +683,11 @@ as_value
 global_asconstructor(const fn_call& fn)
 {
 
-    if (fn.nargs < 2)
-    {
+    if (fn.nargs < 2) {
         IF_VERBOSE_ASCODING_ERRORS(    
-        log_aserror(_("ASNative(%s): needs at least two arguments"),
-            fn.dump_args());
+            std::ostringstream ss; fn.dump_args(ss);
+            log_aserror(_("ASNative(%s): needs at least two arguments"),
+                ss.str());
         )
         return as_value();
     }
@@ -1024,8 +697,9 @@ global_asconstructor(const fn_call& fn)
 
     if (sx < 0 || sy < 0) {
         IF_VERBOSE_ASCODING_ERRORS(    
+            std::ostringstream ss; fn.dump_args(ss);
             log_aserror(_("ASconstructor(%s): args must be 0 or above"),
-                fn.dump_args());
+                ss.str());
         )
         return as_value();
     }
@@ -1056,8 +730,9 @@ global_asnative(const fn_call& fn)
     if (fn.nargs < 2)
     {
         IF_VERBOSE_ASCODING_ERRORS(    
-        log_aserror(_("ASNative(%s): needs at least two arguments"),
-            fn.dump_args());
+            std::ostringstream ss; fn.dump_args(ss);
+            log_aserror(_("ASNative(%s): needs at least two arguments"),
+                ss.str());
         )
         return as_value();
     }
@@ -1067,8 +742,9 @@ global_asnative(const fn_call& fn)
 
     if (sx < 0 || sy < 0) {
         IF_VERBOSE_ASCODING_ERRORS(    
+            std::ostringstream ss; fn.dump_args(ss);
             log_aserror(_("ASnative(%s): args must be 0 or above"),
-                fn.dump_args());
+                ss.str());
         )
         return as_value();
     }
@@ -1562,7 +1238,7 @@ registerNatives(as_object& global)
     registerMovieClipLoaderNative(global);
     registerXMLSocketNative(global);
     registerSharedObjectNative(global);
-    registerKeyboardNative(global);
+    registerKeyNative(global);
     registerNetStreamNative(global);
     registerCameraNative(global);
     registerMicrophoneNative(global);
@@ -1571,6 +1247,7 @@ registerNatives(as_object& global)
     registerLocalConnectionNative(global);
     registerBitmapFilterNative(global);
     registerColorTransformNative(global);
+    registerExternalInterfaceNative(global);
 
     AsBroadcaster::registerNative(global);
     registerTextFormatNative(global);
