@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,7 +20,9 @@
 #ifndef GNASH_ASOBJ_MATH_H
 #define GNASH_ASOBJ_MATH_H
 
-#include "as_object.h"
+namespace gnash {
+    class as_object;
+}
 
 namespace gnash {
 
