@@ -21,14 +21,12 @@
 #ifndef GNASH_NUMBER_H
 #define GNASH_NUMBER_H
 
-#include "smart_ptr.h"
 
-#include <memory> // for auto_ptr
 
 namespace gnash {
 
 class as_object;
-class ObjectURI;
+struct ObjectURI;
 class Global_as;
 
 /// Initialize the global Number class
