@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,9 +23,10 @@
 namespace gnash {
 
 class as_object;
+struct ObjectURI;
 
 /// Initialize the global Selection class
-void selection_class_init(as_object& global);
+void selection_class_init(as_object& global, const ObjectURI& uri);
 
 /// Register native selection methods
 void registerSelectionNative(as_object& global);
