@@ -1,6 +1,7 @@
 // string.cpp:  ActionScript "String" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,11 +17,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "smart_ptr.h"
 #include "fn_call.h"
 #include "Global_as.h"
@@ -28,12 +24,10 @@
 #include "builtin_function.h" // need builtin_function
 #include "NativeFunction.h" 
 #include "log.h"
-#include "Array_as.h"
 #include "as_value.h"
 #include "GnashException.h"
 #include "movie_definition.h" 
 #include "VM.h" 
-#include "Object.h" // for getObjectInterface
 #include "namedStrings.h"
 #include "utf8.h"
 #include "String_as.h"
@@ -73,6 +67,10 @@ namespace {
 
     inline bool checkArgs(const fn_call& fn, size_t min, size_t max,
             const std::string& function);
+
+    inline int getStringVersioned(const fn_call& fn, const as_value& arg,
+            std::string& str);
+
 }
 
 String_as::String_as(const std::string& s)
@@ -86,21 +84,21 @@ registerStringNative(as_object& global)
 {
     VM& vm = getVM(global);
     vm.registerNative(string_ctor, 251, 0);
-	vm.registerNative(string_valueOf, 251, 1);
-	vm.registerNative(string_toString, 251, 2);
-	vm.registerNative(string_oldToUpper, 102, 0);
-	vm.registerNative(string_toUpperCase, 251, 3);
-	vm.registerNative(string_oldToLower, 102, 1);
-	vm.registerNative(string_toLowerCase, 251, 4);
-	vm.registerNative(string_charAt, 251, 5);
-	vm.registerNative(string_charCodeAt, 251, 6);
-	vm.registerNative(string_concat, 251, 7);
-	vm.registerNative(string_indexOf, 251, 8);
-	vm.registerNative(string_lastIndexOf, 251, 9);
-	vm.registerNative(string_slice, 251, 10);
-	vm.registerNative(string_substring, 251, 11);
-	vm.registerNative(string_split, 251, 12);
-	vm.registerNative(string_substr, 251, 13);
+    vm.registerNative(string_valueOf, 251, 1);
+    vm.registerNative(string_toString, 251, 2);
+    vm.registerNative(string_oldToUpper, 102, 0);
+    vm.registerNative(string_toUpperCase, 251, 3);
+    vm.registerNative(string_oldToLower, 102, 1);
+    vm.registerNative(string_toLowerCase, 251, 4);
+    vm.registerNative(string_charAt, 251, 5);
+    vm.registerNative(string_charCodeAt, 251, 6);
+    vm.registerNative(string_concat, 251, 7);
+    vm.registerNative(string_indexOf, 251, 8);
+    vm.registerNative(string_lastIndexOf, 251, 9);
+    vm.registerNative(string_slice, 251, 10);
+    vm.registerNative(string_substring, 251, 11);
+    vm.registerNative(string_split, 251, 12);
+    vm.registerNative(string_substr, 251, 13);
     vm.registerNative(string_fromCharCode, 251, 14);
 }
 
@@ -111,9 +109,9 @@ string_class_init(as_object& where, const ObjectURI& uri)
     // This is going to be the global String "class"/"function"
     
     VM& vm = getVM(where);
-    Global_as* gl = getGlobal(where);
+    Global_as& gl = getGlobal(where);
 
-    as_object* proto = gl->createObject();
+    as_object* proto = gl.createObject();
     as_object* cl = vm.getNative(251, 0);
     cl->init_member(NSV::PROP_PROTOTYPE, proto);
     proto->init_member(NSV::PROP_CONSTRUCTOR, cl);
@@ -123,7 +121,7 @@ string_class_init(as_object& where, const ObjectURI& uri)
     cl->init_member("fromCharCode", vm.getNative(251, 14)); 
 
     const int flags = PropFlags::dontEnum; 
-    where.init_member(getName(uri), cl, flags, getNamespace(uri));
+    where.init_member(uri, cl, flags);
 }
 
 
@@ -133,37 +131,34 @@ namespace {
 void
 attachStringInterface(as_object& o)
 {
-	VM& vm = getVM(o);
+    VM& vm = getVM(o);
 
-	o.init_member("valueOf", vm.getNative(251, 1));
-	o.init_member("toString", vm.getNative(251, 2));
-	o.init_member("toUpperCase", vm.getNative(251, 3));
-	o.init_member("toLowerCase", vm.getNative(251, 4));
-	o.init_member("charAt", vm.getNative(251, 5));
-	o.init_member("charCodeAt", vm.getNative(251, 6));
-	o.init_member("concat", vm.getNative(251, 7));
-	o.init_member("indexOf", vm.getNative(251, 8));
-	o.init_member("lastIndexOf", vm.getNative(251, 9));
-	o.init_member("slice", vm.getNative(251, 10));
-	o.init_member("substring", vm.getNative(251, 11));
-	o.init_member("split", vm.getNative(251, 12));
-	o.init_member("substr", vm.getNative(251, 13));
+    o.init_member("valueOf", vm.getNative(251, 1));
+    o.init_member("toString", vm.getNative(251, 2));
+    o.init_member("toUpperCase", vm.getNative(251, 3));
+    o.init_member("toLowerCase", vm.getNative(251, 4));
+    o.init_member("charAt", vm.getNative(251, 5));
+    o.init_member("charCodeAt", vm.getNative(251, 6));
+    o.init_member("concat", vm.getNative(251, 7));
+    o.init_member("indexOf", vm.getNative(251, 8));
+    o.init_member("lastIndexOf", vm.getNative(251, 9));
+    o.init_member("slice", vm.getNative(251, 10));
+    o.init_member("substring", vm.getNative(251, 11));
+    o.init_member("split", vm.getNative(251, 12));
+    o.init_member("substr", vm.getNative(251, 13));
 }
 
 // all the arguments will be converted to string and concatenated.
 as_value
 string_concat(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
-
-    const int swfVersion = fn.env().get_version();
-
     as_value val(fn.this_ptr);
-    
-    std::string str = val.to_string();
 
-    for (unsigned int i = 0; i < fn.nargs; i++) {
-        str += fn.arg(i).to_string_versioned(swfVersion);
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
+
+    for (size_t i = 0; i < fn.nargs; i++) {
+        str += fn.arg(i).to_string(version);
     }
 
     return as_value(str);
@@ -174,36 +169,22 @@ string_concat(const fn_call& fn)
 as_value
 string_slice(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    /// version to use is the one of the SWF containing caller code.
-    /// If callerDef is null, this calls is spontaneous (system-event?)
-    /// in which case we should research on which version should drive
-    /// behaviour.
-    /// NOTE: it is unlikely that a system event triggers string_split so
-    ///       in most cases a null callerDef means the caller forgot to 
-    ///       set the field (ie: a programmatic error)
-    if ( ! fn.callerDef )
-    {
-        log_error("No fn_call::callerDef in string_slice call");
-    }
-    const int version = fn.callerDef ? fn.callerDef->get_version() :
-        getSWFVersion(fn);
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
     if (!checkArgs(fn, 1, 2, "String.slice()")) return as_value();
 
-    size_t start = validIndex(wstr, fn.arg(0).to_int());
+    size_t start = validIndex(wstr, toInt(fn.arg(0)));
 
     size_t end = wstr.length();
 
     if (fn.nargs >= 2)
     {
-    	end = validIndex(wstr, fn.arg(1).to_int());
+        end = validIndex(wstr, toInt(fn.arg(1)));
 
     } 
 
@@ -243,34 +224,21 @@ string_slice(const fn_call& fn)
 as_value
 string_split(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    /// version to use is the one of the SWF containing caller code.
-    /// If callerDef is null, this calls is spontaneous (system-event?)
-    /// in which case we should research on which version should drive
-    /// behaviour.
-    /// NOTE: it is unlikely that a system event triggers string_split so
-    ///       in most cases a null callerDef means the caller forgot to 
-    ///       set the field (ie: a programmatic error)
-    if (!fn.callerDef ) {
-        log_error("No fn_call::callerDef in string_split call");
-    }
-
-    const int version = fn.callerDef ? fn.callerDef->get_version() :
-        getSWFVersion(fn);
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
     
     std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
-    boost::intrusive_ptr<Array_as> array(new Array_as());
+    Global_as& gl = getGlobal(fn);
+    as_object* array = gl.createArray();
 
     if (fn.nargs == 0)
     {
         // Condition 1:
-        array->push(str);
-        return as_value(array.get());
+        callMethod(array, NSV::PROP_PUSH, str);
+        return as_value(array);
     }
 
     const std::wstring& delim = utf8::decodeCanonicalString(
@@ -281,8 +249,8 @@ string_split(const fn_call& fn)
         (version >= 6 && fn.arg(0).is_undefined()))
     {
         // Condition 2:
-        array->push(str);
-        return as_value(array.get());
+        callMethod(array, NSV::PROP_PUSH, str);
+        return as_value(array);
     }
 
     size_t max = wstr.size() + 1;
@@ -292,11 +260,11 @@ string_split(const fn_call& fn)
         // SWF5
         if (fn.nargs > 1 && !fn.arg(1).is_undefined())
         {
-            int limit = fn.arg(1).to_int();
+            int limit = toInt(fn.arg(1));
             if (limit < 1)
             {
                 // Return empty array.
-                return as_value(array.get());
+                return as_value(array);
             }
             max = clamp<size_t>(limit, 0, max);
         }
@@ -305,8 +273,8 @@ string_split(const fn_call& fn)
         {
             // Condition 3 (plus a shortcut if the string itself
             // is empty).
-	        array->push(str);
-	        return as_value(array.get());            
+            callMethod(array, NSV::PROP_PUSH, str);
+            return as_value(array);            
         }
     }
     else
@@ -317,29 +285,31 @@ string_split(const fn_call& fn)
             // If the string itself is empty, SWF6 returns a 0-sized
             // array only if the delimiter is also empty. Otherwise
             // it returns an array with 1 empty element.
-            if (delimiterSize) array->push(str);
-            return as_value(array.get());
+            if (delimiterSize) callMethod(array, NSV::PROP_PUSH, str);
+            return as_value(array);
         }
 
         // If we reach this point, the string is not empty and
         // the delimiter is defined.
         if (fn.nargs > 1 && !fn.arg(1).is_undefined())
         {
-	        int limit = fn.arg(1).to_int();
-	        if (limit < 1) {
-	            // Return empty array if 
-	            return as_value(array.get());
-	        }
+            int limit = toInt(fn.arg(1));
+            if (limit < 1) {
+                // Return empty array if 
+                return as_value(array);
+            }
             max = clamp<size_t>(limit, 0, max);
         }
 
-        // If the delimiter is empty, put each DisplayObject in an
+        // If the delimiter is empty, put each character in an
         // array element.
-        if ( delim.empty() ) {
-            for (size_t i = 0, e = wstr.size(); i < e; ++i) {
-                array->push(utf8::encodeCanonicalString(wstr.substr(i, 1), version));
+        if (delim.empty()) {
+            for (size_t i = 0, e = std::min<size_t>(wstr.size(), max);
+                    i < e; ++i) {
+                callMethod(array, NSV::PROP_PUSH,
+                       utf8::encodeCanonicalString(wstr.substr(i, 1), version));
             }
-            return as_value(array.get());
+            return as_value(array);
         }
 
     }
@@ -350,16 +320,16 @@ string_split(const fn_call& fn)
     while (num < max) {
         pos = wstr.find(delim, pos);
 
-        array->push(utf8::encodeCanonicalString(
-               		wstr.substr(prevpos, pos - prevpos),
-               		version));
+        callMethod(array, NSV::PROP_PUSH, utf8::encodeCanonicalString(
+                       wstr.substr(prevpos, pos - prevpos), version));
+
         if (pos == std::wstring::npos) break;
         num++;
         prevpos = pos + delimiterSize;
         pos++;
     }
 
-    return as_value(array.get());
+    return as_value(array);
 }
 
 /// String.lastIndexOf[string[, pos]]
@@ -369,19 +339,19 @@ string_split(const fn_call& fn)
 as_value
 string_lastIndexOf(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     if (!checkArgs(fn, 1, 2, "String.lastIndexOf()")) return as_value(-1);
 
-    const std::string& toFind = fn.arg(0).to_string();
+    const std::string& toFind = fn.arg(0).to_string(version);
 
     int start = str.size();
 
     if (fn.nargs >= 2) {
-        start = fn.arg(1).to_int();
+        start = toInt(fn.arg(1));
     }
     
     if (start < 0) {
@@ -405,33 +375,31 @@ string_lastIndexOf(const fn_call& fn)
 as_value
 string_substr(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    const int version = getSWFVersion(fn);
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
     if (!checkArgs(fn, 1, 2, "String.substr()")) return as_value(str);
     
-    int start = validIndex(wstr, fn.arg(0).to_int());
+    int start = validIndex(wstr, toInt(fn.arg(0)));
 
     int num = wstr.length();
 
     if (fn.nargs >= 2 && !fn.arg(1).is_undefined())
     {
-        num = fn.arg(1).to_int();
-	    if ( num < 0 )
-	    {
-		    if ( -num <= start ) num = 0;
-		    else
-		    {
-			    num = wstr.length() + num;
-			    if ( num < 0 ) return as_value("");
-		    }
-	    }
+        num = toInt(fn.arg(1));
+        if ( num < 0 )
+        {
+            if ( -num <= start ) num = 0;
+            else
+            {
+                num = wstr.length() + num;
+                if ( num < 0 ) return as_value("");
+            }
+        }
     }
 
     return as_value(utf8::encodeCanonicalString(wstr.substr(start, num), version));
@@ -445,21 +413,21 @@ string_substr(const fn_call& fn)
 as_value
 string_substring(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    const int version = getSWFVersion(fn);
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     const std::wstring& wstr = utf8::decodeCanonicalString(str, version);
 
     if (!checkArgs(fn, 1, 2, "String.substring()")) return as_value(str);
 
-    int start = fn.arg(0).to_int();
+    const as_value& s = fn.arg(0);
+
+    int start = toInt(s);
     int end = wstr.size();
 
-    if (start < 0) {
+    if (s.is_undefined() || start < 0) {
         start = 0;
     }
 
@@ -467,8 +435,8 @@ string_substring(const fn_call& fn)
         return as_value("");
     }
 
-    if (fn.nargs >= 2) {
-        int num = fn.arg(1).to_int();
+    if (fn.nargs >= 2 && !fn.arg(1).is_undefined()) {
+        int num = toInt(fn.arg(1));
 
         if (num < 0) {
             num = 0;
@@ -497,39 +465,39 @@ string_substring(const fn_call& fn)
 as_value
 string_indexOf(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
  
     /// Do not return before this, because the toString method should always
     /// be called. (TODO: test).   
-    const std::string& str = val.to_string();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     if (!checkArgs(fn, 1, 2, "String.indexOf")) return as_value(-1);
 
-    const int version = getSWFVersion(fn);
-
     const std::wstring& wstr = utf8::decodeCanonicalString(str, version);
 
     const as_value& tfarg = fn.arg(0); // to find arg
-    const std::wstring& toFind = utf8::decodeCanonicalString(tfarg.to_string(), version);
+    const std::wstring& toFind =
+        utf8::decodeCanonicalString(tfarg.to_string(version),
+                version);
 
     size_t start = 0;
 
     if (fn.nargs >= 2)
     {
         const as_value& saval = fn.arg(1); // start arg val
-        int start_arg = saval.to_int();
+        int start_arg = toInt(saval);
         if ( start_arg > 0 ) start = (size_t) start_arg;
-	else
-	{
-		IF_VERBOSE_ASCODING_ERRORS(
-		if ( start_arg < 0 )
-		{
-			log_aserror("String.indexOf(%s, %s): second argument casts to invalid offset (%d)",
-				tfarg, saval, start_arg);
-		}
-		);
-	}
+    else
+    {
+        IF_VERBOSE_ASCODING_ERRORS(
+        if ( start_arg < 0 )
+        {
+            log_aserror("String.indexOf(%s, %s): second argument casts to invalid offset (%d)",
+                tfarg, saval, start_arg);
+        }
+        );
+    }
     }
 
     size_t pos = wstr.find(toFind, start);
@@ -556,7 +524,7 @@ string_fromCharCode(const fn_call& fn)
         for (unsigned int i = 0; i < fn.nargs; i++)
         {
             // Maximum 65535, as with all DisplayObject codes.
-            boost::uint16_t c = static_cast<boost::uint16_t>(fn.arg(i).to_int());
+            boost::uint16_t c = static_cast<boost::uint16_t>(toInt(fn.arg(i)));
             
             // If more than 255, push 'overflow' byte.
             if (c > 255)
@@ -576,7 +544,7 @@ string_fromCharCode(const fn_call& fn)
 
     for (unsigned int i = 0; i < fn.nargs; i++)
     {
-        boost::uint16_t c = static_cast<boost::uint16_t>(fn.arg(i).to_int());
+        boost::uint16_t c = static_cast<boost::uint16_t>(toInt(fn.arg(i)));
         if (c == 0) break;
         wstr.push_back(c);
     }
@@ -588,12 +556,10 @@ string_fromCharCode(const fn_call& fn)
 as_value
 string_charCodeAt(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    const int version = getSWFVersion(fn);
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     const std::wstring& wstr = utf8::decodeCanonicalString(str, version);
 
@@ -602,8 +568,8 @@ string_charCodeAt(const fn_call& fn)
             log_aserror(_("string.charCodeAt needs one argument"));
         )
         as_value rv;
-        rv.set_nan();
-        return rv;	// Same as for out-of-range arg
+        setNaN(rv);
+        return rv;    // Same as for out-of-range arg
     }
 
     IF_VERBOSE_ASCODING_ERRORS(
@@ -616,7 +582,7 @@ string_charCodeAt(const fn_call& fn)
 
     if (index >= wstr.length()) {
         as_value rv;
-        rv.set_nan();
+        setNaN(rv);
         return rv;
     }
 
@@ -626,17 +592,15 @@ string_charCodeAt(const fn_call& fn)
 as_value
 string_charAt(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    const int version = getSWFVersion(fn);
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     if (!checkArgs(fn, 1, 1, "String.charAt()")) return as_value("");
 
     // to_int() makes this safe from overflows.
-    const size_t index = static_cast<size_t>(fn.arg(0).to_int());
+    const size_t index = static_cast<size_t>(toInt(fn.arg(0)));
 
     size_t currentIndex = 0;
 
@@ -662,12 +626,12 @@ string_charAt(const fn_call& fn)
 as_value
 string_toUpperCase(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
 
-    const int version = getSWFVersion(fn);
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
-    std::wstring wstr = utf8::decodeCanonicalString(val.to_string(), version);
+    std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
     // If this is the C locale, the conversion will be wrong.
     // Most other locales are correct. FIXME: get this to
@@ -682,15 +646,32 @@ string_toUpperCase(const fn_call& fn)
         currentLocale = std::locale::classic();
     }
 
-    if (currentLocale == std::locale::classic())
-    {
+    if (currentLocale == std::locale::classic()) {
         LOG_ONCE(
             log_error(_("Your locale probably can't convert non-ascii "
             "DisplayObjects to upper case. Using a UTF8 locale may fix this."));
         );
     }
 
-    boost::to_upper(wstr, currentLocale);
+#if !defined(__HAIKU__) && !defined(__amigaos4__)
+    try {
+        boost::to_upper(wstr, currentLocale);
+    }
+    catch (std::bad_cast&)
+        {
+            // std::use_facet<wchar_t>(currentLocale) called from
+            // boost::to_upper may throw std::bad_cast
+            log_error(_("bad_cast caught"));
+            std::abort();
+        }
+#else
+    size_t l = wstr.size();
+    for (size_t i = 0; i < l; ++i) {
+        if (wstr[i] >= 'a' && wstr[i] <= 'z') {
+            wstr[i] += 'A' - 'a';
+        }
+    }
+#endif
 
     return as_value(utf8::encodeCanonicalString(wstr, version));
 
@@ -699,12 +680,12 @@ string_toUpperCase(const fn_call& fn)
 as_value
 string_toLowerCase(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const int version = getSWFVersion(fn);
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
-    std::wstring wstr = utf8::decodeCanonicalString(val.to_string(), version);
+    std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
     // If this is the C locale, the conversion will be wrong.
     // Most other locales are correct. FIXME: get this to
@@ -727,7 +708,25 @@ string_toLowerCase(const fn_call& fn)
         );
     }
 
-    boost::to_lower(wstr, currentLocale);
+#if !defined(__HAIKU__) && !defined(__amigaos4__)
+    try {
+        boost::to_lower(wstr, currentLocale);
+    }
+    catch (std::bad_cast&)
+    {
+        // std::use_facet<wchar_t>(currentLocale) called from
+        // boost::to_lower may throw std::bad_cast
+        log_error(_("bad_cast caught"));
+        std::abort();
+    }
+#else
+    size_t l = wstr.size();
+    for (size_t i = 0; i < l; ++i) {
+        if (wstr[i] >= 'A' && wstr[i] <= 'Z') {
+            wstr[i] -= 'A' - 'a';
+        }
+    }
+#endif
 
     return as_value(utf8::encodeCanonicalString(wstr, version));
 }
@@ -735,7 +734,6 @@ string_toLowerCase(const fn_call& fn)
 as_value
 string_oldToLower(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
 
     // This should use the C locale; extended DisplayObjects are
@@ -748,7 +746,6 @@ string_oldToLower(const fn_call& fn)
 as_value
 string_oldToUpper(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
 
     // This should use the C locale; extended DisplayObjects are
@@ -763,13 +760,14 @@ string_oldToUpper(const fn_call& fn)
 as_value
 string_valueOf(const fn_call& fn)
 {
-    return as_value(fn.this_ptr).to_string();
+    const int version = getSWFVersion(fn);
+    return as_value(fn.this_ptr).to_string(version);
 }
 
 as_value
 string_toString(const fn_call& fn)
 {
-    String_as* str = ensureNativeType<String_as>(fn.this_ptr);
+    String_as* str = ensure<ThisIsNative<String_as> >(fn);
     return as_value(str->value());
 }
 
@@ -777,24 +775,51 @@ string_toString(const fn_call& fn)
 as_value
 string_ctor(const fn_call& fn)
 {
-	std::string str;
-	
-	if (fn.nargs) {
-		str = fn.arg(0).to_string();
-	}
+    const int version = getSWFVersion(fn);
 
-	if (!fn.isInstantiation())
-	{
-		return as_value(str);
-	}
-	
+    std::string str;
+
+    if (fn.nargs) {
+        str = fn.arg(0).to_string(version);
+    }
+
+    if (!fn.isInstantiation())
+    {
+        return as_value(str);
+    }
+    
     as_object* obj = fn.this_ptr;
 
     obj->setRelay(new String_as(str));
     std::wstring wstr = utf8::decodeCanonicalString(str, getSWFVersion(fn));
     obj->init_member(NSV::PROP_LENGTH, wstr.size(), as_object::DefaultFlags);
 
-	return as_value();
+    return as_value();
+}
+    
+inline int
+getStringVersioned(const fn_call& fn, const as_value& val, std::string& str)
+{
+
+    /// version to use is the one of the SWF containing caller code.
+    /// If callerDef is null, this calls is spontaneous (system-event?)
+    /// in which case we should research on which version should drive
+    /// behaviour.
+    /// NOTE: it is unlikely that a system event triggers string_split so
+    ///       in most cases a null callerDef means the caller forgot to 
+    ///       set the field (ie: a programmatic error)
+    if (!fn.callerDef) {
+        log_error("No fn_call::callerDef in string function call");
+    }
+
+    const int version = fn.callerDef ? fn.callerDef->get_version() :
+        getSWFVersion(fn);
+    
+    str = val.to_string(version);
+
+    return version;
+
+
 }
 
 /// Check the number of arguments, returning false if there
