@@ -1,52 +1,56 @@
-// XMLNode_as.cpp:  ActionScript "XMLNode_as" class, for Gnash.
-// 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
-// 
+// XMLNode_as.cpp:  ActionScript "XMLNode" class, for Gnash.
+//
+//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+//
 
-#include "smart_ptr.h" // GNASH_USE_GC
+
+#include "XMLNode_as.h"
+#include "XML_as.h"
+#include "VM.h"
 #include "log.h"
 #include "fn_call.h"
-#include "builtin_function.h"
-#include "Array_as.h" // for childNodes
-#include "XMLNode_as.h"
-#include "XML_as.h" // escape / unescape
-#include "log.h"
-#include "Object.h" // for getObjectInterface
-#include "VM.h" // for getting the string_table.
-#include "string_table.h" 
+#include "Global_as.h"
+#include "smart_ptr.h" // for boost intrusive_ptr
+#include "builtin_function.h" // need builtin_function
+#include "NativeFunction.h"
+#include "string_table.h"
 #include "PropertyList.h"
+#include "Global_as.h"
+#include "Object.h"
+#include "Array_as.h"
 
+#include "namedStrings.h"
 #include <boost/bind.hpp>
 #include <string>
 #include <sstream>
 #include <vector>
 #include <algorithm>
 
-//#define DEBUG_MEMORY_ALLOCATION 1
-
 namespace gnash {
 
+// Function Prototypes
 namespace {
     void enumerateAttributes(const XMLNode_as& node,
-            PropertyList::SortedPropertyList&attributes);
-    bool prefixMatches(const PropertyList::SortedPropertyList::value_type& val,
+            as_object::SortedPropertyList& attributes);
+    bool prefixMatches(const as_object::SortedPropertyList::value_type& val,
             const std::string& prefix);
     bool namespaceMatches(
-            const PropertyList::SortedPropertyList::value_type& val,
-            const std::string& ns);
+            const as_object::SortedPropertyList::value_type& val,
+            const std::string& ns);    
 
     as_value xmlnode_new(const fn_call& fn);
     as_value xmlnode_nodeName(const fn_call& fn);
@@ -55,215 +59,226 @@ namespace {
     as_value xmlnode_attributes(const fn_call& fn);
     as_value xmlnode_appendChild(const fn_call& fn);
     as_value xmlnode_cloneNode(const fn_call& fn);
+    as_value xmlnode_lastChild(const fn_call& fn);
+    as_value xmlnode_firstChild(const fn_call& fn);
+    as_value xmlnode_nextSibling(const fn_call& fn);
+    as_value xmlnode_childNodes(const fn_call& fn);
+    as_value xmlnode_previousSibling(const fn_call& fn);
+    as_value xmlnode_parentNode(const fn_call& fn);
+    as_value xmlnode_getNamespaceForPrefix(const fn_call& fn);
+    as_value xmlnode_getPrefixForNamespace(const fn_call& fn);
+    as_value xmlnode_namespaceURI(const fn_call& fn);
     as_value xmlnode_hasChildNodes(const fn_call& fn);
     as_value xmlnode_insertBefore(const fn_call& fn);
     as_value xmlnode_removeNode(const fn_call& fn);
     as_value xmlnode_toString(const fn_call& fn);
-    as_value xmlnode_firstChild(const fn_call& fn);
-    as_value xmlnode_lastChild(const fn_call& fn);
-    as_value xmlnode_nextSibling(const fn_call& fn);
-    as_value xmlnode_previousSibling(const fn_call& fn);
-    as_value xmlnode_childNodes(const fn_call& fn);
-    as_value xmlnode_parentNode(const fn_call& fn);
-    as_value xmlnode_getPrefixForNamespace(const fn_call& fn);
-    as_value xmlnode_getNamespaceForPrefix(const fn_call& fn);
-    as_value xmlnode_namespaceURI(const fn_call& fn);
     as_value xmlnode_localName(const fn_call& fn);
     as_value xmlnode_prefix(const fn_call& fn);
-
     void attachXMLNodeInterface(as_object& o);
+    void attachXMLNodeStaticInterface(as_object& o);
 }
 
-
-XMLNode_as::XMLNode_as()
+XMLNode_as::XMLNode_as(Global_as& gl)
     :
-    as_object(getXMLNodeInterface()),
+    _global(gl),
+    _object(0),
     _parent(0),
-    _attributes(new as_object),
+    _attributes(new as_object(gl)),
+    _childNodes(0),
     _type(Element)
 {
-    //log_debug("%s: %p", __PRETTY_FUNCTION__, this);
-#ifdef DEBUG_MEMORY_ALLOCATION
-    log_debug(_("\tCreating XMLNode data at %p"), this);
-#endif
 }
 
 XMLNode_as::XMLNode_as(const XMLNode_as& tpl, bool deep)
     :
-    as_object(getXMLNodeInterface()),
-    _parent(0), // _parent is never implicitly copied
-    _attributes(0),
+    _global(tpl._global),
+    _object(0),
+    _parent(0), 
+    _attributes(new as_object(_global)),
+    _childNodes(0),
     _name(tpl._name),
     _value(tpl._value),
     _type(tpl._type)
 {
     // only clone children if in deep mode
-    if ( deep ) 
-    {
+    if (deep) {
         const Children& from=tpl._children;
         for (Children::const_iterator it=from.begin(), itEnd=from.end();
-                        it != itEnd; ++it)
-        {
-                _children.push_back(new XMLNode_as(*(*it), deep));
+                        it != itEnd; ++it) {
+            _children.push_back(new XMLNode_as(*(*it), deep));
         }
     }
 }
 
 XMLNode_as::~XMLNode_as()
 {
-    //log_debug("%s: %p", __PRETTY_FUNCTION__, this);
-#ifdef DEBUG_MEMORY_ALLOCATION
-    log_debug(_("\tDeleting XMLNode data %s with as_value %s at %p"),
-            this->_name, this->as_value, this);
-#endif
+    clearChildren();
+}
+
+as_object*
+XMLNode_as::object() 
+{
+
+    // This is almost the same as if the XMLNode constructor were called,
+    // but not quite. There is no __constructor__ property, and when we
+    // override _global.XMLNode, we can show that it is not called.
+    if (!_object) {
+        as_object* o = _global.createObject();
+        as_object* xn =
+            _global.getMember(NSV::CLASS_XMLNODE).to_object(_global);
+        if (xn) {
+            o->set_prototype(xn->getMember(NSV::PROP_PROTOTYPE));
+            o->init_member(NSV::PROP_CONSTRUCTOR, xn);
+        }
+        o->setRelay(this);
+        setObject(o);
+    }
+    return _object;
+}
+
+void
+XMLNode_as::updateChildNodes()
+{
+    if (!_childNodes) return;
+
+    // Clear array of all elements.
+    _childNodes->set_member(NSV::PROP_LENGTH, 0.0);
+
+    if (_children.empty()) return;
+
+    string_table& st = getStringTable(_global);
+
+    // Set up the array without calling push()!
+    const size_t size = _children.size();
+    Children::const_iterator it = _children.begin();
+    for (size_t i = 0; i != size; ++i, ++it) {
+        XMLNode_as* node = *it;
+        const string_table::key key = arrayKey(st, i);
+        _childNodes->set_member(key, node->object());
+
+        // All elements are set to readonly.
+        _childNodes->set_member_flags(key, PropFlags::readOnly);
+    }
+}
+
+as_object*
+XMLNode_as::childNodes()
+{
+    if (!_childNodes) {
+        _childNodes = _global.createArray();
+        updateChildNodes();
+    }
+    return _childNodes;
 }
 
 bool
 XMLNode_as::hasChildNodes()
 {
-    //GNASH_REPORT_FUNCTION;
-    if (_children.size()) {
-        return true;
-    }
-    return false;
+    return !_children.empty();
 }
 
-boost::intrusive_ptr<XMLNode_as>
+XMLNode_as*
 XMLNode_as::firstChild()
 {
-    //GNASH_REPORT_FUNCTION;
-    if ( _children.empty() ) return NULL;
+    if (_children.empty()) return 0;
     return _children.front();
 }
 
-boost::intrusive_ptr<XMLNode_as>
+XMLNode_as*
+XMLNode_as::cloneNode(bool deep)
+{
+    XMLNode_as* newnode = new XMLNode_as(*this, deep);
+    return newnode;
+}
+
+XMLNode_as*
 XMLNode_as::lastChild()
 {
-	//GNASH_REPORT_FUNCTION;
-	if ( _children.empty() )
-	{
+	if (_children.empty()) {
 			log_debug(_("XMLNode_as %p has no children"), (void*)this);
-			return NULL;
+			return 0;
 	}
 	return _children.back();
 }
 
+void
+XMLNode_as::removeChild(XMLNode_as* node)
+{
+    node->setParent(0);
+    _children.remove(node);
+    updateChildNodes();
+}
 
 void
-XMLNode_as::appendChild(boost::intrusive_ptr<XMLNode_as> node)
+XMLNode_as::appendChild(XMLNode_as* node)
 {
-    assert (node);
-
-    boost::intrusive_ptr<XMLNode_as> oldparent = node->getParent();
+    assert(node);
     node->setParent(this);
     _children.push_back(node);
-    if ( oldparent ) {
-        oldparent->_children.remove(node);
-    }
-
-}
-
-boost::intrusive_ptr<XMLNode_as> 
-XMLNode_as::cloneNode(bool deep)
-{
-    boost::intrusive_ptr<XMLNode_as> newnode = new XMLNode_as(*this, deep);
-    return newnode;
+    updateChildNodes();
 }
 
 void
-XMLNode_as::insertBefore(boost::intrusive_ptr<XMLNode_as> newnode,
-        boost::intrusive_ptr<XMLNode_as> pos)
+XMLNode_as::insertBefore(XMLNode_as* newnode, XMLNode_as* pos)
 {
-    // find iterator for positional parameter
+    assert(_object);
+
+	// find iterator for positional parameter
     Children::iterator it = std::find(_children.begin(), _children.end(), pos);
-    if ( it == _children.end() )
-    {
+    if (it == _children.end()) {
         IF_VERBOSE_ASCODING_ERRORS(
-        log_aserror(_("XMLNode.insertBefore(): positional parameter is not a child of this node"));
+        log_aserror(_("XMLNode.insertBefore(): positional parameter "
+                "is not a child of this node"));
         );
         return;
     }
 
     _children.insert(it, newnode);
-    boost::intrusive_ptr<XMLNode_as> oldparent = newnode->getParent();
-    newnode->setParent(this);
-    if ( oldparent )
-    {
-        oldparent->_children.remove(newnode);
-    }
-}
 
-/// \brief removes the specified XML object from its parent. Also
-/// deletes all descendants of the node.
-void
-XMLNode_as::removeNode()
-{
-#ifndef GNASH_USE_GC
-    assert(get_ref_count() > 1);
-#endif
-    boost::intrusive_ptr<XMLNode_as> oldparent = getParent();
-    if ( oldparent )
-    {
-        oldparent->_children.remove(this);
+    XMLNode_as* parent = newnode->getParent();
+    if (parent) {
+        parent->removeChild(newnode);
     }
-    _parent = NULL;
-#ifndef GNASH_USE_GC
-    assert(get_ref_count() > 0);
-#endif
+    
+    newnode->setParent(this);
+    updateChildNodes();
 }
 
 XMLNode_as *
 XMLNode_as::previousSibling()
 {
-    //GNASH_REPORT_FUNCTION;
+    if (!_parent) return 0;
+ 	if (_parent->_children.size() <= 1) return 0;
 
-    if ( ! _parent) return NULL;
- 	if (_parent->_children.size() <= 1) return NULL;
+    XMLNode_as *previous_node = 0;
+    for (Children::iterator itx = _parent->_children.begin();
+            itx != _parent->_children.end(); itx++) {
 
-    XMLNode_as *previous_node = NULL;
-    Children::iterator itx;
-    for (itx = _parent->_children.begin(); itx != _parent->_children.end(); itx++)
-    {
-        if (itx->get() == this)
-        {
-            // log_debug("Found the previous XMLNode_as child !!!! %s <%p>", (*itx)->nodeName(), (void*)*itx);
-		    return previous_node;
-		}
-		previous_node = itx->get();
+        if (*itx == this) return previous_node;
+		
+        previous_node = *itx;
     }
 
-    return NULL;
+    return 0;
 }
 
 XMLNode_as *
 XMLNode_as::nextSibling()
 {
-    //GNASH_REPORT_FUNCTION;
 
-    if ( ! _parent)
-    {
-            //log_debug("Node %p has no parent, returning NULL", this);
-            return NULL;
-    }
-    if (_parent->_children.size() <= 1)
-    {
-            //log_debug("Node %p parent has only this node, returning NULL", this);
-            return NULL;
+    if (!_parent) return 0;
+
+    if (_parent->_children.size() <= 1) return 0;
+
+    XMLNode_as *previous_node = 0;
+    for (Children::reverse_iterator itx = _parent->_children.rbegin();
+            itx != _parent->_children.rend(); ++itx) {
+
+        if (*itx == this) return previous_node;
+		previous_node = *itx;
     }
 
-    XMLNode_as *previous_node = NULL;
-    Children::reverse_iterator itx;
-    for (itx = _parent->_children.rbegin(); itx != _parent->_children.rend();
-            itx++)
-    {
-        if (itx->get() == this)
-        {
-		    return previous_node;
-		}
-		previous_node = itx->get();
-    }
-
-    return NULL;
+    return 0;
 }
 
 void
@@ -276,7 +291,7 @@ void
 XMLNode_as::setAttribute(const std::string& name, const std::string& value)
 {
     if (_attributes) {
-        string_table& st = _vm.getStringTable();
+        string_table& st = getStringTable(_global);
         _attributes->set_member(st.find(name), value);
     }
 }
@@ -285,11 +300,10 @@ bool
 XMLNode_as::getPrefixForNamespace(const std::string& ns, std::string& prefix)
 {
     XMLNode_as* node = this;
-    PropertyList::SortedPropertyList::const_iterator it; 
-    PropertyList::SortedPropertyList attrs;
+    as_object::SortedPropertyList::const_iterator it; 
+    as_object::SortedPropertyList attrs;
     
-    while (node)
-    {
+    while (node) {
         enumerateAttributes(*node, attrs);
         if (!attrs.empty())
         {
@@ -316,22 +330,22 @@ XMLNode_as::getPrefixForNamespace(const std::string& ns, std::string& prefix)
 
     // Can also be empty.
     prefix = name.substr(6);
-    return true;
-    
+    return true;    
 }
 
 void
 XMLNode_as::getNamespaceForPrefix(const std::string& prefix, std::string& ns)
 {
     XMLNode_as* node = this;
-    PropertyList::SortedPropertyList::const_iterator it; 
-    PropertyList::SortedPropertyList attrs;
+    as_object::SortedPropertyList::const_iterator it; 
+    as_object::SortedPropertyList attrs;
     
-    while (node)
-    {
+    while (node) {
+
         enumerateAttributes(*node, attrs);
-        if (!attrs.empty())
-        {
+
+        if (!attrs.empty()) {
+
             it = std::find_if(attrs.begin(), attrs.end(), 
                         boost::bind(prefixMatches, _1, prefix));
             if (it != attrs.end()) break;
@@ -347,7 +361,6 @@ XMLNode_as::getNamespaceForPrefix(const std::string& prefix, std::string& ns)
 
 }
 
-
 bool
 XMLNode_as::extractPrefix(std::string& prefix)
 {
@@ -363,6 +376,21 @@ XMLNode_as::extractPrefix(std::string& prefix)
     return true;
 }
 
+void
+XMLNode_as::clearChildren()
+{
+    for (Children::const_iterator it = _children.begin(), e = _children.end();
+            it != e; ++it) {
+        const XMLNode_as* node = *it;
+        if (!node->_object) {
+            delete node;
+        }
+    }
+    _children.clear();
+
+    // Reset so that it is reinitialized on next access.
+    _childNodes = 0;
+}
 
 void
 XMLNode_as::stringify(const XMLNode_as& xml, std::ostream& xmlout, bool encode) 
@@ -379,24 +407,24 @@ XMLNode_as::stringify(const XMLNode_as& xml, std::ostream& xmlout, bool encode)
 #endif
 
     // Create the beginning of the tag
-    if (!nodeName.empty())
-    {
+    if (!nodeName.empty()) {
+
         xmlout << "<" << nodeName;
     
         // Process the attributes, if any
-        PropertyList::SortedPropertyList attrs;
+        as_object::SortedPropertyList attrs;
         enumerateAttributes(xml, attrs);
         if (!attrs.empty()) {
 
-            for (PropertyList::SortedPropertyList::iterator i = 
+            for (as_object::SortedPropertyList::iterator i = 
                     attrs.begin(), e = attrs.end(); i != e; ++i) { 
-                XML_as::escape(i->second);
+                escapeXML(i->second);
                 xmlout << " " << i->first << "=\"" << i->second << "\"";
             }
         }
 
     	// If the node has no content, just close the tag now
-    	if ( nodeValue.empty() && xml._children.empty() ) {
+    	if (nodeValue.empty() && xml._children.empty()) {
     		xmlout << " />";
             return;
     	}
@@ -408,101 +436,80 @@ XMLNode_as::stringify(const XMLNode_as& xml, std::ostream& xmlout, bool encode)
     }
 
     // Node as_value first, then children
-    if ( type == Text )
+    if (type == Text)
     {
+        Global_as& gl = xml._global;
+
         // Insert entities.
         std::string escaped(nodeValue);
-        XML_as::escape(escaped);
-        if (encode) URL::encode(escaped);
+        escapeXML(escaped);
+        const std::string& val = encode ? 
+            callMethod(&gl, NSV::PROP_ESCAPE, escaped).to_string() :
+            escaped;
 
-	    xmlout << escaped;
+	    xmlout << val;
     }
 
     // Childs, after node as_value.
-    Children::const_iterator itx;
-    for (itx = xml._children.begin(); itx != xml._children.end(); itx++)
-    {
+    for (Children::const_iterator itx = xml._children.begin(); 
+            itx != xml._children.end(); ++itx) {
+
         (*itx)->toString(xmlout, encode);
     }
 
-    if (!nodeName.empty())
-    {
+    if (!nodeName.empty()) {
 	    xmlout << "</" << nodeName << ">";
     }
 }
 
-
-#ifdef GNASH_USE_GC
 void
-XMLNode_as::markReachableResources() const
+XMLNode_as::setReachable() 
 {
-	// Mark childs
-	for (Children::const_iterator i=_children.begin(),
-            e=_children.end(); i!=e; ++i)
-	{
-		(*i)->setReachable();
-	}
-
-	// Mark parent
-	if ( _parent ) _parent->setReachable();
+	// Mark children
+    std::for_each(_children.begin(), _children.end(),
+            boost::mem_fn(&XMLNode_as::setReachable));
 
 	// Mark attributes object
 	if (_attributes) _attributes->setReachable();
 
-	markAsObjectReachable();
+    if (_object) _object->setReachable();
+
+    if (_childNodes) _childNodes->setReachable();
 }
-#endif // GNASH_USE_GC
 
 void
-XMLNode_as::registerNative(as_object& global)
+registerXMLNodeNative(as_object& where)
 {
-    VM& vm = global.getVM();
+    VM& vm = getVM(where);
     vm.registerNative(xmlnode_cloneNode, 253, 1);
     vm.registerNative(xmlnode_removeNode, 253, 2);
     vm.registerNative(xmlnode_insertBefore, 253, 3);
     vm.registerNative(xmlnode_appendChild, 253, 4);
     vm.registerNative(xmlnode_hasChildNodes, 253, 5);
     vm.registerNative(xmlnode_toString, 253, 6);
+    vm.registerNative(xmlnode_getNamespaceForPrefix, 253, 7);
+    vm.registerNative(xmlnode_getPrefixForNamespace, 253, 8);
 }
 
-as_object*
-XMLNode_as::getXMLNodeInterface()
-{
-    static boost::intrusive_ptr<as_object> o;
-    if ( o == NULL ) {
-        o = new as_object(getObjectInterface());
-        attachXMLNodeInterface(*o);
-    }
-    return o.get();
-}
-
-
 void
-XMLNode_as::init(as_object& global)
+xmlnode_class_init(as_object& where, const ObjectURI& uri)
 {
-    // This is the global XMLNode_as "class"
-    static boost::intrusive_ptr<builtin_function> cl;
+    Global_as& gl = getGlobal(where);
+    as_object* proto = gl.createObject();
+    attachXMLNodeInterface(*proto);
+    as_object* cl = gl.createClass(&xmlnode_new, proto);
 
-    if ( cl == NULL )
-    {
-        cl=new builtin_function(&xmlnode_new, getXMLNodeInterface());
-    }
-
-    global.init_member("XMLNode", cl.get());
+    where.init_member(uri, cl, as_object::DefaultFlags);
 
 }
 
-
-// XMLNode interface implementation
-
 namespace {
 
 void
 attachXMLNodeInterface(as_object& o)
 {
-    // These need to be full-featured AS functions (builtin_function)
     
-    VM& vm = o.getVM();
+    VM& vm = getVM(o);
 
     const int noFlags = 0;
     
@@ -513,13 +520,10 @@ attachXMLNodeInterface(as_object& o)
     o.init_member("appendChild", vm.getNative(253, 4), noFlags);
     o.init_member("hasChildNodes", vm.getNative(253, 5), noFlags);
     o.init_member("toString", vm.getNative(253, 6), noFlags);
-    o.init_member("getPrefixForNamespace", new builtin_function(
-                xmlnode_getPrefixForNamespace), noFlags);
-    o.init_member("getNamespaceForPrefix", new builtin_function(
-                xmlnode_getNamespaceForPrefix), noFlags);
+    o.init_member("getNamespaceForPrefix", vm.getNative(253, 7), noFlags);
+    o.init_member("getPrefixForNamespace", vm.getNative(253, 8), noFlags);
 
-
-    const int protectedFlags = as_prop_flags::isProtected;
+    const int protectedFlags = 0;
 
     // Just the protected flag:
     o.init_property("nodeValue", &xmlnode_nodeValue, 
@@ -549,69 +553,86 @@ as_value
 xmlnode_new(const fn_call& fn)
 {
     
-    XMLNode_as *xml_obj = new XMLNode_as;
-    if ( fn.nargs > 0 )
-    {
-        xml_obj->nodeTypeSet(XMLNode_as::NodeType(fn.arg(0).to_int()));
-        if (fn.nargs > 1)
+    as_object* obj = ensure<ValidThis>(fn);
+
+    if (!fn.nargs) {
+        return as_value();
+    }
+
+    std::auto_ptr<XMLNode_as> xml(new XMLNode_as(getGlobal(fn)));
+    xml->nodeTypeSet(XMLNode_as::NodeType(toInt(fn.arg(0))));
+
+    if (fn.nargs > 1) {
+        const std::string& str = fn.arg(1).to_string();
+        switch (xml->nodeType())
         {
-            xml_obj->nodeValueSet(fn.arg(1).to_string());
+            case XMLNode_as::Element:
+                xml->nodeNameSet(str);
+                break;
+            default:
+                xml->nodeValueSet(str);
+                break;
         }
     }
     
-    return as_value(xml_obj);
+    // This sets the relay!
+    xml->setObject(obj);
+    obj->setRelay(xml.release());
+
+    return as_value();
 }
 
 
 as_value
 xmlnode_appendChild(const fn_call& fn)
 {
-//    GNASH_REPORT_FUNCTION;
 
-	boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+	XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
-	if ( ! fn.nargs )
-	{
+	if (!fn.nargs) {
 		IF_VERBOSE_ASCODING_ERRORS(
-		log_aserror(_("XMLNode::appendChild() needs at least one argument"));
+            log_aserror(_("XMLNode::appendChild() needs at least one "
+                    "argument"));
 		);
 		return as_value();
 	}
 
-	boost::intrusive_ptr<XMLNode_as> xml_obj = 
-        boost::dynamic_pointer_cast<XMLNode_as>(fn.arg(0).to_object());	
-	if ( ! xml_obj )
-	{
+	XMLNode_as* node;
+    if (!isNativeType(fn.arg(0).to_object(getGlobal(fn)), node)) {
 		IF_VERBOSE_ASCODING_ERRORS(
-		log_aserror(_("First argument to XMLNode::appendChild() is not "
-                "an XMLNode"));
+            log_aserror(_("First argument to XMLNode::appendChild() is not "
+                    "an XMLNode"));
 		);
 		return as_value();
 	}
 
-	ptr->appendChild(xml_obj);
-	return as_value(); // undefined
+    XMLNode_as* parent = node->getParent();
+    if (parent) {
+        parent->removeChild(node);
+    }
+	ptr->appendChild(node);
+
+	return as_value(); 
 
 }
 
-
 as_value
 xmlnode_cloneNode(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
     bool deep = false;
     if (fn.nargs > 0) deep = fn.arg(0).to_bool();
 
-    boost::intrusive_ptr<XMLNode_as> newnode = ptr->cloneNode(deep);
-    return as_value(newnode.get());
+    as_object* newnode = ptr->cloneNode(deep)->object();
+    return as_value(newnode);
 }
 
 
 as_value
 xmlnode_insertBefore(const fn_call& fn)
 {
-	boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+	XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
 	if ( fn.nargs < 2 )
 	{
@@ -623,10 +644,9 @@ xmlnode_insertBefore(const fn_call& fn)
 		return as_value();
 	}
 
-	boost::intrusive_ptr<XMLNode_as> newnode = 
-        boost::dynamic_pointer_cast<XMLNode_as>(fn.arg(0).to_object());
-	if ( ! newnode )
-	{
+	XMLNode_as* newnode;
+
+    if (!isNativeType(fn.arg(0).to_object(getGlobal(fn)), newnode)) {
 		IF_VERBOSE_ASCODING_ERRORS(
 		std::stringstream ss; fn.dump_args(ss);
 		log_aserror(_("First argument to XMLNode.insertBefore(%s) is not "
@@ -635,10 +655,9 @@ xmlnode_insertBefore(const fn_call& fn)
 		return as_value();
 	}
 
-	boost::intrusive_ptr<XMLNode_as> pos = 
-        boost::dynamic_pointer_cast<XMLNode_as>(fn.arg(1).to_object());
-	if ( ! pos )
-	{
+	XMLNode_as* pos;
+
+    if (!isNativeType(fn.arg(1).to_object(getGlobal(fn)), pos)) {
 		IF_VERBOSE_ASCODING_ERRORS(
         std::stringstream ss; fn.dump_args(ss);
 		log_aserror(_("Second argument to XMLNode.insertBefore(%s) is not "
@@ -656,7 +675,7 @@ xmlnode_insertBefore(const fn_call& fn)
 as_value
 xmlnode_getNamespaceForPrefix(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     if (!fn.nargs) {
         return as_value();
     }
@@ -672,7 +691,7 @@ xmlnode_getNamespaceForPrefix(const fn_call& fn)
 as_value
 xmlnode_getPrefixForNamespace(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     if (!fn.nargs) {
         return as_value();
     }
@@ -699,7 +718,7 @@ xmlnode_getPrefixForNamespace(const fn_call& fn)
 as_value
 xmlnode_namespaceURI(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
     // Read-only property
     
@@ -720,7 +739,7 @@ xmlnode_namespaceURI(const fn_call& fn)
 
     // Search recursively for a namespace. Return an empty string
     // if none found.
-    XMLNode_as* node = ptr.get();
+    XMLNode_as* node = ptr;
     while (node && node->getNamespaceURI().empty()) {
         node = node->getParent();
     }
@@ -736,7 +755,7 @@ xmlnode_namespaceURI(const fn_call& fn)
 as_value
 xmlnode_prefix(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
     // Read-only property
     
@@ -758,7 +777,7 @@ xmlnode_prefix(const fn_call& fn)
 as_value
 xmlnode_localName(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
     // Read-only property
     
@@ -783,20 +802,19 @@ xmlnode_localName(const fn_call& fn)
 as_value
 xmlnode_removeNode(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     
-    ptr->removeNode();
+    XMLNode_as* parent = ptr->getParent();
+    if (parent) parent->removeChild(ptr);
     return as_value();
 }
 
 
-// TODO: shouldn't overriding get_text_value() be fine ?
 as_value
 xmlnode_toString(const fn_call& fn)
 {
-    //GNASH_REPORT_FUNCTION;
     
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     
     std::stringstream ss;
     ptr->toString(ss);
@@ -808,7 +826,7 @@ xmlnode_toString(const fn_call& fn)
 as_value
 xmlnode_hasChildNodes(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     return as_value(ptr->hasChildNodes());
 }
 
@@ -816,7 +834,7 @@ xmlnode_hasChildNodes(const fn_call& fn)
 as_value
 xmlnode_nodeValue(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     as_value rv;
     rv.set_null();
     
@@ -836,7 +854,7 @@ xmlnode_nodeValue(const fn_call& fn)
 as_value
 xmlnode_nodeName(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     as_value rv;
     rv.set_null();
 
@@ -844,8 +862,7 @@ xmlnode_nodeName(const fn_call& fn)
         const std::string& val = ptr->nodeName();
         if ( ! val.empty() ) rv = val;
     }
-    else
-    {
+    else {
         ptr->nodeNameSet(fn.arg(0).to_string());
     }
     return rv;
@@ -855,7 +872,7 @@ xmlnode_nodeName(const fn_call& fn)
 as_value
 xmlnode_nodeType(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     return as_value(ptr->nodeType());
 }
 
@@ -863,7 +880,7 @@ xmlnode_nodeType(const fn_call& fn)
 as_value
 xmlnode_attributes(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
     as_object* attrs = ptr->getAttributes();
     if (attrs) return as_value(attrs);
@@ -882,13 +899,13 @@ xmlnode_attributes(const fn_call& fn)
 as_value
 xmlnode_firstChild(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     as_value rv;
     rv.set_null();
 
-    boost::intrusive_ptr<XMLNode_as> node = ptr->firstChild();
+    XMLNode_as* node = ptr->firstChild();
     if (node) {
-       rv = node.get();
+        rv = node->object();
     }
 
     return rv;
@@ -904,12 +921,14 @@ xmlnode_firstChild(const fn_call& fn)
 as_value
 xmlnode_lastChild(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     as_value rv;
     rv.set_null();
 
-    boost::intrusive_ptr<XMLNode_as> node = ptr->lastChild();
-    if (node) rv = node.get();
+    XMLNode_as* node = ptr->lastChild();
+    if (node) {
+        rv = node->object();
+    }
 
     return rv;
 }
@@ -921,10 +940,10 @@ xmlnode_nextSibling(const fn_call& fn)
     as_value rv;
     rv.set_null();
 
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     XMLNode_as *node = ptr->nextSibling();
     if (node) {
-	rv = node;
+        rv = node->object();
     }
     return rv;
 }
@@ -936,10 +955,10 @@ xmlnode_previousSibling(const fn_call& fn)
     as_value rv;
     rv.set_null();
 
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     XMLNode_as *node = ptr->previousSibling();
     if (node) {
-	rv = node;
+        rv = node->object();
     }
     return rv;
 }
@@ -951,43 +970,30 @@ xmlnode_parentNode(const fn_call& fn)
     as_value rv;
     rv.set_null();
 
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     XMLNode_as *node = ptr->getParent();
     if (node) {
-	rv = node;
+        rv = node->object();
     }
     return rv;
 }
 
-
 as_value
 xmlnode_childNodes(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
-    boost::intrusive_ptr<Array_as> ary = new Array_as();
-
-    typedef XMLNode_as::Children Children;
-
-    Children& child = ptr->childNodes();
-    for ( Children::const_iterator it=child.begin(), itEnd=child.end();
-                    it != itEnd; ++it )
-    {
-        boost::intrusive_ptr<XMLNode_as> node = *it;
-        ary->push(as_value(node.get()));
-    }
-
-    return as_value(ary.get());
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
+    return ptr->childNodes();
 }
 
 
 void
 enumerateAttributes(const XMLNode_as& node,
-            PropertyList::SortedPropertyList& attrs)
+        as_object::SortedPropertyList& attrs)
 {
     attrs.clear();
-    const as_object* obj = node.getAttributes();
+    as_object* obj = node.getAttributes();
     if (obj) {
-        obj->enumerateProperties(attrs);
+        enumerateProperties(*obj, attrs);
     }
 
 }
@@ -995,7 +1001,7 @@ enumerateAttributes(const XMLNode_as& node,
 /// Return true if this attribute is a namespace specifier and the
 /// namespace matches.
 bool
-namespaceMatches(const PropertyList::SortedPropertyList::value_type& val,
+namespaceMatches(const as_object::SortedPropertyList::value_type& val,
         const std::string& ns)
 {
     StringNoCaseEqual noCaseCompare;
@@ -1005,7 +1011,7 @@ namespaceMatches(const PropertyList::SortedPropertyList::value_type& val,
 
 
 bool
-prefixMatches(const PropertyList::SortedPropertyList::value_type& val,
+prefixMatches(const as_object::SortedPropertyList::value_type& val,
         const std::string& prefix)
 {
     const std::string& name = val.first;
@@ -1023,11 +1029,10 @@ prefixMatches(const PropertyList::SortedPropertyList::value_type& val,
     return noCaseCompare(prefix, name.substr(6));
 }
 
-} // anonymous namespace
-
-} // end of gnash namespace
-
-// Local Variables:
+} // anonymous namespace 
+} // gnash namespace
+// local Variables:
 // mode: C++
 // indent-tabs-mode: t
 // End:
+
