@@ -1,32 +1,43 @@
-// 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
-// 
+// XMLSocket_as.h:  ActionScript 3 "XMLSocket" class, for Gnash.
+//
+//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+//
 
-#ifndef GNASH_XMLSOCKET_H
-#define GNASH_XMLSOCKET_H
+#ifndef GNASH_ASOBJ3_XMLSOCKET_H
+#define GNASH_ASOBJ3_XMLSOCKET_H
 
 namespace gnash {
-  
-void xmlsocket_class_init(as_object& global);
 
-} // end of gnash namespace
+// Forward declarations
+class as_object;
+struct ObjectURI;
 
+/// Initialize the global XMLSocket class
+void xmlsocket_class_init(as_object& where, const ObjectURI& uri);
+
+void registerXMLSocketNative(as_object& global);
+
+} // gnash namespace
+
+// GNASH_ASOBJ3_XMLSOCKET_H
 #endif
 
-// Local Variables:
+// local Variables:
 // mode: C++
 // indent-tabs-mode: t
 // End:
+
