@@ -22,7 +22,7 @@
 namespace gnash {
 
 class as_object;
-class ObjectURI;
+struct ObjectURI;
 
 /// Initialize the global ConvolutionFilter class
 void convolutionfilter_class_init(as_object& where, const ObjectURI& uri);
