@@ -1,6 +1,6 @@
 // DisplacementMapFilterMode_as.h:  ActionScript 3 "DisplacementMapFilterMode" class, for Gnash.
 //
-//   Copyright (C) 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,16 +20,13 @@
 #ifndef GNASH_ASOBJ3_DISPLACEMENTMAPFILTERMODE_H
 #define GNASH_ASOBJ3_DISPLACEMENTMAPFILTERMODE_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
 
 
 namespace gnash {
 
 // Forward declarations
 class as_object;
-class ObjectURI;
+struct ObjectURI;
 
 /// Initialize the global DisplacementMapFilterMode class
 void displacementmapfiltermode_class_init(as_object& where, const ObjectURI& uri);
