@@ -22,7 +22,12 @@
 #define GNASH_ASOBJ_COLORTRANSFORM_H
 
 
-#include "as_object.h"
+#include "Relay.h"
+
+namespace gnash {
+    class as_object;
+    struct ObjectURI;
+}
 
 namespace gnash {
 
@@ -61,6 +66,7 @@ public:
     double getBlueOffset() const { return _blueOffset; }
     double getGreenOffset() const { return _greenOffset; }
 
+    void concat(const ColorTransform_as& other);
 
 private:
 
