@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,11 +22,9 @@
 namespace gnash {
 
 class as_object;
-
 void flash_geom_package_init(as_object& pkg, const ObjectURI& uri);
 
-} // end of gnash namespace
+} // gnash namespace
 
-// __FLASH_GEOM_PACKAGE_H__
 #endif
 
