@@ -22,12 +22,11 @@
 #define __GNASH_ASOBJ_FILEREFERENCELIST_H__
 
 
-//#include <memory> // for auto_ptr
 
 namespace gnash {
 
 class as_object;
-class ObjectURI;
+struct ObjectURI;
 
 /// Initialize the global FileReferenceList class
 void filereferencelist_class_init(as_object& where, const ObjectURI& uri);
