@@ -15,7 +15,9 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+#include "RunResources.h"
 #include "DefineEditTextTag.h"
+#include "TextField_as.h"
 #include "TextField.h"
 #include "movie_definition.h"
 #include "Font.h"
@@ -32,11 +34,11 @@ DefineEditTextTag::loader(SWFStream& in, TagType tag, movie_definition& m,
 	assert(tag == SWF::DEFINEEDITTEXT); // 37
 
     in.ensureBytes(2);
-    boost::uint16_t DisplayObjectID = in.read_u16();
+    const boost::uint16_t id = in.read_u16();
 
-    std::auto_ptr<DefineEditTextTag> editText(new DefineEditTextTag(in, m));
+    std::auto_ptr<DefineEditTextTag> editText(new DefineEditTextTag(in, m, id));
 
-    m.addDisplayObject(DisplayObjectID, editText.release());
+    m.addDisplayObject(id, editText.release());
 }
 
 DisplayObject*
@@ -48,27 +50,6 @@ DefineEditTextTag::createDisplayObject(Global_as& gl, DisplayObject* parent)
     as_object* obj = createTextFieldObject(gl);
 	TextField* ch = new TextField(obj, parent, *this);
 
-	// This gives an "instance name" to the TextField, but
-	// it is not really what we need.
-	//
-	// First of all the VariableName ("_variableName") is
-	// NOT the default name of an instance, rather it is
-	// a variable associated with it and can contain path
-	// information (ie. we can associate a variable in a different
-	// timeline)
-	//
-	// We actually need to set that variable to an object which
-	// is a TextField dinamic variable. The object should take
-	// care of updating this TextField text when assigned to
-	// and to retrive this TextField text when extracted value from.
-	//
-	// The DefineEditTextVariableNameTest.swf file under
-	// testsuite/misc-ming.all gives an idea of the problem
-	// (in particular it shows a case in which VariableName is
-	// outside of TextField timeline/scope)
-	//
-	//ch->set_name(m_variable_name.c_str());
-
 	return ch;
 }
 
@@ -141,9 +122,8 @@ DefineEditTextTag::read(SWFStream& in, movie_definition& m)
                 fontClassName);
 	}
 	
-	if (hasColor)
-	{
-		_color.read_rgba(in);
+	if (hasColor) {
+		_color = readRGBA(in);
 	}
 
 	if (hasMaxChars)
@@ -179,8 +159,10 @@ DefineEditTextTag::read(SWFStream& in, movie_definition& m)
 	);
 }
 
-DefineEditTextTag::DefineEditTextTag(SWFStream& in, movie_definition& m)
+DefineEditTextTag::DefineEditTextTag(SWFStream& in, movie_definition& m,
+        boost::uint16_t id)
     :
+    DefinitionTag(id),
 	_hasText(true),
 	_wordWrap(false),
 	_multiline(false),
