@@ -31,6 +31,8 @@
 #include "GnashNumeric.h"
 #include "RunResources.h"
 #include "Global_as.h"
+#include "Renderer.h"
+#include "FillStyle.h"
 
 namespace gnash {
 namespace SWF {
@@ -40,18 +42,20 @@ DefineMorphShapeTag::loader(SWFStream& in, TagType tag, movie_definition& md,
         const RunResources& r)
 {
     in.ensureBytes(2);
-    boost::uint16_t id = in.read_u16();
+    const boost::uint16_t id = in.read_u16();
 
     IF_VERBOSE_PARSE(
             log_parse("DefineMorphShapeTag: id = %d", id);
     );
 
-    DefineMorphShapeTag* morph = new DefineMorphShapeTag(in, tag, md, r);
+    DefineMorphShapeTag* morph = new DefineMorphShapeTag(in, tag, md, r, id);
     md.addDisplayObject(id, morph);
 }
 
 DefineMorphShapeTag::DefineMorphShapeTag(SWFStream& in, TagType tag,
-        movie_definition& md, const RunResources& r)
+        movie_definition& md, const RunResources& r, boost::uint16_t id)
+    :
+    DefinitionTag(id)
 {
     read(in, tag, md, r);
 }
@@ -103,11 +107,10 @@ DefineMorphShapeTag::read(SWFStream& in, TagType tag, movie_definition& md,
     // Next line will throw ParserException on malformed SWF
     const boost::uint16_t fillCount = in.read_variable_count();
     
-    fill_style fs1, fs2;
     for (size_t i = 0; i < fillCount; ++i) {
-        fs1.read(in, tag, md, r, &fs2);
-        _shape1.addFillStyle(fs1);
-        _shape2.addFillStyle(fs2);
+        OptionalFillPair fp = readFills(in, tag, md, true);
+        _shape1.addFillStyle(fp.first);
+        _shape2.addFillStyle(*fp.second);
     }
 
     const boost::uint16_t lineCount = in.read_variable_count();
