@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,18 +16,12 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-
 #ifndef GNASH_SWF_DEFINEVIDEOSTREAMTAG_H
 #define GNASH_SWF_DEFINEVIDEOSTREAMTAG_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "DefinitionTag.h"
 #include "SWF.h"
-#include "rect.h" // for composition
+#include "SWFRect.h" // for composition
 #include "MediaParser.h" // for videoFrameType and videoCodecType enums
 
 #include <boost/shared_array.hpp>
@@ -83,7 +78,8 @@ public:
 
 	~DefineVideoStreamTag();
 
-	DisplayObject* createDisplayObject(DisplayObject* parent, int id) const;
+	DisplayObject* createDisplayObject(Global_as& gl, DisplayObject* parent)
+        const;
 
 	/// Read tag SWF::DEFINEVIDEOSTREAM 
 	//
@@ -107,7 +103,7 @@ public:
             movie_definition& m);
 
 	/// Return local video bounds in twips
-	const rect&	bounds() const
+	const SWFRect&	bounds() const
 	{
 		return m_bound;
 	}
@@ -154,13 +150,6 @@ private:
 
 	void read(SWFStream& in);
 
-	/// Id of this DisplayObject definition, set by constructor.
-	///
-	/// The id is currently set to -1 when the definition is actually
-	/// created dynamically (instantiating the ActionScript Video class)
-	///
-	boost::uint16_t m_char_id;
-
 	/// Reserved flags read from DEFINEVIDEOSTREAM tag
 	boost::uint8_t m_reserved_flags;
 
@@ -189,7 +178,7 @@ private:
 	media::videoCodecType m_codec_id;
 
 	/// Bounds of the video, as read from the DEFINEVIDEOSTREAM tag.
-	rect m_bound;
+	SWFRect m_bound;
 
     // Mutable for locking in const member functions.
 	mutable boost::mutex _video_mutex;
