@@ -18,9 +18,9 @@
 #ifndef GNASH_SWF_DOABCTAG_H
 #define GNASH_SWF_DOABCTAG_H
 
+#include <string>
 #include "ControlTag.h" // for inheritance
 #include "SWF.h" // for tag_type definition
-#include "action_buffer.h" // for composition
 #include "MovieClip.h" // for inlines
 #include "SWFStream.h" // for inlines
 #include "AbcBlock.h"
@@ -42,7 +42,7 @@ class DoABCTag : public ControlTag
 {
 public:
 
-    virtual void execute(MovieClip* m, DisplayList& /* dlist */) const
+    virtual void executeActions(MovieClip* m, DisplayList& /* dlist */) const
 	{
 
         if (!_abc) {
@@ -63,12 +63,6 @@ public:
 		mach->execute();
 	}
 
-	// Tell the caller that we are an action tag.
-	virtual bool is_action_tag() const
-	{
-	    return true;
-	}
-
     void read(SWFStream* /*in*/)
     {
     }
