@@ -1,4 +1,5 @@
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,10 +20,11 @@
 #include "SWF.h"
 #include "SWFStream.h"
 #include "movie_definition.h"
-#include "fill_style.h"
+#include "smart_ptr.h"
+#include "FillStyle.h"
 #include "Geometry.h"
 #include "GnashNumeric.h"
-#include "RunResources.h"
+#include "log.h"
 
 #include <vector>
 
@@ -32,10 +34,10 @@ namespace SWF {
 // Forward declarations
 namespace {
     void readFillStyles(ShapeRecord::FillStyles& styles, SWFStream& in,
-        SWF::TagType tag, movie_definition& md, const RunResources& r);
+        SWF::TagType tag, movie_definition& md, const RunResources& /*r*/);
     void readLineStyles(ShapeRecord::LineStyles& styles, SWFStream& in,
-        SWF::TagType tag, movie_definition& md, const RunResources& r);
-    void computeBounds(rect& bounds, const ShapeRecord::Paths& paths,
+        SWF::TagType tag, movie_definition& md, const RunResources& /*r*/);
+    void computeBounds(SWFRect& bounds, const ShapeRecord::Paths& paths,
         const ShapeRecord::LineStyles& lineStyles, int swfVersion);
 }
 
@@ -56,7 +58,7 @@ public:
 
     void operator()(typename T::value_type& st)
     {
-        st.set_lerp(*_style1, *_style2, _ratio);
+        setLerp(st, *_style1, *_style2, _ratio);
         ++_style1, ++_style2;
     }
 
@@ -142,6 +144,48 @@ ShapeRecord::ShapeRecord(SWFStream& in, SWF::TagType tag, movie_definition& m,
     read(in, tag, m, r);
 }
 
+ShapeRecord::ShapeRecord()
+{
+}
+
+ShapeRecord::~ShapeRecord()
+{
+}
+
+ShapeRecord::ShapeRecord(const ShapeRecord& other)
+    :
+    _fillStyles(other._fillStyles),
+    _lineStyles(other._lineStyles),
+    _paths(other._paths),
+    _bounds(other._bounds)
+{
+}
+    
+ShapeRecord&
+ShapeRecord::operator=(const ShapeRecord& other)
+{
+    _fillStyles = other._fillStyles;
+    _lineStyles = other._lineStyles;
+    _paths = other._paths;
+    _bounds = other._bounds;
+    return *this;
+}
+
+void
+ShapeRecord::clear()
+{
+    _fillStyles.clear();
+    _lineStyles.clear();
+    _paths.clear();
+    _bounds.set_null();
+}
+
+void
+ShapeRecord::addFillStyle(const FillStyle& fs)
+{
+    _fillStyles.push_back(fs);
+}
+
 void
 ShapeRecord::setLerp(const ShapeRecord& a, const ShapeRecord& b,
         const double ratio)
@@ -179,8 +223,8 @@ ShapeRecord::setLerp(const ShapeRecord& a, const ShapeRecord& b,
         const Path& p1 = i < paths1.size() ? paths1[i] : empty_path;
         const Path& p2 = n < paths2.size() ? paths2[n] : empty_path;
 
-        const float new_ax = flerp(p1.ap.x, p2.ap.x, ratio);
-        const float new_ay = flerp(p1.ap.y, p2.ap.y, ratio);
+        const float new_ax = lerp<float>(p1.ap.x, p2.ap.x, ratio);
+        const float new_ay = lerp<float>(p1.ap.y, p2.ap.y, ratio);
 
         p.reset(new_ax, new_ay, p1.getLeftFill(),
                 p2.getRightFill(), p1.getLineStyle());
@@ -196,10 +240,10 @@ ShapeRecord::setLerp(const ShapeRecord& a, const ShapeRecord& b,
 
             const Edge& e2 = k < p2.size() ? p2[k] : empty_edge;
 
-            e.cp.x = static_cast<int>(flerp(e1.cp.x, e2.cp.x, ratio));
-            e.cp.y = static_cast<int>(flerp(e1.cp.y, e2.cp.y, ratio));
-            e.ap.x = static_cast<int>(flerp(e1.ap.x, e2.ap.x, ratio));
-            e.ap.y = static_cast<int>(flerp(e1.ap.y, e2.ap.y, ratio));
+            e.cp.x = static_cast<int>(lerp<float>(e1.cp.x, e2.cp.x, ratio));
+            e.cp.y = static_cast<int>(lerp<float>(e1.cp.y, e2.cp.y, ratio));
+            e.ap.x = static_cast<int>(lerp<float>(e1.ap.x, e2.ap.x, ratio));
+            e.ap.y = static_cast<int>(lerp<float>(e1.ap.y, e2.ap.y, ratio));
             ++k;
 
             if (p2.size() <= k) {
@@ -229,13 +273,13 @@ ShapeRecord::read(SWFStream& in, SWF::TagType tag, movie_definition& m,
     
         IF_VERBOSE_PARSE(
             std::string b = _bounds.toString();
-            log_parse(_("  bound rect: %s"), b);
+            log_parse(_("  bound SWFRect: %s"), b);
         );
     
         // TODO: Store and use these. Unfinished.
         if (tag == SWF::DEFINESHAPE4 || tag == SWF::DEFINESHAPE4_)
         {
-            rect tbound;
+            SWFRect tbound;
             tbound.read(in);
             in.ensureBytes(1);
             static_cast<void>(in.read_u8());
@@ -350,7 +394,7 @@ ShapeRecord::read(SWFStream& in, SWF::TagType tag, movie_definition& m,
             }
             if ((flags & SHAPE_FILLSTYLE0_CHANGE) && num_fill_bits > 0)
             {
-                // fill_style_0_change = 1;
+                // FillStyle_0_change = 1;
                 if (! current_path.empty())
                 {
                     _paths.push_back(current_path);
@@ -401,7 +445,7 @@ ShapeRecord::read(SWFStream& in, SWF::TagType tag, movie_definition& m,
             }
             if ((flags & SHAPE_FILLSTYLE1_CHANGE) && num_fill_bits > 0)
             {
-                // fill_style_1_change = 1;
+                // FillStyle_1_change = 1;
                 if (! current_path.empty())
                 {
                     _paths.push_back(current_path);
@@ -611,16 +655,17 @@ ShapeRecord::read(SWFStream& in, SWF::TagType tag, movie_definition& m,
         //       the bounds as edges are parsed.
         computeBounds(_bounds, _paths, _lineStyles, m.get_version());
     }
+
 #ifdef GNASH_DEBUG_SHAPE_BOUNDS
     else
     {
-        rect computedBounds;
-        computeBounds(computedBounds, _paths, _lineStyles, m->get_version());
-        if ( computedBounds != m_bounds )
+        SWFRect computedBounds;
+        computeBounds(computedBounds, _paths, _lineStyles, m.get_version());
+        if ( computedBounds != _bounds )
         {
             log_debug("Shape object read for tag %d contained embedded "
                     "bounds %s, while we computed bounds %s",
-                    tag, m_bound, computedBounds);
+                    tag, _bounds, computedBounds);
         }
     }
 #endif
@@ -631,30 +676,29 @@ namespace {
 // Read fill styles, and push them onto the given style array.
 void
 readFillStyles(ShapeRecord::FillStyles& styles, SWFStream& in,
-                 SWF::TagType tag, movie_definition& m, const RunResources& r)
+         SWF::TagType tag, movie_definition& m, const RunResources& /*r*/)
 {
     in.ensureBytes(1);
-    boost::uint16_t fill_style_count = in.read_u8();
+    boost::uint16_t FillStyle_count = in.read_u8();
     if (tag != SWF::DEFINESHAPE)
     {
-        if (fill_style_count == 0xFF)
+        if (FillStyle_count == 0xFF)
         {
             in.ensureBytes(2);
-            fill_style_count = in.read_u16();
+            FillStyle_count = in.read_u16();
         }
     }
 
     IF_VERBOSE_PARSE (
-        log_parse(_("  readFillStyles: count = %u"), fill_style_count);
+        log_parse(_("  readFillStyles: count = %u"), FillStyle_count);
     );
 
     // Read the styles.
-    styles.reserve(styles.size()+fill_style_count);
-    for (boost::uint16_t i = 0; i < fill_style_count; ++i) {
-        // TODO: add a fill_style constructor directly reading from stream
-        fill_style fs;
-        fs.read(in, tag, m, r);
-        styles.push_back(fs);
+    styles.reserve(styles.size()+FillStyle_count);
+    for (boost::uint16_t i = 0; i < FillStyle_count; ++i) {
+        // TODO: add a FillStyle constructor directly reading from stream
+        OptionalFillPair fp = readFills(in, tag, m, false);
+        styles.push_back(fp.first);
     }
 }
 
@@ -689,7 +733,7 @@ readLineStyles(ShapeRecord::LineStyles& styles, SWFStream& in,
 
 // Find the bounds of this shape, and store them in the given rectangle.
 void
-computeBounds(rect& bounds, const ShapeRecord::Paths& paths,
+computeBounds(SWFRect& bounds, const ShapeRecord::Paths& paths,
         const ShapeRecord::LineStyles& lineStyles, int swfVersion)
 {
     bounds.set_null();
@@ -721,6 +765,13 @@ computeBounds(rect& bounds, const ShapeRecord::Paths& paths,
 
 } // anonymous namespace
 
+std::ostream&
+operator<<(std::ostream& o, const ShapeRecord& sh)
+{
+    o << "Shape Record: bounds " << sh.getBounds();
+    return o;
+}
+
 } // namespace SWF
 } // namespace gnash
 
