@@ -25,7 +25,7 @@
 
 #include "ActionExec.h"
 #include "action_buffer.h"
-#include "swf_function.h"
+#include "Function.h"
 #include "log.h"
 #include "VM.h"
 #include "GnashException.h"
@@ -36,6 +36,7 @@
 #include "as_environment.h"
 #include "debugger.h"
 #include "SystemClock.h"
+#include "CallStack.h"
 
 #include <sstream>
 #include <string>
@@ -54,7 +55,7 @@
 # define STACK_DUMP_LIMIT 32
 
 // Define to get debugging messages for try / catch
-#define GNASH_DEBUG_TRY 1
+//#define GNASH_DEBUG_TRY 1
 
 #endif
 
@@ -66,16 +67,17 @@ namespace gnash {
 static Debugger& debugger = Debugger::getDefaultInstance();
 #endif
 
-ActionExec::ActionExec(const swf_function& func, as_environment& newEnv,
+ActionExec::ActionExec(const Function& func, as_environment& newEnv,
         as_value* nRetVal, as_object* this_ptr)
     :
+    code(func.getActionBuffer()),
+    env(newEnv),
+    retval(nRetVal),
     _withStack(),
     _scopeStack(func.getScopeStack()),
-    _withStackLimit(7),
     _func(&func),
     _this_ptr(this_ptr),
     _initialStackSize(0),
-    _initialCallStackDepth(0),
     _originalTarget(0),
     _origExecSWFVersion(0),
     _tryList(),
@@ -83,19 +85,10 @@ ActionExec::ActionExec(const swf_function& func, as_environment& newEnv,
     _abortOnUnload(false),
     pc(func.getStartPC()),
     next_pc(pc),
-    stop_pc(pc + func.getLength()),
-    code(func.getActionBuffer()),
-    env(newEnv),
-    retval(nRetVal)
+    stop_pc(pc + func.getLength())
 {
     assert(stop_pc < code.size());
 
-    // See comment in header
-    // TODO: stack limit dependent on function version or VM version ?
-    if ( env.get_version() > 5 ) {
-        _withStackLimit = 15;
-    }
-
     // Functions defined in SWF version 6 and higher pushes
     // the activation object to the scope stack
     // NOTE: if we query env.get_version() we get version of the calling
@@ -108,10 +101,10 @@ ActionExec::ActionExec(const swf_function& func, as_environment& newEnv,
     //              push 'i' getvariable
     //              get var: i=[undefined] 
     if (code.getDefinitionVersion() > 5) {
-        // We assume that the swf_function () operator already initialized
+        // We assume that the Function () operator already initialized
         // its environment so that its activation object is now in the
         // top element of the CallFrame stack
-        CallFrame& topFrame = newEnv.topCallFrame();
+        CallFrame& topFrame = getVM(newEnv).currentCall();
         assert(&topFrame.function() == &func);
         _scopeStack.push_back(&topFrame.locals());
     }
@@ -120,12 +113,13 @@ ActionExec::ActionExec(const swf_function& func, as_environment& newEnv,
 ActionExec::ActionExec(const action_buffer& abuf, as_environment& newEnv,
         bool abortOnUnloaded)
     :
+    code(abuf),
+    env(newEnv),
+    retval(0),
     _withStack(),
-    _scopeStack(), // TODO: initialize the scope stack somehow
-    _withStackLimit(7),
+    _scopeStack(),
     _func(0),
     _initialStackSize(0),
-    _initialCallStackDepth(0),
     _originalTarget(0),
     _origExecSWFVersion(0),
     _tryList(),
@@ -133,16 +127,8 @@ ActionExec::ActionExec(const action_buffer& abuf, as_environment& newEnv,
     _abortOnUnload(abortOnUnloaded),
     pc(0),
     next_pc(0),
-    stop_pc(abuf.size()),
-    code(abuf),
-    env(newEnv),
-    retval(0)
+    stop_pc(abuf.size())
 {
-    /// See comment in header
-    // TODO: stack limit dependent on function version or VM version ?
-    if (env.get_version() > 5) {
-        _withStackLimit = 15;
-    }
 }
 
 void
@@ -164,18 +150,13 @@ ActionExec::operator()()
 
     _initialStackSize = env.stack_size();
 
-    _initialCallStackDepth = env.callStackDepth();
-
 #if DEBUG_STACK
     IF_VERBOSE_ACTION (
             log_action(_("at ActionExec operator() start, pc=%d"
                    ", stop_pc=%d, code.size=%d, func=%d, codeVersion=%d"),
                 pc, stop_pc, code.size(), _func ? _func : 0, codeVersion);
         std::stringstream ss;
-        env.dump_stack(ss, STACK_DUMP_LIMIT);
-        env.dump_global_registers(ss);
-        env.dump_local_registers(ss);
-        env.dump_local_variables(ss);
+        getVM(env).dumpState(ss, STACK_DUMP_LIMIT);
         log_action("%s", ss.str());
     );
 #endif
@@ -318,10 +299,7 @@ ActionExec::operator()()
                 log_action(_("After execution: PC %d, next PC %d, "
                         "stack follows"), pc, next_pc);
                 std::stringstream ss;
-                env.dump_stack(ss, STACK_DUMP_LIMIT);
-                env.dump_global_registers(ss);
-                env.dump_local_registers(ss);
-                env.dump_local_variables(ss);
+                getVM(env).dumpState(ss, STACK_DUMP_LIMIT);
                 log_action("%s", ss.str());
             );
 #endif
@@ -416,7 +394,7 @@ ActionExec::processExceptions(TryBlock& t)
                     as_value ex = env.pop();
                     ex.unflag_exception();
                     
-                    env.setRegister(t._registerIndex, ex);
+                    getVM(env).setRegister(t._registerIndex, ex);
                 }
             }
             else
@@ -598,9 +576,6 @@ ActionExec::cleanupAfterRun()
     // Have movie_root flush any newly pushed actions in higher priority queues
     getRoot(env).flushHigherPriorityActionQueues();
 
-//    log_debug("After cleanup of ActionExec %p, env %p has "
-//        "stack size of %d and callStackDepth of %d",
-//        (void*)this, (void*)&env, env.stack_size(), env.callStackDepth());
 }
 
 void
@@ -646,24 +621,21 @@ ActionExec::skip_actions(size_t offset)
 }
 
 bool
-ActionExec::pushWithEntry(const with_stack_entry& entry)
+ActionExec::pushWith(const With& entry)
 {
-    // See comment in header about _withStackLimit
-    if (_withStack.size() >= _withStackLimit)
-    {
-        IF_VERBOSE_ASCODING_ERRORS (
-        log_aserror(_("'With' stack depth (%d) "
-            "exceeds the allowed limit for current SWF "
-            "target version (%d for version %d)."
-            " Don't expect this movie to work with all players."),
-            _withStack.size()+1, _withStackLimit,
-            env.get_version());
+    // The maximum number of withs supported is 13, regardless of the
+    // other documented figures. See actionscript.all/with.as.
+    const size_t withLimit = 13;
+
+    if (_withStack.size() == withLimit) {
+        IF_VERBOSE_ASCODING_ERRORS(
+            log_aserror("With stack limit of %s exceeded");
         );
         return false;
     }
     
     _withStack.push_back(entry);
-    _scopeStack.push_back(const_cast<as_object*>(entry.object()));
+    _scopeStack.push_back(entry.object());
     return true;
 }
 
@@ -702,9 +674,10 @@ ActionExec::getVariable(const std::string& name, as_object** target)
 void
 ActionExec::setLocalVariable(const std::string& name, const as_value& val)
 {
-    if ( isFunction() ) {
+    if (isFunction()) {
+        string_table& st = getStringTable(env);
         // TODO: set local in the function object?
-        env.set_local(name, val);
+        setLocal(getVM(env).currentCall(), st.find(name), val);
     } else {
         // TODO: set target member  ?
         //       what about 'with' stack ?
@@ -712,35 +685,17 @@ ActionExec::setLocalVariable(const std::string& name, const as_value& val)
     }
 }
 
-void
-ActionExec::setObjectMember(as_object& obj, const std::string& var,
-        const as_value& val)
-{
-    string_table& st = getStringTable(env);
-    obj.set_member(st.find(var), val);
-}
-
-bool
-ActionExec::getObjectMember(as_object& obj, const std::string& var,
-        as_value& val)
-{
-    string_table& st = getStringTable(env);
-    return obj.get_member(st.find(var), &val);
-}
-
 as_object*
 ActionExec::getTarget()
 {
-    if (!_withStack.empty()) {
-        return _withStack.back().object();
-    }
-    else {
+    if (_withStack.empty()) {
         return getObject(env.get_target());
     }
+    return _withStack.back().object();
 }
 
 void
-ActionExec::pushTryBlock(TryBlock& t)
+ActionExec::pushTryBlock(TryBlock t)
 {
     // The current block should end at the end of the try block.
     t._savedEndOffset = stop_pc;
@@ -771,12 +726,6 @@ ActionExec::adjustNextPC(int offset)
     next_pc += offset;
 }
 
-bool
-ActionExec::isFunction2() const
-{
-    return _func ? _func->isFunction2() : false;
-}
-
 void
 ActionExec::dumpActions(size_t from, size_t to, std::ostream& os)
 {
