@@ -9,35 +9,23 @@
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA    02110-1301    USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
 #endif
 
-#ifdef HAVE_FFMPEG_SWSCALE_H
-extern "C" {
-#include <ffmpeg/swscale.h>
-}
-#define HAVE_SWSCALE_H 1
-#endif
-
-#ifdef HAVE_LIBSWSCALE_SWSCALE_H
-extern "C" {
-#include <libswscale/swscale.h>
-}
-#define HAVE_SWSCALE_H 1
-#endif
-
+#include "ffmpegHeaders.h"
 #include "VideoDecoderFfmpeg.h"
 #include "MediaParserFfmpeg.h" // for ExtraVideoInfoFfmpeg 
 #include "GnashException.h" // for MediaException
+#include "utility.h"
 
 #include <boost/scoped_array.hpp>
 #include <boost/format.hpp>
@@ -45,9 +33,31 @@ extern "C" {
 
 #include "FLVParser.h"
 
+#ifdef HAVE_VA_VA_H
+#  include "vaapi_utils.h"
+#  include "VideoDecoderFfmpegVaapi.h"
+#  include "GnashVaapiImage.h"
+#endif
+
 namespace gnash {
 namespace media {
 namespace ffmpeg {
+ 
+class VaapiContextFfmpeg;
+
+// Forward declarations of VAAPI functions.
+namespace {
+
+
+    VaapiContextFfmpeg* get_vaapi_context(AVCodecContext* avctx);
+    void set_vaapi_context(AVCodecContext* avctx, VaapiContextFfmpeg* vactx);
+    void clear_vaapi_context(AVCodecContext* avctx);
+    void reset_context(AVCodecContext* avctx, VaapiContextFfmpeg* vactx = 0);
+    PixelFormat get_format(AVCodecContext* avctx, const PixelFormat* fmt);
+    int get_buffer(AVCodecContext* avctx, AVFrame* pic);
+    int reget_buffer(AVCodecContext* avctx, AVFrame* pic);
+    void release_buffer(AVCodecContext *avctx, AVFrame *pic);
+}
 
 #ifdef HAVE_SWSCALE_H
 /// A wrapper round an SwsContext that ensures it's
@@ -74,6 +84,7 @@ private:
 };
 #endif
 
+
 // A Wrapper ensuring an AVCodecContext is closed and freed
 // on destruction.
 class CodecContextWrapper
@@ -89,6 +100,7 @@ public:
         if (_codecCtx)
         {
             avcodec_close(_codecCtx);
+            clear_vaapi_context(_codecCtx);
             av_free(_codecCtx);
         }
     }
@@ -178,6 +190,19 @@ VideoDecoderFfmpeg::init(enum CodecID codecId, int /*width*/, int /*height*/,
     ctx->extradata = extradata;
     ctx->extradata_size = extradataSize;
 
+    ctx->get_format     = get_format;
+    ctx->get_buffer     = get_buffer;
+    ctx->reget_buffer   = reget_buffer;
+    ctx->release_buffer = release_buffer;
+
+#ifdef HAVE_VA_VA_H
+    if (vaapi_is_enabled()) {
+        VaapiContextFfmpeg *vactx = VaapiContextFfmpeg::create(codecId);
+        if (vactx)
+            reset_context(ctx, vactx);
+    }
+#endif
+
     int ret = avcodec_open(ctx, _videoCodec);
     if (ret < 0) {
         boost::format msg = boost::format(_("libavcodec"
@@ -189,7 +214,7 @@ VideoDecoderFfmpeg::init(enum CodecID codecId, int /*width*/, int /*height*/,
     }
     
     log_debug(_("VideoDecoder: initialized FFMPEG codec %s (%d)"), 
-		_videoCodec->name, (int)codecId);
+                _videoCodec->name, (int)codecId);
 
 }
 
@@ -213,8 +238,10 @@ VideoDecoderFfmpeg::height() const
 
 std::auto_ptr<GnashImage>
 VideoDecoderFfmpeg::frameToImage(AVCodecContext* srcCtx,
-                                 const AVFrame& srcFrame)
+                                 const AVFrame& srcFrameRef)
 {
+    const AVFrame *srcFrame = &srcFrameRef;
+    PixelFormat srcPixFmt = srcCtx->pix_fmt;
 
     const int width = srcCtx->width;
     const int height = srcCtx->height;
@@ -228,13 +255,26 @@ VideoDecoderFfmpeg::frameToImage(AVCodecContext* srcCtx,
 
     std::auto_ptr<GnashImage> im;
 
+#ifdef HAVE_VA_VA_H
+    VaapiContextFfmpeg * const vactx = get_vaapi_context(srcCtx);
+    if (vactx) {
+        VaapiSurfaceFfmpeg * const vaSurface = vaapi_get_surface(&srcFrameRef);
+        if (!vaSurface) {
+            im.reset();
+            return im;
+        }
+        im.reset(new GnashVaapiImage(vaSurface->get(), GNASH_IMAGE_RGBA));
+        return im;
+    }
+#endif
+
 #ifdef HAVE_SWSCALE_H
     // Check whether the context wrapper exists
     // already.
     if (!_swsContext.get()) {
 
         _swsContext.reset(new SwsContextWrapper(
-            sws_getContext(width, height, srcCtx->pix_fmt, width, height,
+            sws_getContext(width, height, srcPixFmt, width, height,
                 pixFmt, SWS_BILINEAR, NULL, NULL, NULL)
         ));
         
@@ -276,11 +316,11 @@ VideoDecoderFfmpeg::frameToImage(AVCodecContext* srcCtx,
     // Let ffmpeg write directly to the GnashImage data. It is an uninitialized
     // buffer here, so do not return the image if there is any error in
     // conversion.
-    avpicture_fill(&picture, im->data(), pixFmt, width, height);
+    avpicture_fill(&picture, im->begin(), pixFmt, width, height);
 
 #ifndef HAVE_SWSCALE_H
-    img_convert(&picture, PIX_FMT_RGB24, (AVPicture*) &srcFrame,
-            srcCtx->pix_fmt, width, height);
+    img_convert(&picture, PIX_FMT_RGB24, (AVPicture*)srcFrame,
+            srcPixFmt, width, height);
 #else
 
     // Is it possible for the context to be reset
@@ -288,8 +328,8 @@ VideoDecoderFfmpeg::frameToImage(AVCodecContext* srcCtx,
     assert(_swsContext->getContext());
 
     int rv = sws_scale(_swsContext->getContext(), 
-            const_cast<uint8_t**>(srcFrame.data),
-            const_cast<int*>(srcFrame.linesize), 0, height, picture.data,
+            const_cast<uint8_t**>(srcFrame->data),
+            const_cast<int*>(srcFrame->linesize), 0, height, picture.data,
             picture.linesize);
 
     if (rv == -1) {
@@ -394,6 +434,138 @@ VideoDecoderFfmpeg::flashToFfmpegCodec(videoCodecType format)
         }
 }
 
+namespace {
+
+inline VaapiContextFfmpeg*
+get_vaapi_context(AVCodecContext* avctx)
+{
+#ifdef HAVE_VA_VA_H
+    return static_cast<VaapiContextFfmpeg *>(avctx->hwaccel_context);
+#else
+    UNUSED(avctx);
+	return 0;
+#endif
+}
+
+inline void
+set_vaapi_context(AVCodecContext* avctx, VaapiContextFfmpeg* vactx)
+{
+#ifdef HAVE_VA_VA_H
+    avctx->hwaccel_context = vactx;
+#else
+    UNUSED(avctx), UNUSED(vactx);
+#endif
+    
+}
+
+inline void
+clear_vaapi_context(AVCodecContext* avctx)
+{
+#ifdef HAVE_VA_VA_H
+    VaapiContextFfmpeg* const vactx = get_vaapi_context(avctx);
+    if (!vactx) return;
+
+    delete vactx;
+    set_vaapi_context(avctx, NULL);
+#else
+    UNUSED(avctx);
+#endif
+}
+
+/// (Re)set AVCodecContext to sane values 
+void
+reset_context(AVCodecContext* avctx, VaapiContextFfmpeg* vactx)
+{
+    clear_vaapi_context(avctx);
+    set_vaapi_context(avctx, vactx);
+
+    avctx->thread_count = 1;
+    avctx->draw_horiz_band = 0;
+    if (vactx) {
+        avctx->slice_flags = SLICE_FLAG_CODED_ORDER|SLICE_FLAG_ALLOW_FIELD;
+    }
+    else avctx->slice_flags = 0;
+}
+
+/// AVCodecContext.get_format() implementation
+PixelFormat
+get_format(AVCodecContext* avctx, const PixelFormat* fmt)
+{
+#ifdef HAVE_VA_VA_H
+    VaapiContextFfmpeg* const vactx = get_vaapi_context(avctx);
+
+    if (vactx) {
+        for (int i = 0; fmt[i] != PIX_FMT_NONE; i++) {
+            if (fmt[i] != PIX_FMT_VAAPI_VLD) continue;
+
+            if (vactx->initDecoder(avctx->width, avctx->height)) {
+                return fmt[i];
+            }
+        }
+    }
+#endif
+
+    reset_context(avctx);
+    return avcodec_default_get_format(avctx, fmt);
+}
+
+/// AVCodecContext.get_buffer() implementation
+int
+get_buffer(AVCodecContext* avctx, AVFrame* pic)
+{
+    VaapiContextFfmpeg* const vactx = get_vaapi_context(avctx);
+    if (!vactx) return avcodec_default_get_buffer(avctx, pic);
+
+#ifdef HAVE_VA_VA_H
+    if (!vactx->initDecoder(avctx->width, avctx->height)) return -1;
+
+    VaapiSurfaceFfmpeg * const surface = vactx->getSurface();
+    if (!surface) return -1;
+
+    vaapi_set_surface(pic, surface);
+
+    static unsigned int pic_num = 0;
+    pic->type = FF_BUFFER_TYPE_USER;
+    pic->age  = ++pic_num - surface->getPicNum();
+    surface->setPicNum(pic_num);
+    return 0;
+#endif
+    return -1;
+}
+
+/// AVCodecContext.reget_buffer() implementation
+int
+reget_buffer(AVCodecContext* avctx, AVFrame* pic)
+{
+    VaapiContextFfmpeg* const vactx = get_vaapi_context(avctx);
+
+    if (!vactx) return avcodec_default_reget_buffer(avctx, pic);
+
+    return get_buffer(avctx, pic);
+}
+
+/// AVCodecContext.release_buffer() implementation
+void
+release_buffer(AVCodecContext *avctx, AVFrame *pic)
+{
+    VaapiContextFfmpeg* const vactx = get_vaapi_context(avctx);
+    if (!vactx) {
+        avcodec_default_release_buffer(avctx, pic);
+        return;
+    }
+
+#ifdef HAVE_VA_VA_H
+    VaapiSurfaceFfmpeg* const surface = vaapi_get_surface(pic);
+    delete surface;
+
+    pic->data[0] = NULL;
+    pic->data[1] = NULL;
+    pic->data[2] = NULL;
+    pic->data[3] = NULL;
+#endif
+}
+
+}
 
 } // gnash.media.ffmpeg namespace 
 } // gnash.media namespace 
