@@ -1,6 +1,7 @@
 // NullSoundHandler - fake sound handler, for testing gnash
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,20 +21,15 @@
 #ifndef NULL_SOUND_HANDLER_H
 #define NULL_SOUND_HANDLER_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "sound_handler.h" // for inheritance
-#include "SoundInfo.h" 
 #include "dsodefs.h" // for DSOEXPORT
 
-#include <vector>
-#include <memory>
-#include <cassert>
-#include <cstring>
-
 namespace gnash {
+
+namespace media {
+    class MediaHandler;
+}
+
 namespace sound {
 
 /// Null sound_handler, for testing 
@@ -43,6 +39,8 @@ namespace sound {
 class DSOEXPORT NullSoundHandler : public sound_handler
 {
 public:
+    NullSoundHandler(media::MediaHandler* m) : sound_handler(m) {}
+
 };
 	
 } // gnash.sound namespace 
