@@ -1,6 +1,7 @@
 // sound_handler_ahi.cpp: Sound handling using standard AHI
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,16 +21,10 @@
 // Based on sound_handler_sdl.cpp by Thatcher Ulrich http://tulrich.com 2003
 // which has been donated to the Public Domain.
 
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "sound_handler_ahi.h"
 #include "SoundInfo.h"
 #include "EmbedSound.h"
 #include "AuxStream.h" // for use..
-#include "../../libcore/vm/VM.h"
 
 #include "log.h" // will import boost::format too
 #include "GnashException.h" // for SoundException
@@ -103,8 +98,10 @@ typedef struct{
 namespace gnash {
 namespace sound {
 
-AOS4_sound_handler::AOS4_sound_handler(const std::string& wavefile)
+AOS4_sound_handler::AOS4_sound_handler(media::MediaHandler* m,
+		const std::string& wavefile)
     :
+    sound_handler(m),
     _audioOpened(false),
 	_closing(false)
 {
@@ -115,7 +112,7 @@ AOS4_sound_handler::AOS4_sound_handler(const std::string& wavefile)
         file_stream.open(wavefile.c_str());
         if (file_stream.fail()) {
             std::cerr << "Unable to write file '" << wavefile << std::endl;
-            exit(1);
+            exit(EXIT_FAILURE);
         } else {
                 write_wave_header(file_stream);
                 std::cout << "# Created 44100 16Mhz stereo wave file:" << std::endl <<
@@ -125,8 +122,9 @@ AOS4_sound_handler::AOS4_sound_handler(const std::string& wavefile)
 
 }
 
-AOS4_sound_handler::AOS4_sound_handler()
+AOS4_sound_handler::AOS4_sound_handler(media::MediaHandler* m)
     :
+    sound_handler(m),
     _audioOpened(false),
 	_closing(false)
 {
@@ -315,17 +313,17 @@ AOS4_sound_handler::tell(int soundHandle)
 }
 
 sound_handler*
-create_sound_handler_aos4()
+create_sound_handler_aos4(media::MediaHandler* m)
 // Factory.
 {
-    return new AOS4_sound_handler;
+    return new AOS4_sound_handler(m);
 }
 
 sound_handler*
-create_sound_handler_aos4(const std::string& wave_file)
+create_sound_handler_aos4(media::MediaHandler* m, const std::string& wave_file)
 // Factory.
 {
-    return new AOS4_sound_handler(wave_file);
+    return new AOS4_sound_handler(m, wave_file);
 }
 
 void
@@ -499,7 +497,7 @@ AOS4_sound_handler::mix(boost::int16_t* outSamples, boost::int16_t* inSamples, u
 	    boost::uint8_t *out = reinterpret_cast<boost::uint8_t*>(outSamples);
     	boost::uint8_t* in = reinterpret_cast<boost::uint8_t*>(inSamples);
 
-	    MixAudio(out, in, nBytes, MIX_MAXVOLUME*volume);
+	    MixAudio(out, in, nBytes, static_cast<int>(MIX_MAXVOLUME*volume));
 	}
 }
 
