@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -33,6 +34,7 @@
 #include "SoundEnvelope.h" // for SoundEnvelopes typedef
 #include "AuxStream.h" // for aux_stramer_ptr typedef
 
+#include <string>
 #include <vector>
 #include <memory>
 #include <cassert>
@@ -387,7 +389,7 @@ public:
 
     /// Fetch mixed samples
     //
-    /// We run trough all the plugged InputStreams fetching decoded
+    /// We run through all the plugged InputStreams fetching decoded
     /// audio blocks and mixing them into the given output stream.
     ///
     /// @param to
@@ -410,19 +412,17 @@ public:
 protected:
 
 
-    sound_handler()
+    sound_handler(media::MediaHandler* m)
         :
         _soundsStarted(0),
         _soundsStopped(0),
         _paused(false),
         _muted(false),
-        _mediaHandler(media::MediaHandler::get()),
         _volume(100),
         _sounds(),
-        _inputStreams()
+        _inputStreams(),
+        _mediaHandler(m)
     {
-        // for now, we rely on this being always available
-        assert(_mediaHandler);
     }
 
     /// Plug an InputStream to the mixer
@@ -480,9 +480,6 @@ private:
     /// True if sound is muted
     bool _muted;
 
-    /// The registered MediaHandler at construction time
-    media::MediaHandler* _mediaHandler;
-
     /// Final output volume
     int _volume;
 
@@ -505,6 +502,8 @@ private:
     ///
     InputStreams _inputStreams;
 
+    media::MediaHandler* _mediaHandler;
+
     /// Unplug any completed input stream
     void unplugCompletedInputStreams();
 
@@ -573,16 +572,26 @@ private:
 
 #ifdef SOUND_SDL
 /// @throw a SoundException if fails to initialize audio card.
-DSOEXPORT sound_handler* create_sound_handler_sdl();
+DSOEXPORT sound_handler* create_sound_handler_sdl(media::MediaHandler* m);
 
 /// @throw a SoundException if fails to initialize audio card.
-DSOEXPORT sound_handler* create_sound_handler_sdl(const std::string& wave_file);
+DSOEXPORT sound_handler* create_sound_handler_sdl(media::MediaHandler* m,
+        const std::string& wave_file);
 #elif defined(SOUND_AHI)
 /// @throw a SoundException if fails to initialize audio card.
-DSOEXPORT sound_handler* create_sound_handler_aos4();
+DSOEXPORT sound_handler* create_sound_handler_aos4(media::MediaHandler* m);
 
 /// @throw a SoundException if fails to initialize audio card.
-DSOEXPORT sound_handler* create_sound_handler_aos4(const std::string& wave_file);
+DSOEXPORT sound_handler* create_sound_handler_aos4(media::MediaHandler* m,
+        const std::string& wave_file);
+
+#elif defined(SOUND_MKIT)
+/// @throw a SoundException if fails to create node.
+DSOEXPORT sound_handler* create_sound_handler_mkit(media::MediaHandler* m);
+
+/// @throw a SoundException if fails to create node.
+DSOEXPORT sound_handler* create_sound_handler_mkit(media::MediaHandler* m,
+        const std::string& wave_file);
 #endif
 
 } // gnash.sound namespace 
