@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ FuzzyPixel::operator==(const FuzzyPixel& o) const
 std::ostream&
 operator<< (std::ostream& o, const FuzzyPixel& p)
 {
-	return o << "FuzzyPixel(" << p._col.toString() << ") [tol:" << p._tol << "]";
+	return o << "FuzzyPixel(" << p._col << ") [tol:" << p._tol << "]";
 }
 
 } // namespace gnash
