@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -273,8 +273,18 @@ main(int /*argc*/, char** /*argv*/)
 
 	}
 
+    tester.scrollMouse(-1);
+    check_equals(text->get_text_value(), "onMouseWheel: -1, , 2");
+    
+    tester.scrollMouse(1);
+    check_equals(text->get_text_value(), "onMouseWheel: 1, , 2");
+
+    tester.movePointerTo(100, 100);
+    tester.scrollMouse(1);
+    check_equals(text->get_text_value(), "onMouseWheel: 1, _level0.textfield2, 2");
+
 	// last advance should not restart the loop (it's in STOP mode)
-        check_equals(root->getPlayState(), MovieClip::PLAYSTATE_STOP);
+    check_equals(root->getPlayState(), MovieClip::PLAYSTATE_STOP);
 	check_equals(root->get_current_frame(), 4);
 
 }
