@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 #define INPUT_FILENAME "Video-EmbedSquareTest.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
-#include "character.h"
+#include "MovieClip.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 
@@ -42,7 +42,7 @@ main(int /*argc*/, char** /*argv*/)
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity(1);
 
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
 	if ( ! tester.canTestRendering() )
@@ -59,7 +59,7 @@ main(int /*argc*/, char** /*argv*/)
 	int scale_y = 1;
 
 	size_t framecount = root->get_frame_count();
-	check_equals(framecount, 11);
+	check_equals(framecount, 12);
 	
 	// Just loop twice, so to catch crashes...
 	for (int j = 0; j < 2; ++j) {
@@ -89,7 +89,7 @@ main(int /*argc*/, char** /*argv*/)
 			// Frame X
 			tester.advance();
 			int framenum = root->get_current_frame();
-			cout << "---- Pixel checking in frame " << framenum+1 << " play state " << root->get_play_state() << endl;
+			cout << "---- Pixel checking in frame " << framenum+1 << " play state " << root->getPlayState() << endl;
 			
 			int i = (framenum-1)*10;
 
@@ -108,10 +108,10 @@ main(int /*argc*/, char** /*argv*/)
 				check_pixel((45 + i)*scale_x, 5, 2, yellow, 5);
 			}
 
-			if ( framenum == framecount-1 )
+			if ( framenum == framecount-2 )
 			{
 				// check we're playing, or we'll never get to next loop...
-				check_equals(root->get_play_state(), sprite_instance::PLAY);
+				check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 				break;
 			}
 
@@ -119,6 +119,18 @@ main(int /*argc*/, char** /*argv*/)
 		}
 
 		tester.advance();
+
+		// Check the color in (5,5) - should be yellow. Well, anything
+		// but white or transparent.
+		check_pixel(5, 5, 2, yellow, 5);
+		check_equals(root->getPlayState(), MovieClip::PLAYSTATE_STOP);
+		tester.click();
+
+		// Sanity check
+		check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
+
+		tester.advance();
+
 	}
 
 }
