@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ using namespace gnash;
 using namespace std;
 
 std::auto_ptr<MovieTester> tester;
-boost::intrusive_ptr<MovieClip> root;
+MovieClip* root;
 
 MovieClip*
 getCoverArt()
@@ -94,7 +94,7 @@ main(int /*argc*/, char** /*argv*/)
 
 
 	root = tester->getRootMovie();
-	assert(root.get());
+	assert(root);
 
 	check_equals(root->get_frame_count(), 2);
 	check_equals(root->get_current_frame(), 0);
@@ -116,12 +116,18 @@ main(int /*argc*/, char** /*argv*/)
 	tester->movePointerTo(80, 80);
 	check(tester->isMouseOverMouseEntity());
 	tester->pressMouseButton();
-	tester->advance(); // loads (should) happen on next advance
-	usleep(500); // give it some time... TODO: drop this test and use a self-containment instead
-	coverartch = const_cast<DisplayObject*>(tester->findDisplayItemByName(*root, "coverart"));
-	check(coverart != coverartch->to_movie());
+
+    // Wait for the movie to load
+    // TODO: drop this test and use a self-containment instead
+    do {
+	    usleep(500); // give it some time...
+	    tester->advance(); // loads (should) happen on next advance
+	    coverartch = const_cast<DisplayObject*>(tester->findDisplayItemByName(*root, "coverart"));
+    } while (coverartch->to_movie() == coverart);
+
 	coverart = coverartch->to_movie();
 	check_equals(coverart->get_root()->url(), lynchURL.str());
+
 	tester->depressMouseButton();
 
 	// Check scribbling on the lynch
@@ -135,9 +141,15 @@ main(int /*argc*/, char** /*argv*/)
 	tester->movePointerTo(280, 80);
 	check(tester->isMouseOverMouseEntity());
 	tester->click();
-	tester->advance(); // loads (should) happen on next advance
-	usleep(500); // give it some time... TODO: drop this test and use a self-containment instead
-	coverartch = const_cast<DisplayObject*>(tester->findDisplayItemByName(*root, "coverart"));
+
+    // Wait for the movie to load
+    // TODO: drop this test and use a self-containment instead
+    do {
+	    usleep(500); // give it some time... 
+	    tester->advance(); // loads (should) happen on next advance
+	    coverartch = const_cast<DisplayObject*>(tester->findDisplayItemByName(*root, "coverart"));
+    } while (coverartch->to_movie() == coverart);
+
 	coverart = coverartch->to_movie();
 	check_equals(coverart->get_root()->url(), greenURL.str());
 	// TODO: find a way to test if the jpeg is really displayed
@@ -154,9 +166,15 @@ main(int /*argc*/, char** /*argv*/)
 	tester->movePointerTo(480, 80);
 	check(tester->isMouseOverMouseEntity());
 	tester->click();
-	tester->advance(); // loads (should) happen on next advance
-	usleep(500); // give it some time... TODO: drop this test and use a self-containment instead
-	coverartch = const_cast<DisplayObject*>(tester->findDisplayItemByName(*root, "coverart"));
+
+    // Wait for the movie to load
+    // TODO: drop this test and use a self-containment instead
+    do {
+	    usleep(500); // give it some time... 
+	    tester->advance(); // loads (should) happen on next advance
+	    coverartch = const_cast<DisplayObject*>(tester->findDisplayItemByName(*root, "coverart"));
+    } while (coverartch->to_movie() == coverart);
+
 	coverart = coverartch->to_movie();
 	check_equals(coverart->get_root()->url(), offspringURL.str());
 
@@ -173,10 +191,10 @@ main(int /*argc*/, char** /*argv*/)
 	tester->releaseKey(key::SHIFT);
 
 	// Consistency checking
-	string_table& st = getStringTable(*root);
+	string_table& st = getStringTable(*getObject(root));
 	as_value eot;
 	// It's an swf6, so lowercase 'END_OF_TEST'
-	bool endOfTestFound = root->get_member(st.find("end_of_test"), &eot);
+	bool endOfTestFound = getObject(root)->get_member(st.find("end_of_test"), &eot);
 	check(endOfTestFound);
 	if ( endOfTestFound )
 	{
