@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,14 +20,13 @@
 #include "gnashconfig.h"
 #endif
 
-#include <dirent.h>
+#include "GnashFileUtilities.h"
 #include <iostream>
-#include <stdarg.h>
+#include <cstdarg>
 #include <cstring>
-#include <sys/stat.h>
 
 #ifdef ENABLE_NLS
-# include <locale>
+# include <clocale>
 #endif
 
 extern "C"{
@@ -52,8 +52,30 @@ gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 gnash::RcInitFile& rcfile = gnash::RcInitFile::getDefaultInstance();
 }
 
+#ifdef BOOST_NO_EXCEPTIONS
+namespace boost
+{
+
+	void throw_exception(std::exception const & e)
+	{
+		std::abort();
+	}
+}
+#endif
+
 const char *SOLDUMPER_VERSION = "0.5";
-static void usage ();
+
+/// \brief  Display the command line arguments
+static void
+usage(ostream &o)
+{
+    o << _("This program dumps the internal data of a .sol file")
+         << endl;
+    o << _("Usage: soldumper [h] filename") << endl;
+    o << _("-h\tHelp") << endl;
+    o << _("-f\tForce local directory access") << endl;
+    o << _("-l\tList all .sol files in default dir") << endl;
+}
 
 int
 main(int argc, char *argv[])
@@ -76,13 +98,13 @@ main(int argc, char *argv[])
     // scan for the two main standard GNU options
     for (c = 0; c < argc; c++) {
       if (strcmp("--help", argv[c]) == 0) {
-        usage();
-        exit(0);
+        usage(cout);
+        exit(EXIT_SUCCESS);
       }
       if (strcmp("--version", argv[c]) == 0) {
         printf (_("Gnash soldumper version: %s, Gnash version: %s\n"),
 		   SOLDUMPER_VERSION, VERSION);
-        exit(0);
+        exit(EXIT_SUCCESS);
       }
     }
  
@@ -90,7 +112,8 @@ main(int argc, char *argv[])
     while ((c = getopt (argc, argv, "hvfl")) != -1) {
         switch (c) {
           case 'h':
-            usage ();
+            usage(cout);
+	    exit(EXIT_SUCCESS);
             break;
             
 	  case 'v':
@@ -109,7 +132,7 @@ main(int argc, char *argv[])
 	      break;
 
           default:
-            usage ();
+            usage(cerr);
             break;
         }
     }
@@ -118,8 +141,8 @@ main(int argc, char *argv[])
     // If no command line arguments have been supplied, do nothing but
     // print the  usage message.
     if (argc < 2) {
-        usage();
-        exit(0);
+        usage(cerr);
+        exit(EXIT_FAILURE);
     }
 
     // get the file name from the command line
@@ -160,7 +183,7 @@ main(int argc, char *argv[])
                 }
             }
         }
-        exit(0);
+        exit(EXIT_SUCCESS);
     }
     
     string newspec;
@@ -172,7 +195,7 @@ main(int argc, char *argv[])
         newspec += filespec;
     }
     
-    amf::SOL sol;
+    cygnal::SOL sol;
     
     if (sol.readFile(newspec)) {
         cout << "SOL file \"" << newspec << "\" read in" << endl;
@@ -183,19 +206,6 @@ main(int argc, char *argv[])
     sol.dump();
 }
 
-/// \brief  Display the command line arguments
-static void
-usage ()
-{
-    cerr << _("This program dumps the internal data of a .sol file")
-         << endl;
-    cerr << _("Usage: soldumper [h] filename") << endl;
-    cerr << _("-h\tHelp") << endl;
-    cerr << _("-f\tForce local directory access") << endl;
-    cerr << _("-l\tList all .sol files in default dir") << endl;
-    exit (-1);
-}
-
 // Local Variables:
 // mode: C++
 // indent-tabs-mode: t
